@@ -951,16 +951,24 @@ brushboxResized(Widget w, BrushboxInfo * l, XConfigureEvent * event, Boolean * f
                      XtNheight, &height, NULL);
     if (width<10) width = 10;
     if (height<35) height = 35;
+#ifdef XAW3DXFT
+    XtResizeWidget(l->box, width-6, height-Global.btn_height-13, 1);
+#else		   
     XtResizeWidget(l->box, width-6, height-32,
-#ifdef XAW3D		   
+#ifdef XAW3D 
 		   1
 #else
 		   0
 #endif		   
 		   );
+#endif    
     XtUnmanageChild(l->close);
     XMapWindow(XtDisplay(l->close), XtWindow(l->close));
+#ifdef XAW3DXFT
+    XtMoveWidget(l->close, 4, height-Global.btn_height-3);		   
+#else
     XtMoveWidget(l->close, 4, height-25);
+#endif		   
 }
 
 static void
@@ -1055,6 +1063,7 @@ createBrushDialog(Widget w)
 				    commandWidgetClass, info->form,
 				    XtNfromVert, info->box,
 				    XtNtop, XtChainBottom,
+				    XtNheight, Global.btn_height,	      
 				    NULL);
 
         XtAddCallback(info->close, XtNcallback, (XtCallbackProc) closePopup,
