@@ -53,9 +53,9 @@
 #define M_PI		3.14159265358979323846
 #endif
 
-#define PIXMAP_WIDTH  120
-#define PIXMAP_HEIGHT 120
-#define RADIUS	      50
+#define PIXMAP_WIDTH  (5*Global.btn_height + 10)
+#define PIXMAP_HEIGHT (5*Global.btn_height + 10)
+#define RADIUS	      (5*Global.btn_height/2 - 2)
 
 #define RECOMPUTE(l)					\
 	if (l->isRGB)					\
@@ -339,9 +339,9 @@ motion(Widget w, LocalInfo * l, XEvent * event, Boolean * flg)
 
     if (dx == 0 && dy == 0)
 	l->h = 0;
-    else if ((l->h = atan2((double) dx, (double) dy)) < 0.0)
+    else if ((l->h = atan2((float) dx, (float) dy)) < 0.0)
 	l->h += 2 * M_PI;
-    l->s = sqrt((double) (dx2 + dy2)) / RADIUS;
+    l->s = sqrt((float) (dx2 + dy2)) / RADIUS;
     l->h *= 360.0 / (2 * M_PI);
 
     if (l->s > 1.0) {
@@ -548,9 +548,9 @@ drawInPixmap(Widget w, Palette * map, Colormap cmap,
 		}
 		if (dx == 0 && dy == 0)
 		    h = 0;
-		else if ((h = atan2((double) dx, (double) dy)) < 0.0)
+		else if ((h = atan2((float) dx, (float) dy)) < 0.0)
 		    h += 2 * M_PI;
-		s = sqrt((double) (dx2 + dy2)) / RADIUS;
+		s = sqrt((float) (dx2 + dy2)) / RADIUS;
 
 		h *= 360.0 / (2 * M_PI);
 
@@ -635,13 +635,15 @@ Ctrl<Key>J: color-text-ok()\n";
     label = XtVaCreateManagedWidget("valueLabel", labelWidgetClass, subform,
 				    XtNlabel, msg,
 				    XtNborderWidth, 0,
+				    XtNwidth, 4*Global.btn_height,
 				    XtNright, XtChainLeft,
 				    XtNleft, XtChainLeft,
 				    NULL);
     *bar = XtVaCreateManagedWidget("valueBar", scrollbarWidgetClass, subform,
 				   XtNorientation, XtorientHorizontal,
-				   XtNwidth, 200,
+				   XtNwidth, 9*Global.btn_height+2,
 				   XtNfromHoriz, label,
+				   XtNvertDistance, (Global.btn_height/2 - 6),
 				   XtNleft, XtChainLeft,
 				   NULL);
     *text = XtVaCreateManagedWidget("valueText", asciiTextWidgetClass, subform,
@@ -650,7 +652,7 @@ Ctrl<Key>J: color-text-ok()\n";
 				    XtNwrap, XawtextWrapNever,
 				    XtNresize, XawtextResizeWidth,
 				    XtNtranslations, trans,
-				    XtNwidth, 50,
+				    XtNwidth, 2*Global.btn_height+6,
 				    XtNlength, 5,
 				    NULL);
     XtAddEventHandler(*text, 
@@ -730,14 +732,17 @@ colorPicker(Widget parent, Colormap cmap, Pixel * pixval)
     button = XtVaCreateManagedWidget("match", commandWidgetClass, form,
 				     XtNfromVert, l->view,
 				     XtNfromHoriz, picker,
+				     XtNheight, Global.btn_height,
 				     NULL);
     l->select = XtVaCreateWidget("select", commandWidgetClass, form,
 				     XtNfromVert, button,
 				     XtNfromHoriz, picker,
+				     XtNheight, Global.btn_height,
 				     NULL);
     l->asbg = XtVaCreateWidget("asbg", commandWidgetClass, form,
 				     XtNfromVert, l->select,
 				     XtNfromHoriz, picker,
+				     XtNheight, Global.btn_height,	       
 				     NULL);
     XtAddCallback(button, XtNcallback, (XtCallbackProc)grabCB, (XtPointer) l);
 
