@@ -15,20 +15,28 @@
 
 /* $Id: fileBrowser.c,v 1.18 2005/03/20 20:15:32 demailly Exp $ */
 
+#include <ctype.h>
 #include <X11/StringDefs.h>
 #include <X11/Intrinsic.h>
+#include <X11/IntrinsicP.h>
 #include <X11/Shell.h>
+#include <X11/Xlib.h>
+#include <X11/Xft/Xft.h>
 
 #include "xaw_incdir/Dialog.h"
 #include "xaw_incdir/Command.h"
 #include "xaw_incdir/Toggle.h"
 #include "xaw_incdir/Form.h"
 #include "xaw_incdir/Label.h"
-#include "xaw_incdir/List.h"
+#include "xaw_incdir/ListP.h"
 #include "xaw_incdir/AsciiText.h"
 #include "xaw_incdir/Text.h"
 #include "xaw_incdir/Viewport.h"
 #include "xaw_incdir/Scrollbar.h"
+#ifdef XAW3DXFT
+#include "xaw_incdir/Xaw3dXft.h"
+extern Xaw3dXftData *xaw3dxft_data;
+#endif
 
 #include "Paint.h"
 #include "PaintP.h"
@@ -47,7 +55,7 @@
 #endif
 #include <pwd.h>
 
-#include "rw/rwTable.h"
+#include "xpaintrw/rwTable.h"
 #include "graphic.h"
 #include "protocol.h"
 
@@ -78,6 +86,8 @@ typedef struct {
 } CScriptInfo;
 
 static void *lastId = NULL;
+static int text_entry_height = 22;
+static int char_entry_width = 18;
 
 extern int file_isSpecialImage;
 extern int file_transparent;
@@ -85,6 +95,7 @@ extern int file_numpages;
 extern int file_force;
 extern int file_bbox;
 extern int file_specified_zoom;
+extern int file_JPEG_quality;
 
 void *
 GetFileNameGetLastId()
@@ -279,13 +290,15 @@ typedef struct arg_s {
     char dirname[MAX_PATH];
     Widget shell, list, browser, parent, 
            form, title_w, name, vport, 
-           home, root, dot, hidden, ongoing, cwd_w, info, ok, cancel,
+           home, root, dot, hidden, ongoing, 
+           quality, quality_w, quality_l, quality_ll, quality_r, quality_rr,
+           cwd_w, info, ok, cancel,
            refresh, delete, edit, create,
            zoomlabel, zoom, dpilabel, dpi, 
            bboxlabel, bboxleft, bbox, bboxright,
            pagelabel, page, pagetotal, pageleft, pageLeft, pageright, pageRight;
     Widget format[NUM_FORMATS+2];
-    int first, numformat, oldwidth;
+    int first, numformat, oldwidth, dobuilt;
     /*
     **	Use for caching, list purposes
      */
@@ -300,9 +313,34 @@ static void
 fileTypeCallback(Widget w, XtPointer argArg, XtPointer junk)
 {
     arg_t *arg = (arg_t *) argArg;
-    String nm = XtName(w);
+    String nm = XtName(w), str, fmt, newname;
+    char value[6];
+    int i, n;
 
-    arg->type = RWtableGetEntry(nm);
+    arg->type = RWtableGetEntry(nm);    
+    fmt = RWtableGetId(arg->type);
+
+    if (!strncmp(fmt, "Auto", 4)) return;
+
+    strncpy(value, fmt, 5);
+    str = strchr(value, '_');
+    if (str) *str = '\0';
+    n = strlen(value);
+    if (n >= 4) value[--n] = '\0';
+    for (i=0; i<n; i++) value[i] = tolower(value[i]);
+    if(!strcmp(value,"jpe")) value[2] = 'g';
+
+    XtVaGetValues(arg->name, XtNstring, &str, NULL);
+    newname = (char *) malloc(strlen(str) + 5);
+    strcpy(newname, str);
+    n = strlen(newname);
+    fmt = strrchr(newname, '.');
+    if (fmt && fmt > newname + n - 5) n = fmt - newname;
+    newname[n] = '.';
+    newname[n+1] = '\0';
+    strcat(newname, value);
+    XtVaSetValues(arg->name, XtNstring, newname, NULL);
+    free(newname);
 }
 
 static void 
@@ -325,9 +363,9 @@ emptyList(arg_t * arg)
 static void
 browserResized(Widget w, arg_t * l, XConfigureEvent * event, Boolean * flag)
 {
-    Dimension width, height, width2, width3;
+    Dimension width, height, width2, width3, wmax;
     Position x, y;
-    int i, j, width1, height1, dw = 0, im;
+    int i, j, width1, height1, dw = 0, a, b, c, im;
     Widget *widget;
 
     XtVaGetValues(XtParent(l->parent), 
@@ -339,16 +377,22 @@ browserResized(Widget w, arg_t * l, XConfigureEvent * event, Boolean * flag)
     i = width2;
     XtVaGetValues(l->cancel, XtNwidth, &width2, NULL);
     i += width2 + 35; 
+    wmax = 0;
     for (j=0; j<l->numformat; j++) {
         XtVaGetValues(l->format[j], XtNwidth, &width3, NULL);
+        if (width3>wmax) wmax = width3;
         width3 += 25;
         if (width3>i) i = width3;
     }
+    for (j=0; j<l->numformat; j++)
+        XtVaSetValues(l->format[j], XtNwidth, wmax, NULL);
 
+    a = Global.btn_height + 2;
+    b = 4 * Global.btn_height;
     width1 = (int)width - i; if (width1<60) width1 = 60;
-    height1 = (int)height - 120;
-    if (l->zoom) height1 -= 100;
-    if (l->home) height1 -= 27;
+    height1 = (int)height - b - 12 - text_entry_height ;
+    if (l->zoom) height1 -= b + 12;
+    if (l->home) height1 -= Global.btn_height + 3;
     if (height1<60) height1 = 60;
     
     XtResizeWidget(l->browser, width, height, 0);
@@ -358,7 +402,7 @@ browserResized(Widget w, arg_t * l, XConfigureEvent * event, Boolean * flag)
         width, height);
     XtVaSetValues(l->browser, XtNwidth, width, XtNheight, height, NULL);
     XtVaSetValues(l->parent, XtNwidth, width, XtNheight, height, NULL);   
-    XtResizeWidget(l->name, width1-2, 20, 1);
+    XtResizeWidget(l->name, width1-2, text_entry_height, 1);
     XtResizeWidget(l->form, width1 + 10, height, 0);
     if (Global.numfiles) {
         XtVaGetValues(l->list, XtNlongest, &i, NULL);
@@ -380,29 +424,40 @@ browserResized(Widget w, arg_t * l, XConfigureEvent * event, Boolean * flag)
     }
     XtVaGetValues(l->cwd_w, XtNx, &x, NULL);
     if (l->home) {
-        XtMoveWidget(l->home, 5, height-84);
-        XtMoveWidget(l->root, 29, height-84);
-        XtMoveWidget(l->dot, 53, height-84);
-        XtMoveWidget(l->hidden, 77, height-84);
-        if (l->ongoing) {
-            XtVaGetValues(l->hidden, XtNwidth, &width1, NULL);
-            XtMoveWidget(l->ongoing, 80+width1, height-84);
+        XtMoveWidget(l->home, 5, height-b);
+        XtMoveWidget(l->root, 5+a, height-b);
+        XtMoveWidget(l->dot, 5+2*a, height-b);
+        XtMoveWidget(l->hidden, 5+3*a, height-b);
+        XtVaGetValues(l->hidden, XtNwidth, &width1, NULL);
+        if (l->ongoing)
+            XtMoveWidget(l->ongoing, 8+3*a+width1, height-b);
+        width2 = 0;
+        if (l->quality_w) {
+	    c = Global.btn_height/2 - 1;
+            XtMoveWidget(l->quality_w, 140+width1, height-b);
+            XtVaGetValues(l->quality_w, XtNwidth, &width2, NULL);       
+            XtMoveWidget(l->quality_ll, 140+c+width1+width2, height-b-1);
+            XtMoveWidget(l->quality_l,  140+3*c+width1+width2, height-b-1);
+            XtMoveWidget(l->quality, 140+5*c+width1+width2, height-b-1);
+            XtMoveWidget(l->quality_r, 193+5*c+width1+width2, height-b-1);
+            XtMoveWidget(l->quality_rr, 193+7*c+width1+width2, height-b-1);
 	}
-        XtMoveWidget(l->cwd_w, x, height-52);
-        XtMoveWidget(l->info, x, height-30);
+        XtMoveWidget(l->cwd_w, x, height-2*Global.btn_height-14);
+        XtMoveWidget(l->info, x, height-Global.btn_height-8);
         return;
     }
 
-    XtMoveWidget(l->cwd_w, x, height-158);
-    XtMoveWidget(l->info, x, height-134);
+    XtMoveWidget(l->cwd_w, x, height-6*(Global.btn_height+4));
+    XtMoveWidget(l->info, x, height-5*(Global.btn_height+4));
 
     im = &l->pageRight - &l->refresh;
     widget = &l->refresh;
+    b = Global.btn_height+12;
     for (i=0; i<=im; i++) {
         XtVaGetValues(widget[i], XtNx, &x, NULL);
         if (i==0) j = 5-x;
         XtMoveWidget(widget[i], x+j, height - 
-                     ((i<=3)? 99: ((i<=11)?67:35)));
+                     ((i<=3)? 3*b: ((i<=11)?(2*b):b)));
     }
 }
 
@@ -635,7 +690,7 @@ setCWD(arg_t * arg, char *dir)
 
   fill:
 
-    XawListChange(arg->list, list, i, 0, True);
+    XawListChange(arg->list, list, i, 0, False);
     XtVaSetValues(arg->info, XtNlabel, infoStr, NULL);
 
     if (arg->browserType != BROWSER_LOADED || arg->isToRefresh) {
@@ -643,7 +698,7 @@ setCWD(arg_t * arg, char *dir)
         if ((sb = XtNameToWidget(arg->vport, "vertical")) != None) {
 	    float top = 0.0;
 	    XawScrollbarSetThumb(sb, top, -1.0);
-	    XtCallCallbacks(sb, XtNjumpProc, (XtPointer) & top);
+	    XtCallCallbacks(sb, XtNjumpProc, (XtPointer) &top);
 	}
         StateSetBusyWatch(False);
     }
@@ -712,7 +767,7 @@ okCallback(Widget w, XtPointer argArg, XtPointer junk)
 	    if (arg->browserType != BROWSER_LOADED || arg->isToRefresh) 
                 setCWD(arg, file);
             if (arg->browserType != BROWSER_LOADED)
-                browserResized(arg->parent, arg, NULL, False);
+	        browserResized(arg->parent, arg, NULL, False);
 	}
 	*cp = '/';
     }
@@ -739,6 +794,7 @@ okCallback(Widget w, XtPointer argArg, XtPointer junk)
         if (paint) {
 	    XtVaSetValues(paint, XtNzoom, zoom, NULL);
 	    loadPrescribedFile(paint, file);
+            StoreName(GetShell(paint), file);
             setZoomButtonLabel(paint, zoom);
             XtVaSetValues(arg->zoom, XtNstring, ZoomToStr(zoom), NULL);
             if (file_numpages != np) {
@@ -759,6 +815,7 @@ okCallback(Widget w, XtPointer argArg, XtPointer junk)
 	            XtVaGetValues(paint, XtNzoom, &zoom, NULL);
                 Global.default_zoom = zoom;
 	        loadPrescribedFile(paint, file);
+                StoreName(GetShell(paint), file);
                 AddFileToGlobalList(file);
                 setZoomButtonLabel(paint, zoom);
                 Global.default_zoom = zoom_prev;
@@ -895,6 +952,25 @@ dotCallback(Widget w, XtPointer argArg, XtPointer itemArg)
     browserResized(arg->parent, arg, NULL, False); 
 }
 
+static void 
+qualityCallback(Widget w, XtPointer argArg, XtPointer itemArg)
+{
+    arg_t *arg = (arg_t *) argArg;
+    static char value[8];
+    if (w == arg->quality_ll) file_JPEG_quality -=10;
+    if (w == arg->quality_rr) file_JPEG_quality +=10;
+    if (w == arg->quality_l) --file_JPEG_quality;
+    if (w == arg->quality_r) ++file_JPEG_quality;
+    if (file_JPEG_quality < 50) file_JPEG_quality = 50;
+    if (file_JPEG_quality > 100) file_JPEG_quality = 100;
+    sprintf(value, "%d", file_JPEG_quality);
+    XtVaSetValues(arg->quality, XtNstring, &value, NULL);
+    XtVaSetValues(arg->quality, XtNdisplayCaret, False, NULL);
+    XawTextSetInsertionPoint(arg->quality, strlen(value));
+    XFlush(XtDisplay(arg->quality));
+    XtVaSetValues(arg->quality, XtNdisplayCaret, True, NULL);
+}
+
 static void 
 refreshCallback(Widget w, XtPointer argArg, XtPointer itemArg)
 {
@@ -1004,7 +1080,7 @@ createLXPCallback(Widget w, XtPointer argArg, XtPointer fileArg)
 
     sprintf(buf, "cd %s ; tar cvfz %s .\n", dir, file);
     free(dir);
-    (void) system(buf);
+    if (system(buf));
     AddFileToGlobalList(file);
     setCWD(arg, NULL);
     browserResized(arg->parent, arg, NULL, False);
@@ -1035,13 +1111,28 @@ listCallback(Widget bar, XtPointer argArg, XtPointer itemArg)
     XawListReturnStruct *item = (XawListReturnStruct *) itemArg;
     arg_t *arg = (arg_t *) argArg;
     String str, label;
+    int n;
+
+    XtVaGetValues(arg->name, XtNstring, &str, NULL);
 
     if (strncmp(item->string, "../", 3) == 0)
 	label = "../";
     else
 	label = item->string;
 
-    XtVaGetValues(arg->name, XtNstring, &str, NULL);
+    n = strlen(label) - 1;
+
+    if (label[n] == '/') {
+        /* if user clicked on a directory, go there immediately */
+        char *ptr = strdup(str);
+        setCWD(arg, doDirname(arg, label));
+        browserResized(arg->parent, arg, NULL, False); 
+        n = strlen(ptr) - 1;
+        if (ptr[n] != '/')
+	    XtVaSetValues(arg->name, XtNstring, ptr, NULL);
+        free(ptr);
+        return;
+    }    
 
     if (strcmp(str, label) == 0) {
         file_force = 1;
@@ -1058,18 +1149,133 @@ listCallback(Widget bar, XtPointer argArg, XtPointer itemArg)
 	XtVaSetValues(arg->name, XtNstring, label, NULL);
     }
 }
-   
+
+void
+buttonPressed(Widget w, XtPointer arg, XConfigureEvent * event, 
+              Boolean * flg)
+{
+    char *v;
+    static char value[8];
+    int n;
+    if (event->type == ButtonPress || event->type == ButtonRelease) {
+        XtSetKeyboardFocus((Widget)arg, None);
+        XtSetKeyboardFocus((Widget)arg, w);
+        return;
+    }
+    XtVaGetValues(w, XtNstring, &v, NULL);
+    if (v) strncpy(value, v, 6); else *value = '\0';
+    value[7] = '\0';
+    if ((v=strchr(value, '\n'))) {
+        *v = '\0';
+        XtVaSetValues(w, XtNstring, &value, NULL);
+        XawTextSetInsertionPoint(w, strlen(value));
+    }
+    n = atoi(value);
+    if (n >= 50 && n<=100)
+        file_JPEG_quality = n;
+}
+
+#ifdef XAW3DXFT
+static void
+ColorSwitch(Widget w, int n, int x, int y, Pixel *p) 
+{
+    static Pixel pix1 = -1, pix2, pix3;
+    XColor xc;
+    char *str, *ptr;
+    char *list_fmt[] = { "png", "gif", "jpg", "jpeg", "jp2", "pgf", 
+                         "tif", "tiff", "tga", "bmp", "ico", "sgi",
+                         "pnm", "ppm", "pbm", "xpm", "xwd", "lxp",
+                         "ps", "pdf", "svg", "dvi", "tex", NULL };
+    char **lfmt;
+
+    str = ((ListWidget) w)->list.list[n];
+    if (!str) { *p = -1; return; }
+
+    if (pix1 == -1) {
+        if (Global.depth > 8 &&
+            XAllocNamedColor(XtDisplayOfObject(w), 
+                             DefaultColormapOfScreen(XtScreenOfObject(w)), 
+			     "#0022CC", &xc, &xc) == 0)
+                pix1 = BlackPixelOfScreen(XtScreenOfObject(w));
+            else
+                pix1 =  xc.pixel;
+        if (Global.depth > 8 &&
+            XAllocNamedColor(XtDisplayOfObject(w), 
+                             DefaultColormapOfScreen(XtScreenOfObject(w)), 
+	 		 "#CC2200", &xc, &xc) == 0)
+                pix2 = BlackPixelOfScreen(XtScreenOfObject(w));
+            else
+                pix2 =  xc.pixel;
+        if (Global.depth > 8 &&
+            XAllocNamedColor(XtDisplayOfObject(w), 
+                             DefaultColormapOfScreen(XtScreenOfObject(w)), 
+	 		 "#C040C0", &xc, &xc) == 0)
+                pix3 = BlackPixelOfScreen(XtScreenOfObject(w));
+            else
+                pix3 =  xc.pixel;
+    }
+
+
+    if (strrchr(str, '/')) {
+#if 0
+        GC gc;
+        gc = XCreateGC(XtDisplayOfObject(w), XtWindowOfObject(w), 0, NULL);
+        XSetForeground(XtDisplayOfObject(w), gc, pix2);
+        XFillRectangle(XtDisplayOfObject(w), XtWindowOfObject(w), gc,
+                       0, y-20, 2, 23);
+#endif
+        if (str[0] == '.' && str[1] == '.' && str[2] == '/')
+            *p = pix2;
+        else
+            *p = pix1;
+        return;
+    } else {
+        /* maybe an image format ? */
+#if 0   /* use testMagic() from rwTable.c -- possibly slow!! */
+        if (testMagic(str)) {
+            *p = pix3;
+            return;
+	} else {
+            *p = -1;
+            return;
+	}
+#endif
+	/* just guess from suffix */
+        if (ptr = strrchr(str, '.')) {
+	    ++ptr;
+            lfmt = list_fmt;
+	    while (*lfmt) 
+	        if (!strcasecmp(ptr, *lfmt++)) goto is_image;
+	}
+        *p = -1;
+        return;
+    }
+    is_image:
+    *p = pix3;
+}
+#endif
+
 static Widget
 buildBrowser(Widget parent, char *title, arg_t * arg, 
              int width, int height)
 {
     Widget form;
     Widget title_w, name, list, vport, cwd, info;
+    char value[8];
+    int a = Global.btn_height - 2;
 
     PopdownMenusGlobal();
 
     arg->parent = parent;
-
+#ifdef XAW3DXFT
+    if (xaw3dxft_data && xaw3dxft_data->default_font)
+        text_entry_height = (int)(xaw3dxft_data->default_font->height+0.5) + 6;
+    else
+        text_entry_height = Global.btn_height;
+    if (text_entry_height < Global.btn_height - 2) 
+       text_entry_height = Global.btn_height - 2;
+    char_entry_width = (18 * text_entry_height) / 22;
+#endif
     if (!trans) {
 	trans = XtParseTranslationTable("#override\n\
 						 <Key>Return: no-op()\n\
@@ -1113,6 +1319,7 @@ buildBrowser(Widget parent, char *title, arg_t * arg,
 				   XtNtranslations, trans,
 				   XtNbottom, XtChainTop,
 				   XtNtop, XtChainTop,
+                                   XtNheight, text_entry_height,
 				   XtNwidth, width,
 				   NULL);
     arg->name = name;
@@ -1134,12 +1341,16 @@ buildBrowser(Widget parent, char *title, arg_t * arg,
 
     list = XtVaCreateManagedWidget("files",
 				   listWidgetClass, vport,
+				   XtNrowSpacing, ROW_SPACING,
 				   XtNborderWidth, 1,
 				   XtNverticalList, True,
 				   XtNforceBars, False,
 				   XtNforceColumns, True,
 				   XtNdefaultColumns, 1,
 				   XtNnumberStrings, 0,
+#ifdef XAW3DXFT
+				   XtNcolorSwitch, (arg->browserType==BROWSER_LOADED)?NULL:ColorSwitch,
+#endif
 				   NULL);
     arg->list = list;
 
@@ -1150,8 +1361,8 @@ buildBrowser(Widget parent, char *title, arg_t * arg,
                                   XtNfromVert, vport,
                                   XtNhorizDistance, 4,
                                   XtNvertDistance, 0,
-                                  XtNwidth, 20,
-                                  XtNheight, 20,
+                                  XtNwidth, a,
+                                  XtNheight, text_entry_height,
 				  NULL);
         arg->root = XtVaCreateManagedWidget("root",
 				  commandWidgetClass, form,
@@ -1160,8 +1371,8 @@ buildBrowser(Widget parent, char *title, arg_t * arg,
                                   XtNfromHoriz, arg->home,
                                   XtNhorizDistance, 3,
                                   XtNvertDistance, 0,
-                                  XtNwidth, 20,
-                                  XtNheight, 20,
+                                  XtNwidth, a,
+                                  XtNheight, text_entry_height,
 				  NULL);
         arg->dot = XtVaCreateManagedWidget("dot",
 				  commandWidgetClass, form,
@@ -1170,8 +1381,8 @@ buildBrowser(Widget parent, char *title, arg_t * arg,
                                   XtNfromHoriz, arg->root,
                                   XtNhorizDistance, 3,
                                   XtNvertDistance, 0,
-                                  XtNwidth, 20,
-                                  XtNheight, 20,
+                                  XtNwidth, a,
+                                  XtNheight, text_entry_height,
 				  NULL);
         arg->hidden = XtVaCreateManagedWidget("hidden",
 				  toggleWidgetClass, form,
@@ -1179,7 +1390,7 @@ buildBrowser(Widget parent, char *title, arg_t * arg,
                                   XtNfromHoriz, arg->dot,
                                   XtNhorizDistance, 3,
                                   XtNvertDistance, 0,
-                                  XtNheight, 20,
+                                  XtNheight, text_entry_height,
 				  NULL);
         if (arg->browserType <= BROWSER_MULTIREAD)
             arg->ongoing = XtVaCreateManagedWidget("ongoing",
@@ -1188,13 +1399,88 @@ buildBrowser(Widget parent, char *title, arg_t * arg,
                                   XtNfromHoriz, arg->hidden,
                                   XtNhorizDistance, 3,
                                   XtNvertDistance, 0,
-                                  XtNheight, 20,
+                                  XtNheight, text_entry_height,
 				  XtNstate, (arg->browserType==BROWSER_MULTIREAD),
 				  NULL);
         else
 	    arg->ongoing = None;
     }
-
+    if (arg->browserType == BROWSER_SAVE) {
+        arg->quality_w = XtVaCreateManagedWidget("quality",
+				  labelWidgetClass, form,
+				  XtNborderWidth, 0,
+				  XtNfromVert, vport,
+				  XtNfromHoriz, arg->hidden,
+                                  XtNhorizDistance, 100,
+                                  XtNvertDistance, 0,
+				  XtNjustify, XtJustifyLeft,
+				  XtNresizable, False,
+				  NULL);
+        arg->quality_ll = XtVaCreateManagedWidget("quality_ll",
+				  commandWidgetClass, form,
+				  XtNlabel, "«",
+                                  XtNfromVert, vport,
+                                  XtNfromHoriz, arg->quality_w,
+                                  XtNhorizDistance, 3,
+                                  XtNvertDistance, 0,
+                                  XtNwidth, char_entry_width,
+                                  XtNheight, text_entry_height,
+				  NULL);
+        arg->quality_l = XtVaCreateManagedWidget("quality_l",
+				  commandWidgetClass, form,
+				  XtNlabel, "<",
+                                  XtNfromVert, vport,
+                                  XtNfromHoriz, arg->quality_ll,
+                                  XtNhorizDistance, 3,
+                                  XtNvertDistance, 0,
+                                  XtNwidth, char_entry_width,
+                                  XtNheight, text_entry_height,
+				  NULL);
+        arg->quality = XtVaCreateManagedWidget("quality_val",
+				  asciiTextWidgetClass, form,
+				  XtNfromVert, vport,
+                                  XtNfromHoriz, arg->quality_l,
+				  XtNeditType, XawtextEdit,
+				  XtNwrap, XawtextWrapNever,
+				  XtNresizable, False,
+                                  XtNhorizDistance, 3,
+                                  XtNvertDistance, 0,
+                                  XtNwidth, 48,
+				  XtNheight, text_entry_height,
+			   	  NULL);
+        XtAddEventHandler(arg->name, ButtonPressMask | ButtonReleaseMask, 
+                                  False, (XtEventHandler) buttonPressed,
+                                  (XtPointer) form);
+        XtAddEventHandler(arg->quality, KeyPressMask | KeyReleaseMask |
+                                  ButtonPressMask | ButtonReleaseMask, 
+                                  False, (XtEventHandler) buttonPressed,
+                                  (XtPointer) form);
+        arg->quality_r = XtVaCreateManagedWidget("quality_r",
+				  commandWidgetClass, form,
+				  XtNlabel, ">",
+                                  XtNfromVert, vport,
+                                  XtNfromHoriz, arg->quality,
+                                  XtNhorizDistance, 3,
+                                  XtNvertDistance, 0,
+                                  XtNwidth, char_entry_width,
+                                  XtNheight, text_entry_height,
+				  NULL);
+        arg->quality_rr = XtVaCreateManagedWidget("quality_rr",
+				  commandWidgetClass, form,
+				  XtNlabel, "»",
+                                  XtNfromVert, vport,
+                                  XtNfromHoriz, arg->quality_r,
+                                  XtNhorizDistance, 3,
+                                  XtNvertDistance, 0,
+                                  XtNwidth, char_entry_width,
+                                  XtNheight, text_entry_height,
+				  NULL);
+    } else {
+       arg->quality_w = None;
+       arg->quality_l = None;
+       arg->quality_r = None;
+       arg->quality = None;
+    }
     cwd = XtVaCreateManagedWidget("cwd",
 				  labelWidgetClass, form,
 				  XtNlabel, arg->dirname,
@@ -1228,13 +1514,28 @@ buildBrowser(Widget parent, char *title, arg_t * arg,
         XtAddCallback(arg->hidden, XtNcallback, 
                       refreshCallback, (XtPointer) arg);
     }
+
+    if (arg->quality_w) {
+        XtAddCallback(arg->quality_l, XtNcallback, 
+                      qualityCallback, (XtPointer) arg);
+        XtAddCallback(arg->quality_ll, XtNcallback, 
+                      qualityCallback, (XtPointer) arg);
+        XtAddCallback(arg->quality_r, XtNcallback, 
+                      qualityCallback, (XtPointer) arg);
+        XtAddCallback(arg->quality_rr, XtNcallback, 
+                      qualityCallback, (XtPointer) arg);
+        sprintf(value, "%d", file_JPEG_quality);
+        XtVaSetValues(arg->quality, XtNstring, &value, NULL);
+    }
+
     XtAddCallback(list, XtNcallback, listCallback, (XtPointer) arg);
     XtAddEventHandler(list, ButtonPressMask, False,
 		      (XtEventHandler) mousewheelScroll, (XtPointer) 1);
 
     arg->first = True;
 
-    if (arg->browserType != BROWSER_LOADED) XtSetKeyboardFocus(form, arg->name);
+    if (arg->browserType != BROWSER_LOADED) 
+        XtSetKeyboardFocus(form, arg->name);
 
     setCWD(arg, doDirname(arg, NULL));
     return form;
@@ -1266,6 +1567,7 @@ buildOpenBrowser(Widget w, arg_t * arg, int width, int height, int mode)
 				   formWidgetClass, shell,
 				   XtNborderWidth, 0,
 				   NULL);
+    
     browser = buildBrowser(form, msgText[OPEN_FILE_NAMED], arg, width, height);
     arg->browser = browser;
 
@@ -1280,6 +1582,7 @@ buildOpenBrowser(Widget w, arg_t * arg, int width, int height, int mode)
 				       XtNbottom, XtChainTop,
 				       XtNleft, XtChainRight,
 				       XtNright, XtChainRight,
+				       XtNheight, Global.btn_height,
 				       NULL);
     arg->ok = okButton;
     cancelButton = XtVaCreateManagedWidget("cancel",
@@ -1288,6 +1591,7 @@ buildOpenBrowser(Widget w, arg_t * arg, int width, int height, int mode)
 					   XtNbottom, XtChainTop,
 					   XtNleft, XtChainRight,
 					   XtNright, XtChainRight,
+				           XtNheight, Global.btn_height,
 					   NULL);
     arg->cancel = cancelButton;
 
@@ -1332,8 +1636,7 @@ buildOpenBrowser(Widget w, arg_t * arg, int width, int height, int mode)
 
 	if (firstToggle == None) {
 	    arg->type = NULL;
-	    XtVaSetValues(toggle, XtNstate, True,
-			  XtNvertDistance, 29, NULL);
+	    XtVaSetValues(toggle, XtNstate, True, XtNvertDistance, 29, NULL);
 	    firstToggle = toggle;
 	}
 	XtAddCallback(toggle, XtNcallback, fileTypeCallback, (XtPointer) arg);
@@ -1350,6 +1653,7 @@ buildOpenBrowser(Widget w, arg_t * arg, int width, int height, int mode)
                                        XtNfromVert, arg->info,
 				       XtNvertDistance, 10,
 				       XtNhorizDistance, 10,
+				       XtNheight, Global.btn_height,
 				       NULL);
 
         arg->delete = XtVaCreateManagedWidget("delete",
@@ -1359,6 +1663,7 @@ buildOpenBrowser(Widget w, arg_t * arg, int width, int height, int mode)
                                        XtNfromHoriz, arg->refresh,
 				       XtNvertDistance, 10,
 				       XtNhorizDistance, 10,
+				       XtNheight, Global.btn_height,      
 				       NULL);
 
         arg->edit = XtVaCreateManagedWidget("edit",
@@ -1368,6 +1673,7 @@ buildOpenBrowser(Widget w, arg_t * arg, int width, int height, int mode)
                                        XtNfromVert, arg->info,
 				       XtNvertDistance, 10,
 				       XtNhorizDistance, 10,
+				       XtNheight, Global.btn_height,	    
 				       NULL);
 
         arg->create = XtVaCreateManagedWidget("create",
@@ -1377,6 +1683,7 @@ buildOpenBrowser(Widget w, arg_t * arg, int width, int height, int mode)
                                        XtNfromVert, arg->info,
 				       XtNvertDistance, 10,
 				       XtNhorizDistance, 10,
+				       XtNheight, Global.btn_height,
 				       NULL);
 
         arg->zoomlabel = XtVaCreateManagedWidget("zoom",
@@ -1385,6 +1692,7 @@ buildOpenBrowser(Widget w, arg_t * arg, int width, int height, int mode)
                                        XtNfromVert, arg->refresh,
 				       XtNvertDistance, 10,
 				       XtNhorizDistance, 10,
+				       XtNheight, text_entry_height,
 				       NULL);
 
         arg->zoom = XtVaCreateManagedWidget("zoomW",
@@ -1398,7 +1706,7 @@ buildOpenBrowser(Widget w, arg_t * arg, int width, int height, int mode)
                                        XtNfromHoriz, arg->zoomlabel,
 				       XtNhorizDistance, 10,
 				       XtNwidth, 50,
-				       XtNheight, 20,
+				       XtNheight, text_entry_height,
 				       NULL);
         XtVaSetValues(arg->zoom, XtNstring, ZoomToStr(Global.default_zoom),
                       NULL);
@@ -1410,6 +1718,7 @@ buildOpenBrowser(Widget w, arg_t * arg, int width, int height, int mode)
                                        XtNfromHoriz, arg->zoom,
 				       XtNvertDistance, 10,
 				       XtNhorizDistance, 10,
+				       XtNheight, text_entry_height,
 				       NULL);
 
         arg->dpi = XtVaCreateManagedWidget("dpiW",
@@ -1423,7 +1732,7 @@ buildOpenBrowser(Widget w, arg_t * arg, int width, int height, int mode)
                                        XtNfromHoriz, arg->dpilabel,
 				       XtNhorizDistance, 10,
 				       XtNwidth, 50,
-				       XtNheight, 20,
+				       XtNheight, text_entry_height,
 				       NULL);
         sprintf(value, "%g", Global.dpi);
         XtVaSetValues(arg->dpi, XtNstring, value, NULL);
@@ -1431,7 +1740,7 @@ buildOpenBrowser(Widget w, arg_t * arg, int width, int height, int mode)
         arg->bboxlabel = XtVaCreateManagedWidget("BBox",
 				       labelWidgetClass, browser,
 				       XtNborderWidth, 0,
-                                       XtNheight, 20,
+                                       XtNheight, text_entry_height,
                                        XtNfromVert, arg->refresh,
 				       XtNvertDistance, 7,
                                        XtNfromHoriz, arg->dpi,
@@ -1444,8 +1753,8 @@ buildOpenBrowser(Widget w, arg_t * arg, int width, int height, int mode)
 				       XtNvertDistance, 7,
 				       XtNfromHoriz, arg->bboxlabel,
 				       XtNhorizDistance, 3,
-				       XtNheight, 20,
-				       XtNwidth, 18,
+				       XtNheight, text_entry_height,
+				       XtNwidth, char_entry_width,
 				       NULL);
 
         arg->bbox = XtVaCreateManagedWidget("bbox",
@@ -1458,7 +1767,8 @@ buildOpenBrowser(Widget w, arg_t * arg, int width, int height, int mode)
 				       XtNvertDistance, 7,
                                        XtNfromHoriz, arg->bboxleft,
 				       XtNhorizDistance, 3,
-				       XtNwidth, 18,
+				       XtNheight, text_entry_height,
+				       XtNwidth, char_entry_width,
 				       NULL);
         sprintf(value, "%d", file_bbox);
         XtVaSetValues(arg->bbox, XtNstring, value, NULL);
@@ -1469,8 +1779,8 @@ buildOpenBrowser(Widget w, arg_t * arg, int width, int height, int mode)
 				       XtNvertDistance, 7,
 				       XtNfromHoriz, arg->bbox,
 				       XtNhorizDistance, 3,
-				       XtNheight, 20,
-				       XtNwidth, 18,
+				       XtNheight, text_entry_height,
+				       XtNwidth, char_entry_width,
 				       NULL);
 
         arg->pagelabel = XtVaCreateManagedWidget("page",
@@ -1479,17 +1789,19 @@ buildOpenBrowser(Widget w, arg_t * arg, int width, int height, int mode)
                                        XtNfromVert, arg->dpi,
 				       XtNvertDistance, 7,
 				       XtNhorizDistance, 10,
+				       XtNheight, text_entry_height,
 				       NULL);
 
         sprintf(value, "/%d", file_numpages);
         arg->pagetotal = XtVaCreateManagedWidget(value,
 				       labelWidgetClass, browser,
 				       XtNborderWidth, 0,
-				       XtNwidth, 31,
+				       XtNwidth, 3*char_entry_width,
                                        XtNfromVert, arg->dpi,
 				       XtNvertDistance, 7,
 				       XtNfromHoriz, arg->pagelabel,
-				       XtNhorizDistance, 0,
+				       XtNhorizDistance, 10,
+				       XtNheight, text_entry_height, 
 				       NULL);
 
         arg->pageLeft = XtVaCreateManagedWidget("xxleft",
@@ -1498,8 +1810,8 @@ buildOpenBrowser(Widget w, arg_t * arg, int width, int height, int mode)
 				       XtNvertDistance, 7,
 				       XtNfromHoriz, arg->pagetotal,
 				       XtNhorizDistance, 10,
-				       XtNheight, 20,
-				       XtNwidth, 18,
+				       XtNheight, text_entry_height,
+				       XtNwidth, char_entry_width,
 				       NULL);
 
         arg->pageleft = XtVaCreateManagedWidget("xleft",
@@ -1508,8 +1820,8 @@ buildOpenBrowser(Widget w, arg_t * arg, int width, int height, int mode)
 				       XtNvertDistance, 7,
 				       XtNfromHoriz, arg->pageLeft,
 				       XtNhorizDistance, 3,
-				       XtNheight, 20,
-				       XtNwidth, 18,
+				       XtNwidth, char_entry_width,
+				       XtNheight, text_entry_height,		
 				       NULL);
 
         arg->page = XtVaCreateManagedWidget("pageW",
@@ -1523,6 +1835,7 @@ buildOpenBrowser(Widget w, arg_t * arg, int width, int height, int mode)
                                        XtNfromHoriz, arg->pageleft,
 				       XtNhorizDistance, 3,
 				       XtNwidth, 48,
+				       XtNheight, text_entry_height,
 				       NULL);
 
         arg->pageright = XtVaCreateManagedWidget("xright",
@@ -1531,8 +1844,8 @@ buildOpenBrowser(Widget w, arg_t * arg, int width, int height, int mode)
 				       XtNvertDistance, 7,
 				       XtNfromHoriz, arg->page,
 				       XtNhorizDistance, 3,
-				       XtNheight, 20,
-				       XtNwidth, 18,
+				       XtNheight, text_entry_height,
+				       XtNwidth, char_entry_width,
 				       NULL);
 
         arg->pageRight = XtVaCreateManagedWidget("xxright",
@@ -1541,8 +1854,8 @@ buildOpenBrowser(Widget w, arg_t * arg, int width, int height, int mode)
 				       XtNvertDistance, 7,
 				       XtNfromHoriz, arg->pageright,
 				       XtNhorizDistance, 3,
-				       XtNheight, 20,
-				       XtNwidth, 18,
+				       XtNheight, text_entry_height,
+				       XtNwidth, char_entry_width,
 				       NULL);
 
         sprintf(value, "%d", Global.numpage);
@@ -1610,7 +1923,7 @@ buildSaveBrowser(Widget w, arg_t * arg)
 				   formWidgetClass, shell,
 				   XtNborderWidth, 0,
 				   NULL);
-    browser = buildBrowser(form, msgText[SAVE_IN_FILE], arg, 250, 380);
+    browser = buildBrowser(form, msgText[SAVE_IN_FILE], arg, 450, 380);
     arg->browser = browser;
    
     accel = XtParseAcceleratorTable("#override\n\
@@ -1702,7 +2015,8 @@ buildSaveBrowser(Widget w, arg_t * arg)
     AddDestroyCallback(shell,
 		       (DestroyCallbackFunc) cancelCallback, (XtPointer) arg);
 
-    XtSetKeyboardFocus(form, arg->name);
+    if (!arg->quality_w)
+        XtSetKeyboardFocus(form, arg->name);
     XtInstallAccelerators(arg->name, okButton);
 
     return shell;
@@ -1726,7 +2040,7 @@ buildSimpleBrowser(Widget w, arg_t * arg, Boolean isSave)
 				   NULL);
     browser = buildBrowser(form, isSave ? 
 			   msgText[SAVE_IN_FILE] : msgText[LOAD_FROM_FILE],
-			   arg, 250, 380);
+			   arg, 450, 380);
     arg->browser = browser;   
 
     accel = XtParseAcceleratorTable("#override\n\
@@ -1820,13 +2134,13 @@ getArg(Widget w, Boolean type, Boolean * built)
 
     switch (type) {
     case BROWSER_READ:
-        shell = buildOpenBrowser(w, cur, 250, 380, BROWSER_READ);
+        shell = buildOpenBrowser(w, cur, 500, 380, BROWSER_READ);
 	break;
     case BROWSER_MULTIREAD:
-        shell = buildOpenBrowser(w, cur, 250, 380, BROWSER_MULTIREAD);
+        shell = buildOpenBrowser(w, cur, 500, 380, BROWSER_MULTIREAD);
         break;
     case BROWSER_LOADED:
-        shell = buildOpenBrowser(w, cur, 340, 300, BROWSER_LOADED);
+        shell = buildOpenBrowser(w, cur, 580, 300, BROWSER_LOADED);
         break;
     case BROWSER_SAVE:
         shell = buildSaveBrowser(w, cur);
@@ -1904,12 +2218,12 @@ GetFileName(Widget w, int type, char *def, XtCallbackProc okFunc, XtPointer data
 	    setCWD(arg, doDirname(arg, dirname));
 
 	XtVaSetValues(arg->name, XtNstring, cp,
-		      XtNinsertPosition, strlen(cp),
-		      NULL);
+		      XtNinsertPosition, strlen(cp), NULL);
     } else if (!built) {
 	/*
 	**  This browser has been used before, rescan directory.
 	 */
+        arg->dobuilt = 0;
 	setCWD(arg, NULL);
     }
     arg->okFunc = okFunc;
@@ -1920,7 +2234,9 @@ GetFileName(Widget w, int type, char *def, XtCallbackProc okFunc, XtPointer data
     shell = GetShell(arg->name);
 
     XtPopup(shell, XtGrabNone);
+    arg->dobuilt = 1;
     XtVaGetValues(shell, XtNtitle, &def, NULL);
+    SetWMInputHint(XtDisplay(shell), XtWindow(shell));
     StoreName(shell, def);
 
     if (arg->bbox)
