@@ -89,12 +89,12 @@ typedef struct {
     XftFont * xft_font;
     char * xft_text;
     char * xft_name;
-    double xft_size, xft_height, xft_ascent, xft_descent, xft_maxadv;
-    double xft_rotation, xft_inclination, xft_dilation, xft_linespacing;
+    float xft_size, xft_height, xft_ascent, xft_descent, xft_maxadv;
+    float xft_rotation, xft_inclination, xft_dilation, xft_linespacing;
     Pixmap pixmap;
     Widget shell, form1, form2, subform;
     Widget vport0, vport1, vport2, sample, paint;
-    Widget family, familyLabel;
+    Widget title, family, familyLabel;
     Widget weight, weightLabel;
     Widget pointBar, pointSelect, pointSelectLabel;
     Widget rotation, rotationLabel;
@@ -133,10 +133,10 @@ int GetCharLength(char *buf)
 void
 FreeTypeDrawString(Display *dpy, arg_t * arg,
                    XftDraw *draw, XftColor * color,
-                   char *buf, int len, int x, int y, double r)
+                   char *buf, int len, int x, int y, float r)
 {
     int j, charlen;
-    double dx=0.5, dy=0.5, a, u, v;
+    float dx=0.5, dy=0.5, a, u, v;
     XGlyphInfo extents = {};
 
     u = cos(r);
@@ -240,15 +240,15 @@ setXftFont(arg_t * arg)
     Colormap cmap;
     GC gc;
     Dimension width, height;
-    Boolean bool;
+    Boolean boolstate;
 
     XftColor color;
     XftDraw *draw = NULL;
     XRenderColor xre_color;
     char matrix[80];
     char *ptr, *str;
-    double r=0.0, i=0.0, d=1.0, ls=1.0, hd;
-    double mat[4];
+    float r=0.0, i=0.0, d=1.0, ls=1.0, hd;
+    float mat[4];
     int j, x, y, w, x0, x1, x2, y0, y1, y2;
 
     if (!arg->font_select) {
@@ -343,8 +343,8 @@ setXftFont(arg_t * arg)
         return;
     }
 
-    XtVaGetValues(arg->editButton, XtNstate, &bool, NULL);
-    if (bool) {
+    XtVaGetValues(arg->editButton, XtNstate, &boolstate, NULL);
+    if (boolstate) {
         XtVaSetValues(arg->editButton, XtNstate, False, NULL);
         editCallback(arg->editButton, arg, NULL);
     }
@@ -438,7 +438,7 @@ barCallback(Widget w, arg_t * arg, float *percent)
 }
 
 static void
-setBar(arg_t * arg, double npts)
+setBar(arg_t * arg, float npts)
 {
     float percent; 
     char val[8];
@@ -465,7 +465,7 @@ static void
 scrollCallback(Widget w, arg_t * arg, XtPointer position)
 {
     char *ptr;
-    double npts;
+    float npts;
 
     XtVaGetValues(arg->pointSelect, XtNstring, &ptr, NULL);
     npts = atof(ptr);
@@ -491,11 +491,11 @@ applyCallback(Widget w, XtPointer argArg, XtPointer junk)
     Display *dpy = XtDisplay(w);
     arg_t *arg = (arg_t *) argArg;
     XftFont * font;
-    Boolean bool;
+    Boolean boolstate;
     char *str;
 
-    XtVaGetValues(arg->editButton, XtNstate, &bool, NULL);
-    if (bool)
+    XtVaGetValues(arg->editButton, XtNstate, &boolstate, NULL);
+    if (boolstate)
         XtVaGetValues(arg->sample, XtNstring, &str, NULL);
     else
         str = arg->xft_text;
@@ -643,9 +643,9 @@ fontboxResized(Widget w, arg_t * arg, XConfigureEvent * event, Boolean * flg)
 {
     Display *dpy = XtDisplay(w);
     Dimension width, height, height1, height2, width2;
-    Position x, y;
+    Position a, x, y, u;
     char *val;
-    double m = 18.0;
+    float m = 18.0;
 
     XtVaGetValues(arg->shell, XtNwidth, &width, XtNheight, &height, NULL);
     XtVaGetValues(arg->form1, XtNheight, &height1, NULL);
@@ -653,28 +653,28 @@ fontboxResized(Widget w, arg_t * arg, XConfigureEvent * event, Boolean * flg)
     if (height1<150) height1 = 150;
     XtResizeWidget(arg->shell, width, height, 0);
     XtResizeWidget(arg->form1, width, height1, 0);
-    height1 -= 32;
-   
-    XtResizeWidget(arg->familyLabel, 150, 20, 0);
-    XtVaSetValues(arg->familyLabel, XtNwidth, 150, XtNheight, 20, NULL);   
-    XtMoveWidget(arg->familyLabel, -2, 4);
+    height1 -= Global.btn_height + 12;
 
-    XtMoveWidget(arg->vport0, 4, 27);
+    XtResizeWidget(arg->familyLabel, 150, Global.btn_height, 0);
+    XtVaSetValues(arg->familyLabel, XtNwidth, 150, XtNheight, Global.btn_height, NULL);   
+    XtMoveWidget(arg->familyLabel, 0, 6);
+
+    XtMoveWidget(arg->vport0, 4, Global.btn_height+5);
     XtResizeWidget(arg->vport0, width-218, height1, BORDERWIDTH);
     XtVaSetValues(arg->vport0, XtNwidth, width-218, NULL);
 
-    XtResizeWidget(arg->weightLabel, 150, 20, 0);
-    XtVaSetValues(arg->weightLabel, XtNwidth, 150, XtNheight, 20, NULL);   
+    XtResizeWidget(arg->weightLabel, 150, 6, 0);
+    XtVaSetValues(arg->weightLabel, XtNwidth, 150, XtNheight, Global.btn_height, NULL);   
     XtMoveWidget(arg->weightLabel, width-220, 4);
 
-    XtMoveWidget(arg->vport1, width-209, 27);
+    XtMoveWidget(arg->vport1, width-209, Global.btn_height+5);
     XtResizeWidget(arg->vport1, 205, height1, BORDERWIDTH);
     XtVaSetValues(arg->vport1, XtNwidth, 205, NULL);   
 
 #ifdef XAW3D   
-    height2 = height-height1-162;
+    height2 = height-height1- 5 * Global.btn_height - 54;
 #else
-    height2 = height-height1-158;
+    height2 = height-height1- 5 * Global.btn_height - 54;
 #endif
     if (height2<4) height2 = 4;
 
@@ -682,36 +682,39 @@ fontboxResized(Widget w, arg_t * arg, XConfigureEvent * event, Boolean * flg)
     XtVaSetValues(arg->vport2, XtNwidth, width-8, NULL);
     XtResizeWidget(arg->sample, width-8, height2, 0);
     XtVaGetValues(arg->editButton, XtNwidth, &width2, NULL);
-    XtResizeWidget(arg->editButton, width2, 22, 0);
+    XtResizeWidget(arg->editButton, width2, Global.btn_height, 0);
 
-    XtResizeWidget(arg->subform, width, 60, 0);
+    XtResizeWidget(arg->subform, width, 2*Global.btn_height+16, 0);
     XtMoveWidget(arg->pointSelectLabel, 4, 8);
     XtVaGetValues(arg->form2, XtNheight, &height2, NULL);
-    y = height2-25;
+    y = height2 - Global.btn_height - 5;
     XtMoveWidget(arg->editButton, 4, y);
     XtVaGetValues(arg->applyButton, XtNx, &x, NULL);
     XtMoveWidget(arg->applyButton, x, y);
     XtVaGetValues(arg->menubar, XtNx, &x, NULL);
     XtMoveWidget(arg->menubar, x, y-4);
-    XtMoveWidget(arg->doneButton, 304, y);
+    XtMoveWidget(arg->doneButton, 700, y);
 
-    XtResizeWidget(arg->pointBar, 350, 20, 0);
-    XtMoveWidget(arg->pointBar, 150, 8);
-    XtMoveWidget(arg->pointSelect, 506, 8);
+    a = 2 + Global.btn_height / 11;
+    u = Global.btn_height + 16;
 
-    XtMoveWidget(arg->rotationLabel, 4, 38);
-    XtMoveWidget(arg->rotation, 90, 38);
+    XtResizeWidget(arg->pointBar, 45*a, 20, 0);
+    XtMoveWidget(arg->pointBar, 46*a, Global.btn_height/2 -2);
+    XtMoveWidget(arg->pointSelect, 95*a, 8);
 
-    XtMoveWidget(arg->inclinationLabel, 164, 38);
-    XtMoveWidget(arg->inclination, 250, 38);
+    XtMoveWidget(arg->rotationLabel, 4, u);
+    XtMoveWidget(arg->rotation, 22*a, u);
 
-    XtMoveWidget(arg->dilationLabel, 324, 38);
-    XtMoveWidget(arg->dilation, 410, 38);
+    XtMoveWidget(arg->inclinationLabel, 42*a, u);
+    XtMoveWidget(arg->inclination, 66*a, u);
 
-    XtMoveWidget(arg->linespacingLabel, 484, 38);
-    XtMoveWidget(arg->linespacing, 570, 38);
+    XtMoveWidget(arg->dilationLabel, 87*a, u);
+    XtMoveWidget(arg->dilation, 110*a, u);
 
-    XtMoveWidget(arg->vport2, 4, 76);
+    XtMoveWidget(arg->linespacingLabel, 130*a, u);
+    XtMoveWidget(arg->linespacing, 154*a, u);
+
+    XtMoveWidget(arg->vport2, 4, 2*u);
 
     XtVaGetValues(arg->pointSelect, XtNstring, &val, NULL);
     if (val) m = atof(val);
@@ -726,7 +729,6 @@ fontboxResized(Widget w, arg_t * arg, XConfigureEvent * event, Boolean * flg)
         cleanSamplePixmap(arg);
     }
 
-
     if (!XPending(XtDisplay(w)))
         setXftFont(arg);
 }
@@ -1022,7 +1024,7 @@ externCallback(Widget w, XtPointer argArg, XtPointer junk)
 
    /* call external editor */
    sprintf(buf, "%s %s", EDITOR, name);
-   system(buf);
+   if (system(buf));
 
    /* Now copy edited file into buffer */
    fd = fopen(name, "r");
@@ -1048,14 +1050,14 @@ static void
 editCallback(Widget w, XtPointer argArg, XtPointer junk)
 {
     arg_t *arg = (arg_t *) argArg;
-    Boolean bool;
+    Boolean boolstate;
     Dimension width;
     char *str;
 
-    XtVaGetValues(w, XtNstate, &bool, XtNwidth, &width, NULL);
-    XtResizeWidget(arg->editButton, width, 22, 0);
+    XtVaGetValues(w, XtNstate, &boolstate, XtNwidth, &width, NULL);
+    XtResizeWidget(arg->editButton, width, Global.btn_height, 0);
 
-    if (bool) {
+    if (boolstate) {
         cleanSamplePixmap(arg);
         XtVaSetValues(arg->sample,
                       XtNeditType, XawtextEdit,
@@ -1101,9 +1103,9 @@ FontSelect(Widget w, Widget paint)
     static arg_t *arg;
 
     Widget pane, form1, form2;
-    Widget editButton, applyButton, doneButton, label;
+    Widget editButton, applyButton, doneButton, title;
     char *ptr1, *ptr2;
-    int j, k, l, lmax;
+    int j, k, l, lmax, m;
 
     PopdownMenusGlobal();
 
@@ -1125,6 +1127,7 @@ FontSelect(Widget w, Widget paint)
                       XFT_FAMILY, XFT_FOUNDRY, XFT_STYLE, (char *)0);
     arg->font_desc = (char **) xmalloc(fs->nfont * sizeof(char *));
     lmax = 0;
+
     for (j = 0; j <fs->nfont; j++) {
         /* Caution : this allocates str */
         str = XftNameUnparseAlloc(fs->fonts[j]);
@@ -1155,7 +1158,8 @@ FontSelect(Widget w, Widget paint)
     qsort(arg->font_desc, fs->nfont, sizeof(char *),
           (int (*)(_Xconst void *, _Xconst void *)) strqsortcmp);
 
-    k = 0;
+    k = -1;
+    m = -1;
     arg->font_family = (char **) xmalloc(fs->nfont * sizeof(char *));
 
     for (j = 0; j<fs->nfont; j++) {
@@ -1169,16 +1173,17 @@ FontSelect(Widget w, Widget paint)
 	    ptr2 = NULL;
         if (ptr2) *ptr2 = '\0';
         l = strlen(arg->font_desc[j]);
-        if (k==0 ||
-            strncasecmp(arg->font_desc[j], arg->font_family[k-1], l)) {
-	  if (ptr1 && strcmp(ptr1+1, "unknown")) {
+        if (l != m ||
+            strncasecmp(arg->font_desc[j], arg->font_family[k], l)) {
+            ++k;
+	    if (ptr1 && strcmp(ptr1+1, "unknown")) {
                 arg->font_family[k] = 
 	            (char *) xmalloc(l+strlen(ptr1+1)+5);
                 sprintf(arg->font_family[k], "%s  (%s)", 
-                       arg->font_desc[j], ptr1+1);
+                    arg->font_desc[j], ptr1+1);
 	    } else
 	        arg->font_family[k] = strdup(arg->font_desc[j]);
-            ++k;
+            m = l;
 	}
         if (ptr1) *ptr1 = ':';
         if (ptr2) *ptr2 = ':';
@@ -1203,17 +1208,20 @@ FontSelect(Widget w, Widget paint)
 				   XtNborderWidth, 0,
 				   NULL);
 
-    label = XtVaCreateManagedWidget("title",
+    title = XtVaCreateManagedWidget("title",
 				    labelWidgetClass, pane,
 	                            XtNlabel, msgText[FONT_SELECT_DESIRED_PROPERTIES],
 				    XtNborderWidth, 0,
 				    XtNshowGrip, False,
+				    XtNheight, Global.btn_height,
 				    NULL);
 
+    arg->title = title;
+    
     form1 = XtVaCreateManagedWidget("form",
 				   formWidgetClass, pane,
 				   XtNborderWidth, 0,
-				   XtNwidth, 600, XtNheight, 200,
+				   XtNwidth, 600, XtNheight, 300,
 				   NULL);
     arg->form1 = form1;
 
@@ -1226,11 +1234,12 @@ FontSelect(Widget w, Widget paint)
 				    XtNborderWidth, 0,
 				    XtNtop, XtChainTop,
 				    XtNbottom, XtChainTop,
+				    XtNheight, Global.btn_height,
 				    NULL);
 
     arg->vport0 = XtVaCreateManagedWidget("vport",
 				    viewportWidgetClass, form1,
-				    XtNwidth, 420, XtNheight, 200,
+				    XtNwidth, 520, XtNheight, 300,
 				    XtNuseBottom, True,
 				    XtNuseRight, True,
 				    XtNforceBars, False,
@@ -1241,12 +1250,13 @@ FontSelect(Widget w, Widget paint)
 				    NULL);
 
     arg->family = XtVaCreateManagedWidget("font",
-					  listWidgetClass, arg->vport0,
-					  XtNverticalList, True,
-					  XtNforceColumns, True,
-					  XtNdefaultColumns, 1,
-					  XtNnumberStrings, 0,
-					  NULL);
+				    listWidgetClass, arg->vport0,
+				    XtNverticalList, True,
+				    XtNforceColumns, True,
+				    XtNdefaultColumns, 1,
+				    XtNnumberStrings, 0,
+				    XtNrowSpacing, ROW_SPACING,
+				    NULL);
 
     arg->weightLabel = XtVaCreateWidget("weightLabel",
 				    labelWidgetClass, form1,
@@ -1254,11 +1264,12 @@ FontSelect(Widget w, Widget paint)
 				    XtNfromHoriz, arg->vport0,
 				    XtNtop, XtChainTop,
 				    XtNbottom, XtChainTop,
+				    XtNheight, Global.btn_height,	
 				    NULL);
 
     arg->vport1 = XtVaCreateManagedWidget("vport",
 				    viewportWidgetClass, form1,
-				    XtNwidth, 220, XtNheight, 200,
+				    XtNwidth, 280, XtNheight, 300,
 				    XtNuseBottom, True,
 				    XtNuseRight, True,
 				    XtNforceBars, False,
@@ -1270,12 +1281,13 @@ FontSelect(Widget w, Widget paint)
 				    NULL);
 
     arg->weight = XtVaCreateManagedWidget("weight",
-					  listWidgetClass, arg->vport1,
-					  XtNverticalList, True,
-					  XtNforceColumns, True,
-					  XtNdefaultColumns, 1,
-					  XtNnumberStrings, 0,
-					  NULL);
+				    listWidgetClass, arg->vport1,
+				    XtNverticalList, True,
+				    XtNforceColumns, True,
+				    XtNdefaultColumns, 1,
+				    XtNnumberStrings, 0,
+				    XtNrowSpacing, ROW_SPACING,
+				    NULL);
 
     /*
     **	The text area and buttons
@@ -1292,8 +1304,8 @@ FontSelect(Widget w, Widget paint)
 				    XtNborderWidth, 0,
 				    XtNtop, XtChainTop,
 				    XtNbottom, XtChainTop,
-				    XtNwidth, 600,
-				    XtNheight, 60,
+				    XtNwidth, 800,
+				    XtNheight, 2 * Global.btn_height + 16,
 				    NULL);
 
     XtAppAddActions(XtWidgetToApplicationContext(arg->subform), &dataAct, 1);
@@ -1303,6 +1315,7 @@ FontSelect(Widget w, Widget paint)
 				   XtNborderWidth, 0,
 				   XtNleft, XtChainLeft,
 				   XtNright, XtChainLeft,
+				   XtNheight, Global.btn_height,
 				   NULL);
 
     arg->pointBar = XtVaCreateManagedWidget("pointBar", 
@@ -1324,6 +1337,7 @@ FontSelect(Widget w, Widget paint)
 					XtNeditType, XawtextEdit,
 					XtNwrap, XawtextWrapNever,
                                         XtNwidth, 64,
+				        XtNheight, Global.btn_height,	
 					XtNlength, 8,
 					XtNtranslations,
 					XtParseTranslationTable("#override\n\
@@ -1339,6 +1353,7 @@ FontSelect(Widget w, Widget paint)
 				          XtNfromVert, arg->pointSelect,
 				          XtNleft, XtChainLeft,
 				          XtNright, XtChainLeft,
+				          XtNheight, Global.btn_height,
 				          NULL);
 
     arg->rotation = XtVaCreateWidget("rotation",
@@ -1369,6 +1384,7 @@ FontSelect(Widget w, Widget paint)
 				   XtNhorizDistance, 20,
 				   XtNleft, XtChainLeft,
 				   XtNright, XtChainLeft,
+				   XtNheight, Global.btn_height,
 				   NULL);
 
     arg->inclination = XtVaCreateWidget("inclination",
@@ -1399,6 +1415,7 @@ FontSelect(Widget w, Widget paint)
 				   XtNhorizDistance, 20,
 				   XtNleft, XtChainLeft,
 				   XtNright, XtChainLeft,
+				   XtNheight, Global.btn_height,
 				   NULL);
 
     arg->linespacing = XtVaCreateWidget("linespacing",
@@ -1453,7 +1470,7 @@ FontSelect(Widget w, Widget paint)
 
     arg->vport2 = XtVaCreateManagedWidget("vport",
 				    viewportWidgetClass, form2,
-				    XtNheight, 160,
+				    XtNheight, 240,
 				    XtNallowResize, True,
 				    XtNuseRight, True,
 				    XtNforceBars, False,
@@ -1469,8 +1486,8 @@ FontSelect(Widget w, Widget paint)
 					  XtNallowResize, True,
                                           XtNeditType, XawtextEdit,
                                           XtNresizable, True,
-                                          XtNwidth, 620,
-                                          XtNheight, 300,
+                                          XtNwidth, 800,
+                                          XtNheight, 200,
 	                                  XtNdisplayCaret, True,
 		                          XtNstring, Global.xft_text, NULL);
 
@@ -1489,7 +1506,7 @@ FontSelect(Widget w, Widget paint)
     applyButton = XtVaCreateManagedWidget("apply",
 					  commandWidgetClass, form2,
 					  XtNfromVert, arg->vport2,
-                                          XtNheight, 22,
+                                          XtNheight, Global.btn_height,
                                           XtNvertDistance, 3,
 					  XtNfromHoriz, editButton,
 					  XtNtop, XtChainBottom,
@@ -1502,6 +1519,7 @@ FontSelect(Widget w, Widget paint)
     doneButton = XtVaCreateManagedWidget("done",
 					 commandWidgetClass, form2,
 					 XtNfromVert, arg->vport2,
+                                         XtNheight, Global.btn_height,	 
                                          XtNvertDistance, 3,
 					 XtNfromHoriz, applyButton,
 					 XtNtop, XtChainBottom,
@@ -1542,6 +1560,7 @@ FontSelect(Widget w, Widget paint)
 
     XtPopup(arg->shell, XtGrabNone);
     XFlush(dpy);
+    SetWMInputHint(dpy, XtWindow(arg->shell));
     XtSetMinSizeHints(arg->shell, 360, 480);
     StateSetBusyWatch(False);
     XtVaGetValues(arg->shell, XtNtitle, &ptr1, NULL);
