@@ -53,6 +53,11 @@ typedef struct {
     char *scriptfile;
 } LocalInfo;
 
+typedef struct {
+    unsigned char color[3];
+    int ratio;
+} BaseTransform;
+
 #define BUFSIZE 65536
 
 void *dl_filter = NULL;
@@ -66,7 +71,7 @@ void *dl_image = NULL;
 extern int file_isSpecialImage;
 extern int file_transparent;
 
-extern struct imageprocessinfo ImgProcessInfo;
+extern imageprocessinfo ImgProcessInfo;
 extern FILE * openTempFile(char **np);
 extern void removeTempFile(void);
 
@@ -75,7 +80,7 @@ extern void removeTempFile(void);
 #define DEG2RAD	(M_PI/180.0)
 
 #define ConvMatrixSize	3
-typedef float ConvMatrix[ConvMatrixSize * ConvMatrixSize];
+typedef double ConvMatrix[ConvMatrixSize * ConvMatrixSize];
 
 static int *histogram(Image * input);
 static LocalInfo *head;
@@ -122,14 +127,14 @@ static PaintMenuBar printMenuBar[] =
 };
 
 static Image *
-convolve(Image * input, float *mat, int n,
-	 unsigned char *basePixel, Boolean absFlag)
+convolve(Image * input, double *mat, int n,
+	 BaseTransform *base, Boolean absFlag)
 {
     int x, y, xx, yy, xv, yv;
-    float sum;
+    double sum;
     unsigned char *p;
     unsigned char *op;
-    float r, g, b;
+    double r, g, b;
     int ir, ig, ib;
     Image *output;
 
@@ -148,8 +153,8 @@ convolve(Image * input, float *mat, int n,
 	    sum += mat[xx + n * yy];
 	}
     }
-    if (sum <= 0)
-	sum = 1;
+
+    if (sum <= 0) sum = 1;
 
     for (y = xy; y < ye; y++) {
 	for (x = xy; x < xe; x++) {
@@ -173,9 +178,9 @@ convolve(Image * input, float *mat, int n,
 			    yv = hh - yv - 1;
 		    }
 		    p = ImagePixel(input, xv, yv);
-		    r += (float) *p++ * mat[xx + n * yy];
-		    g += (float) *p++ * mat[xx + n * yy];
-		    b += (float) *p * mat[xx + n * yy];
+		    r += (double) *p++ * mat[xx + n * yy];
+		    g += (double) *p++ * mat[xx + n * yy];
+		    b += (double) *p * mat[xx + n * yy];
 		}
 	    }
 	    if (absFlag) {
@@ -189,10 +194,12 @@ convolve(Image * input, float *mat, int n,
 	    ir = r / sum;
 	    ig = g / sum;
 	    ib = b / sum;
-	    if (basePixel) {
-		ir += basePixel[0];
-		ig += basePixel[1];
-		ib += basePixel[2];
+	    if (base) {
+	        /* linear rescaling */
+                double f = (double) base->ratio / 100.0;
+	        ir = f * ir + (int)base->color[0] + 0.5;
+	        ig = f * ig + (int)base->color[1] + 0.5;
+	        ib = f * ib + (int)base->color[2] + 0.5;
 	    }
 	    CLAMP(0, ir, 255);
 	    CLAMP(0, ig, 255);
@@ -218,8 +225,8 @@ normalize(Image * image)
 {
     int i, count;
     unsigned char *sp;
-    char *ip;
-    int maxval = 0;
+    unsigned char *ip;
+    unsigned int maxval = 0;
 
     if (image->cmapSize != 0) {
 	sp = image->cmapData;
@@ -229,18 +236,25 @@ normalize(Image * image)
 	count = image->width * image->height * 3;
     }
 
-    for (ip = (char*) sp, i = 0; i < count; i++, ip++)
+    for (ip = sp, i = 0; i < count; i++, ip++)
 	if (*ip > maxval)
 	    maxval = *ip;
     if (maxval == 0)
 	return;
-    for (ip = (char*) sp, i = 0; i < count; i++, ip++)
-	*ip = ((int) *ip * 255) / maxval;
+    for (ip = sp, i = 0; i < count; i++, ip++)
+        *ip = (((unsigned int)(*ip)) * 255) / maxval;
 }
 
 /*
 **  Convert image into a monochrome image
  */
+
+unsigned char 
+GreyScale(unsigned char r, unsigned char g, unsigned char b)
+{
+    return (30*r + 59*g + 11*b)/100;
+}
+
 static void 
 monochrome(Image * image)
 {
@@ -258,7 +272,7 @@ monochrome(Image * image)
     }
 
     for (ip = sp, i = 0; i < count; i++) {
-	v = (ip[0] * 11 + ip[1] * 16 + ip[2] * 5) >> 5;		/* pp = .33R+.5G+.17B */
+        v = GreyScale(ip[0], ip[1], ip[2]);
 	*ip++ = v;
 	*ip++ = v;
 	*ip++ = v;
@@ -269,7 +283,7 @@ monochrome(Image * image)
 Image *
 ImageSmooth(Image * input)
 {
-    float *mat;
+    double *mat;
     int n, i;
     Image *output;
 
@@ -277,7 +291,7 @@ ImageSmooth(Image * input)
      * Build n x n convolution matrix (all 1's)
      */
     n = ImgProcessInfo.smoothMaskSize;
-    mat = xmalloc(n * n * sizeof(float));
+    mat = xmalloc(n * n * sizeof(double));
     for (i = 0; i < n * n; ++i)
 	mat[i] = 1.0;
 
@@ -298,14 +312,97 @@ ImageSharpen(Image * input)
     return convolve(input, mat, ConvMatrixSize, NULL, False);
 }
 
+Image *
+ImageMerge(Image * input)
+{
+    Image *output, *canvas;
+    int width = input->width, height = input->height;
+    int x, y, x1, x2, y1, y2, alpha = 0;
+    unsigned char *ip, *cp, *op, *ia, *ca, *oa;
+    unsigned int p=0, q=0;
+    unsigned long omi=0;
+    double d, coef=0;
+
+    canvas = ImgProcessInfo.canvas;
+
+    if (ImgProcessInfo.mergeType == 0) {
+        d = (double)ImgProcessInfo.mergePercent / 10001.0;
+        d = 160000.0 * d/(1.0-d);
+        coef = (double) ((width-1) * (height-1));
+        coef = d/(coef * coef);
+    }
+    if (ImgProcessInfo.mergeType == 1) {
+        p = ImgProcessInfo.mergePercent;
+        q = 10000 - q;
+    }
+
+    output = ImageNew(width, height);
+    ImageCopyData(input, output);
+    if (input->alpha && canvas->alpha) {
+        alpha = width*height;
+        output->alpha = (unsigned char*) xmalloc(alpha);
+        memcpy(output->alpha, input->alpha, alpha);
+        omi = (unsigned long)output->alpha - (unsigned long)input->alpha;
+    }
+
+    x1 = MAX(0, -ImgProcessInfo.shiftX);
+    x2 = MIN(input->width, canvas->width-ImgProcessInfo.shiftX);
+    y1 = MAX(0, -ImgProcessInfo.shiftY);
+    y2 = MIN(input->height, canvas->height-ImgProcessInfo.shiftY);
+
+    for (y = y1; y < y2; y++) {
+        for (x = x1; x < x2; x++) {
+	    ip = ImagePixel(input, x, y);
+	    op = ImagePixel(output, x, y);            
+	    cp = ImagePixel(canvas, x+ImgProcessInfo.shiftX, y+ImgProcessInfo.shiftY);
+            if (ImgProcessInfo.mergeType <= 1) {
+	        if (ImgProcessInfo.mergeType == 0) {
+		    p =   (int) ( (double) (x * (width-1-x)) * 
+			          (double) (y * (height-1-y)) * coef );
+                    if (p > 10000) p = 10000;
+		    q = 10000 - p;
+		}
+                op[0] = (p * ip[0] + q * cp[0])/10000;
+                op[1] = (p * ip[1] + q * cp[1])/10000;
+                op[2] = (p * ip[2] + q * cp[2])/10000;
+	    } else {
+	        if (ImgProcessInfo.mergeType == 2) {
+		    op[0] = MAX(ip[0], cp[0]);
+		    op[1] = MAX(ip[1], cp[1]);
+		    op[2] = MAX(ip[2], cp[2]);
+	        } else {
+		    op[0] = MIN(ip[0], cp[0]);
+		    op[1] = MIN(ip[1], cp[1]);
+		    op[2] = MIN(ip[2], cp[2]);
+		}
+	    }
+            if (alpha) {
+	        ia = input->alpha + x + y * input->width;
+	        oa = input->alpha + omi;
+                ca = canvas->alpha + x + ImgProcessInfo.shiftX +
+		     (y+ImgProcessInfo.shiftY) * canvas->width;
+                if (ImgProcessInfo.mergeType <= 1)
+                    *oa = (p * *ia + q * *ca)/10000;
+                else
+                if (ImgProcessInfo.mergeType == 2)
+		    *oa = MAX(*ia, *ca);
+                else
+		    *oa = MIN(*ia, *ca);
+	    }
+        }
+    }
+
+    return output;
+}
+
 Image *
 ImageEdge(Image * input)
 {
     static ConvMatrix mat =
     {
-	-1, -2, 0,
-	-2, 0, 2,
-	0, 2, 1
+	-1, -2,  0,
+	-2,  0,  2,
+	 0,  2,  1
     };
     Image *image = convolve(input, mat, ConvMatrixSize, NULL, True);
 
@@ -319,16 +416,19 @@ ImageEmboss(Image * input)
 {
     static ConvMatrix mat =
     {
-	-1, -2, 0,
-	-2, 0, 2,
-	0, 2, 1
+	-1, -2,  0,
+	-2,  0,  2,
+	 0,  2,  1
     };
-    static unsigned char base[3] =
-    {128, 128, 128};
-    Image *image = convolve(input, mat, ConvMatrixSize, base, False);
+    static BaseTransform base;
+
+    Image *image;
+
+    memset(base.color, ImgProcessInfo.embossBG, 3);
+    base.ratio = ImgProcessInfo.embossRatio;
+    image = convolve(input, mat, ConvMatrixSize, &base, False);
 
     monochrome(image);
-    normalize(image);
 
     return image;
 }
@@ -338,7 +438,41 @@ ImageInvert(Image * input)
 {
     Image *output;
     unsigned char *op;
-    int x,y;
+
+#define MEMORY_SAVINGS 1
+#if MEMORY_SAVINGS
+    unsigned char *ip;
+    int i, count;
+
+    /*
+    **	 If the input has a colormap, just invert that.
+     */
+    if (input->cmapSize != 0) {
+	output = ImageNewCmap(input->width, input->height, input->cmapSize);
+	ip = input->cmapData;
+	op = output->cmapData;
+        output->isGrey = input->isGrey;
+        output->isBW = input->isBW;
+        count = input->cmapSize;
+        
+	memcpy(output->data, input->data,
+	     sizeof(char) * input->scale * input->width * input->height);
+
+    } else {
+	output = ImageNew(input->width, input->height);
+	ip = input->data;
+	op = output->data;
+	count = input->width * input->height;
+    }
+
+    for (i = 0; i < count; i++) {
+         *op++ = 255 - *ip++;
+	 *op++ = 255 - *ip++;
+	 *op++ = 255 - *ip++;
+    }
+
+#else
+    int x, y;
     unsigned char *p;
 
     output = ImageNew(input->width, input->height);
@@ -354,31 +488,6 @@ ImageInvert(Image * input)
 	    *op++ = 255 - *p;
 	}
     }
-
-#if 0
-    /*
-    **	 If the input has a colormap, just invert that.
-     */
-    if (input->scale == 1 && input->cmapSize != 0) {
-	output = ImageNewCmap(input->width, input->height, input->cmapSize);
-	ip = input->cmapData;
-	op = output->cmapData;
-	count = input->cmapSize;
-
-	memcpy(output->data, input->data,
-	     sizeof(char) * input->scale * input->width * input->height);
-    } else {
-	output = ImageNew(input->width, input->height);
-	ip = input->data;
-	op = output->data;
-	count = input->width * input->height;
-    }
-
-    for (i = 0; i < count; i++) {
-	*op++ = 255 - *ip++;
-	*op++ = 255 - *ip++;
-	*op++ = 255 - *ip++;
-    }
 #endif
 
     return output;
@@ -483,7 +592,7 @@ ImageAddNoise(Image * input)
 
 /*
  * This function works in-place.
- * Because it swaps pixels in the input image, which may be colour mapped
+ * Because it swaps pixels in the input image, which may be color mapped
  * or not, is has knowledge about the Image format that should probably
  * have stayed in image.h. Too bad.
  */
@@ -494,7 +603,6 @@ ImageSpread(Image * input)
     unsigned char *p, *np;
     int x, y, minx, miny, maxx, maxy, xn, yn, dist;
 
-
     dist = ImgProcessInfo.spreadDistance;
     for (y = 0; y < h; y++) {
 	for (x = 0; x < w; x++) {
@@ -561,7 +669,7 @@ ImageBlend(Image * input)
     unsigned char *rgb;
     int x, y, n, ox, oy, px, py, dx, dy;
     int rSum, gSum, bSum, r, g, b;
-    float diagonal, gradient, ap;
+    double diagonal, gradient, ap;
 
 
     ox = w / 2;
@@ -838,7 +946,7 @@ ImageDespeckle(Image * input)
  * Normalize the contrast of an image.
  */
 Image *
-ImageNormContrast(Image * input)
+ImageNormalizeContrast(Image * input)
 {
     unsigned char *p;
     int *hist;
@@ -867,9 +975,9 @@ ImageNormContrast(Image * input)
     /* Likewise for 'white' threshold. */
     cumsum = 0;
     limit = w * h * whitepcnt / 100;
-    for (i = 256; i > 0; --i) {
+    for (i = 255; i > 0; --i) {
 	if ((max == 0) && hist[i])
-	    max = i + 1;	/* max is index of highest non-zero entry */
+	    max = i;	/* max is index of highest non-zero entry */
 	if ((cumsum += hist[i]) > limit)
 	    break;
     }
@@ -938,7 +1046,7 @@ histogram(Image * input)
 	    r = *p++;
 	    g = *p++;
 	    b = *p;
-	    i = (r * 169 + g * 256 + b * 87) / 512;	/* i = .33R+.5G+.17B */
+	    i = GreyScale(r, g, b);
 	    ++hist[i];
 	}
     }
@@ -1020,10 +1128,10 @@ HistogramImage(char *hist)
  * to the overlapping areas.
  */
 static void 
-interpol(float x, float y, Image * input, unsigned char *op)
+interpolate(double x, double y, Image * input, unsigned char *op)
 {
     int xl, xh, yl, yh;
-    float a, b, c, d, A, B, C, D;
+    double a, b, c, d, A, B, C, D;
     unsigned char *plh, *phh, *pll, *phl;
 
     xl = floor(x);
@@ -1031,6 +1139,11 @@ interpol(float x, float y, Image * input, unsigned char *op)
     yl = floor(y);
     yh = ceil(y);
 
+    if (xl < 0) xl = 0;
+    if (yl < 0) yl = 0;
+    if (xh >= input->width) xh = input->width - 1;
+    if (yh >= input->height) yh = input->height - 1;
+
     pll = ImagePixel(input, xl, yl);
 
     if (xh == xl) {
@@ -1041,18 +1154,18 @@ interpol(float x, float y, Image * input, unsigned char *op)
 	    *op++ = *pll++;
 	} else {
 	    /* pixel overlaps with two pixels on top of each other */
-	    plh = ImagePixel(input, xl, yh);
 	    A = y - yl;
 	    B = yh - y;
+            plh = ImagePixel(input, xl, yh);
 	    *op++ = A * *plh++ + B * *pll++;
 	    *op++ = A * *plh++ + B * *pll++;
 	    *op++ = A * *plh++ + B * *pll++;
 	}
     } else if (yh == yl) {
 	/* pixel overlaps with two side-by-side pixels */
-	phl = ImagePixel(input, xh, yl);
 	A = x - xl;
-	B = xh - x;
+	B = xh - x;        
+	phl = ImagePixel(input, xh, yl);
 	*op++ = A * *phl++ + B * *pll++;
 	*op++ = A * *phl++ + B * *pll++;
 	*op++ = A * *phl++ + B * *pll++;
@@ -1076,24 +1189,82 @@ interpol(float x, float y, Image * input, unsigned char *op)
 }
 
 Image *
-ImageTilt(Image * input)
+ImageDistort(Image * input)
+{
+    int x, y, xi, yi;
+    int width = input->width, height = input->height;
+    Image *output;
+    unsigned char *p, *op;
+    double a, b, sx, sy, sx2, sy2, sp, w, h;
+
+    a = ImgProcessInfo.distortX;
+    b = ImgProcessInfo.distortY;
+    w = width - 1.0;
+    h = height - 1.0;
+
+    if (width<=1 || height<=1) return input;
+
+    output = ImageNew(width, height);
+
+    for (y = 0; y < height; y++) {
+        sy = (double)y / h;
+        sy2 = 1.0 - sy;
+        sy2 = sy*sy*sy2*sy2;
+	for (x = 0; x < width; x++) {
+	    sx = (double)x / w;
+            sx2 = 1.0 - sx;
+            sx2 = sx*sx*sx2*sx2;
+            sp = sx2 * sy2;
+            xi = (int) (w * (sx + a * sp) + 0.5);
+            CLAMP(0, xi, width-1);
+	    yi = (int) (h * (sy + b * sp) + 0.5);
+            CLAMP(0, yi, height-1);
+            p = ImagePixel(input, xi, yi);
+	    op = ImagePixel(output, x, y);
+	    *op++ = *p++;
+	    *op++ = *p++;
+	    *op = *p;
+	}
+    }
+
+    return output;
+}
+
+Image *
+ImageProjTransform(Image * input)
 {
     int x, y, br, bg, bb;
     int width = input->width, height = input->height;
     Image *output;
     unsigned char *op;
-    float xt, yt;
-    float X1, Y1, X2, Y2;
+    double xs, ys, xt, yt, d;
+    double X1, Y1, X2, Y2;
+    double a11, a12, a21, a22;
 
+    X1 = width * (ImgProcessInfo.projX1 / 100.0);
+    Y1 = height * (ImgProcessInfo.projY1 / 100.0);
 
-    X1 = width * ImgProcessInfo.tiltX1 / 100;
-    X2 = width * ImgProcessInfo.tiltX2 / 100;
+    X2 = width * (ImgProcessInfo.projX2 / 100.0) - X1;
+    Y2 = height * (ImgProcessInfo.projY2 / 100.0) - Y1;
 
-    Y1 = height * ImgProcessInfo.tiltY1 / 100;
-    Y2 = height * ImgProcessInfo.tiltY2 / 100;
+    d = X2*X2 + Y2*Y2;
 
-    if (((Y1 >= 0) && (Y1 < height)) || ((X2 >= 0) && (X2 < width)))
-	return input;
+    if (ImgProcessInfo.linearDX == 0 || ImgProcessInfo.linearDY == 0 || d == 0) 
+        return input;
+
+    X2 = X2/d;
+    Y2 = Y2/d;
+
+    d = ImgProcessInfo.linearAngle * M_PI / 180.0;
+    xs = cos(d);
+    ys = sin(d);
+    xt = 1.0 / ImgProcessInfo.linearDX;
+    yt = 1.0 / ImgProcessInfo.linearDY;
+    d = ImgProcessInfo.linearIncli;
+    a11 = xs * xt + ys * d;
+    a12 = xs * d - ys * xt;
+    a21 = ys * yt;
+    a22 = xs * yt;
 
     output = ImageNew(width, height);
 
@@ -1112,20 +1283,27 @@ ImageTilt(Image * input)
     for (y = 0; y < height; y++) {
 	for (x = 0; x < width; x++) {
 	    op = ImagePixel(output, x, y);
-	    /* find the input coords corresponding to output coords (x,y) */
-	    xt = (X1 * y - Y1 * x) / (y - Y1);
-	    yt = (-X2 * y + x * Y2) / (x - X2);
-	    if ((xt >= 0) && (xt < width) && (yt >= 0) && (yt < height)) {
+	    /* find the input coords corresponding to output coords (x,y) 
+               (X1,Y1) fixed point, (X2,Y2) sent to infinity */
+            xs = x - X1;
+            ys = y - Y1;
+            d =  1.0 - (X2 * xs + Y2 * ys);
+            if (fabs(d)> 1E-5) {
+	        xs = xs / d;
+	        ys = ys / d;
+                xt = a11 * xs + a12 * ys + X1;
+                yt = a21 * xs + a22 * ys + Y1;
+	        if ((xt > -1) && (xt < width) && (yt > -1.0) && (yt < height)) {
 #if 0
-		unsigned char *p;
-		/* rounding */
-		p = ImagePixel(input, (int) (xt + 0.5), (int) (yt + 0.5));
-		*op++ = *p++;
-		*op++ = *p++;
-		*op = *p;
+		    unsigned char *p;
+		    p = ImagePixel(input, xp, yp);
+		    *op++ = *p++;
+		    *op++ = *p++;
+		    *op = *p;
 #else
-		interpol(xt, yt, input, op);
+		    interpolate(xt, yt, input, op);
 #endif
+		}
 	    }
 	}
     }
@@ -1283,6 +1461,261 @@ nearest_color(unsigned char *p)
     return (*cache = nearest);
 }
 
+void 
+GammaScale(unsigned char * scale, double f)
+{
+    int c;
+    scale[0] = 0;
+    scale[255] = 255;
+    if (f > 12) {
+        for (c = 1; c <= 254; c++) scale[c] = 0;
+        return;
+    }
+    if (f < -12) {
+        for (c = 1; c <= 254; c++) scale[c] = 255;
+        return;
+    }
+    /* logarithmic gamma scale */
+    f = pow(2.0, -f);
+    for (c = 1; c <= 254; c++)
+        scale[c] = (int)(255.5 * pow(c/255.0, f));
+}
+
+Image *
+ImageGammaCorrection(Image * input)
+{
+    int w = input->width, h = input->height;
+    Image *output;
+    unsigned char scale[768];
+    unsigned char *ip, *op;
+    int c, nc, x, y, i, j;
+    double a;
+    double *coef = (double *)ImgProcessInfo.rgb_mat;
+
+    /* find out whether output will be grey image */
+    if (ImgProcessInfo.RGB_correction >= 3 ||
+        (ImgProcessInfo.RGB_correction == 0 && input->isGrey) ||
+        (ImgProcessInfo.RGB_correction == -1 && input->isGrey) ||
+        (ImgProcessInfo.RGB_correction == 1 && input->isGrey &&
+         ImgProcessInfo.r_gamma == ImgProcessInfo.g_gamma &&
+	 ImgProcessInfo.g_gamma == ImgProcessInfo.b_gamma)) {
+        output = ImageNewGrey(w, h);
+        nc = 1;
+    } else {
+        output = ImageNew(w, h);
+        nc = 3;
+    }
+
+    if (ImgProcessInfo.RGB_correction == 0) {
+        /* threshold */
+        a = 255.0 / (double)(ImgProcessInfo.RGB_max-ImgProcessInfo.RGB_min);
+        for (y = 0; y < h; y++) {
+            op = output->data + nc*w*y;
+	    for (x = 0; x < w; x++) {
+                ip = ImagePixel(input, x, y);
+                for (i=0; i<nc; i++) {
+		    c = (int) (((int)(*ip++)-ImgProcessInfo.RGB_min) * a + 0.5);
+                    if (c < 0) c = 0;
+                    if (c > 255) c = 255;
+                    *op++ = c;
+		}
+	    }
+	}
+        return output;
+    }
+
+    if (ImgProcessInfo.RGB_correction == 1 ||
+        ImgProcessInfo.RGB_correction == -1) {
+        /* gamma correction */
+        GammaScale(scale, ImgProcessInfo.r_gamma);
+        if (nc == 3) {
+            if (ImgProcessInfo.RGB_correction == -1) {
+	        memcpy(scale+256, scale, 256);
+	        memcpy(scale+512, scale, 256);
+	    } else {
+                GammaScale(scale+256, ImgProcessInfo.g_gamma);
+                GammaScale(scale+512, ImgProcessInfo.b_gamma);
+	    }
+        }
+        for (y = 0; y < h; y++) {
+            op = output->data + nc*w*y;
+            if (nc == 1)
+	        for (x = 0; x < w; x++) {
+                    ip = ImagePixel(input, x, y);
+	            *op++ = scale[*ip++];
+	        }
+            else
+  	        for (x = 0; x < w; x++) {
+                    ip = ImagePixel(input, x, y);
+ 	            for (c=0; c<nc; c++) 
+	                *op++ = scale[(c<<8) + *ip++];
+	        }
+        }
+        return output;
+    }
+
+    if (ImgProcessInfo.RGB_correction == 2) {
+        /* linear correction */
+        for (y = 0; y < h; y++) {
+	    /* This is a color image then */
+            op = output->data + 3*w*y;
+	    for (x = 0; x < w; x++) {
+                ip = ImagePixel(input, x, y);
+                for (i=0; i<=2; i++) {
+		    j = i<<2;
+		    a = (ip[0]*coef[j]+ip[1]*coef[j+1]+ip[2]*coef[j+2])/100.0
+                        + coef[j+3] + 0.5; 
+                    c = (int) a;
+                    if (c < 0) c = 0;
+                    if (c > 255) c = 255;
+                    *op++ = c;
+		}
+	    }
+	}
+        return output;
+    }
+
+    if (ImgProcessInfo.RGB_correction >= 3 &&
+        ImgProcessInfo.RGB_correction <= 5) {
+        /* R,G,B transform */
+        c = ImgProcessInfo.RGB_correction - 3;
+        for (y = 0; y < h; y++) {
+            op = output->data + w*y;
+	    for (x = 0; x < w; x++) {
+                ip = ImagePixel(input, x, y);
+	        *op++ = ip[c];
+	    }
+	}
+        output->isGrey = False;
+        memset(output->cmapData, 0, 3*256);
+        for (x = 0; x < 256; x++)
+	    output->cmapData[3*x+c] = x;
+        return output;
+    }
+
+    if (ImgProcessInfo.RGB_correction == 6) {
+        /* maximum intensity */
+        for (y = 0; y < h; y++) {
+            op = output->data + w*y;
+	    for (x = 0; x < w; x++) {
+                ip = ImagePixel(input, x, y);
+	        *op++ = MAX(MAX(ip[0], ip[1]), ip[2]);
+	    }
+	}
+        return output;
+    }
+
+    if (ImgProcessInfo.RGB_correction == 7) {
+        /* minimum intensity */
+        for (y = 0; y < h; y++) {
+            op = output->data + w*y;
+	    for (x = 0; x < w; x++) {
+                ip = ImagePixel(input, x, y);
+	        *op++ = MIN(MIN(ip[0], ip[1]), ip[2]);
+	    }
+	}
+        return output;
+    }
+
+    if (ImgProcessInfo.RGB_correction == 8) {
+        /* find mask with respect to background */
+        for (y = 0; y < h; y++) {
+            op = output->data + w*y;
+	    for (x = 0; x < w; x++) {
+                ip = ImagePixel(input, x, y);
+                if (memcmp(ip, Global.bg, 3))
+                    *op++ = 0xff; 
+                else 
+                    *op++ = 0;
+	    }
+	}
+        return output;
+    }
+
+    return NULL;
+}
+
+Image *
+ImageFloydSteinberg(Image * input)
+{
+    int w = input->width, h = input->height;
+    Image *output;
+    int d, wp, c, nc, nd, p, x, y, error = 0;
+    unsigned char *op;
+    unsigned char subst[256];
+    unsigned char scale[256];
+
+    if (input->isGrey) {
+        nc = 1;
+        output = ImageNewGrey(w, h);
+    } else {
+        nc = 3;
+        output = ImageNew(w, h);
+    }
+    wp = nc * w;
+
+    ImageCopyData(input, output);
+
+    /* calculate substitution values */
+    x = ImgProcessInfo.FSsteps - 1;
+    p = ImgProcessInfo.FSmidrange - 1;
+    GammaScale(scale, ImgProcessInfo.FSgamma);
+
+    for (c = 0; c <= 255; c++) {
+        y = (c * x + p) / 255;
+        subst[c] = (unsigned char)((255 * y  + x/2) / x);
+    }
+
+    for (c = 0; c < nc; c++)
+    for (y = 0; y < h; y++) {
+        d = 1 - ((y&1)<<1);
+        nd = d*nc;
+        op = output->data + wp*y + c;
+        if (d == -1)
+            op += wp - nc;
+        for (x = 0; x < w; x++) {
+	    p = (int)(scale[*op]);
+            *op = subst[p];
+            error = p - (int)(*op);
+            if (!error) goto fast_done;
+            
+	    /*  error diffusion */
+            if (x < w-1) {
+	        /* Right pixel */
+	        p = (int) op[nd] + (int) (error * 7/16);
+	        if (p < 0) p = 0;
+	        if (p > 255) p = 255;
+                op[nd] = p;
+	    }
+            if (y < h-1) {
+	        if (x > 0) {
+		    /* Bottom left pixel */
+		    p = (int) op[wp-nd] + (int) (error * 3/16);
+	            if (p < 0) p = 0;
+	            if (p > 255) p = 255;
+		    op[wp-nd] = p;
+		}
+	        /* Bottom pixel */
+	        p = (int) op[wp] + (int) (error * 5/16);
+	        if (p < 0) p = 0;
+	        if (p > 255) p = 255;
+	        op[wp] = p;
+	        if (x < w-1) {
+	            /* Bottom right pixel */
+	            p = (int) op[wp+nd] + (int) (error / 16);
+	            if (p < 0) p = 0;
+	            if (p > 255) p = 255;
+	            op[wp+nd] = p;
+		}
+	    }
+	  fast_done:
+            op += nd;
+	}
+    }
+
+    return output;
+}
+
 Image *
 ImageQuantize(Image * input)
 {
@@ -1458,20 +1891,18 @@ ImageQuantize(Image * input)
 /*
  * Convert an image to grey scale.
  */
+
 Image *
 ImageGrey(Image * input)
 {
-    Image *output = ImageNew(input->width, input->height);
+    Image *output = ImageNewGrey(input->width, input->height);
     unsigned char *p, *op = output->data;
     int x, y, v;
 
-
     for (y = 0; y < input->height; y++) {
 	for (x = 0; x < input->width; x++) {
 	    p = ImagePixel(input, x, y);
-	    v = (32*p[0]+50*p[1]+18*p[2])/100;	/* 0.32R + 0.5G + 0.18B */
-	    *op++ = v;
-	    *op++ = v;
+	    v = GreyScale(p[0], p[1], p[2]);
 	    *op++ = v;
 	}
     }
@@ -1481,7 +1912,7 @@ ImageGrey(Image * input)
 Image *
 ImageBWMask(Image * input)
 {
-    Image *output = ImageNew(input->width, input->height);
+    Image *output = ImageNewGrey(input->width, input->height);
     unsigned char *p, *op = output->data;
     int x, y, v;
 
@@ -1490,53 +1921,11 @@ ImageBWMask(Image * input)
 	    p = ImagePixel(input, x, y);
             if (memcmp(p, Global.bg, 3)) v = 0xff; else v = 0;
 	    *op++ = v;
-	    *op++ = v;
-	    *op++ = v;
 	}
     }
     return output;
 }
 
-/*
- * Modify RGB values of an image.
- */
-Image *
-ImageModifyRGB(Image * input)
-{
-    Image *output;
-    unsigned char *p, *op;
-    int x, y;
-    double fr, fg, fb, frp, fgp, fbp;
-
-    if (!ImgProcessInfo.redshift &&
-        !ImgProcessInfo.greenshift &&
-        !ImgProcessInfo.blueshift) return input;
-  
-    output = ImageNew(input->width, input->height);
-    op = output->data;
-
-    fr = 2.0*ImgProcessInfo.redshift / (100.01-ImgProcessInfo.redshift);
-    fg = 2.0*ImgProcessInfo.greenshift / (100.01-ImgProcessInfo.greenshift);
-    fb = 2.0*ImgProcessInfo.blueshift / (100.01-ImgProcessInfo.blueshift);
-    frp = fr / 255.0;
-    fgp = fg / 255.0;
-    fbp = fb / 255.0;
-    fr = fr + 1.0;
-    fg = fg + 1.0;
-    fb = fb + 1.0;
-
-    for (y = 0; y < input->height; y++) {
-	for (x = 0; x < input->width; x++) {
-	    p = ImagePixel(input, x, y);
-	    *op++ = (char) ( 0.25 + p[0] * fr / (1.0 + p[0] * frp) );
-	    *op++ = (char) ( 0.25 + p[1] * fg / (1.0 + p[1] * fgp) );
-	    *op++ = (char) ( 0.25 + p[2] * fb / (1.0 + p[2] * fbp) );
-	}
-    }
-
-    return output;
-}
-
 /*
  * Pipe command routines
  */
@@ -1607,7 +1996,7 @@ void pipe_command(command, input, output, error)
                 exit(1);
             }
         }
-                  system(command);
+	if(system(command));
         exit(1);        
     }
     else
@@ -1824,6 +2213,7 @@ compileCallback(Widget w, XtPointer wlArg, XtPointer junk)
 	 cmap = XCreateColormap(XtDisplay(Global.toplevel), 
 		    RootWindowOfScreen(screen), visual, AllocNone);
          alpha = image->alpha;
+         image->alpha =  NULL;
 	 ImageToPixmap(image, Global.toplevel, &pix, &cmap);
       }
       if (!pix) {
@@ -1837,7 +2227,6 @@ compileCallback(Widget w, XtPointer wlArg, XtPointer junk)
       paint = graphicCreate(makeGraphicShell(Global.toplevel), 0, 0, -1, 
                             pix, cmap, alpha);
       SetAlphaMode((Widget)paint, -1);
-      free(alpha);
       return;
    }
 
@@ -1887,7 +2276,7 @@ externCallback(Widget w, XtPointer wlArg, XtPointer junk)
 
    /* call external editor */
    sprintf(buf, "%s %s", EDITOR, name);
-   system(buf);
+   if(system(buf));
 
    /* Now copy edited file into buffer */
    fd = fopen(name, "r");
@@ -2036,12 +2425,18 @@ editorResized(Widget w, LocalInfo * l, XConfigureEvent * event, Boolean * flg)
     Dimension width1, height1, height2;
 
     XtVaGetValues(l->shell, XtNwidth, &width, XtNheight, &height, NULL);
-    XtResizeWidget(XtParent(l->name), width, 38, BORDERWIDTH);
+    XtResizeWidget(XtParent(l->name), width, Global.btn_height + 16, BORDERWIDTH);
     XtResizeWidget(l->pane, width, height, BORDERWIDTH);
     XtVaSetValues(l->pane, XtNwidth, width, XtNheight, height, NULL);
     XtVaGetValues(XtParent(l->program), XtNheight, &height1, NULL);
 
-#ifdef XAW3DG   
+#ifdef XAW3DXFT
+    width1 = (width>13)? width-11 : 2;
+    height1 = (height1>10)? height1-8 : 2;
+    height2 = (height>height1+63)? 
+               height - height1 - Global.btn_height - 32 : 10;
+#else
+#ifdef XAW3DG
     width1 = (width>12)? width-10 : 2;
     height1 = (height1>10)? height1-8 : 2;
     height2 = (height>height1+63)? height - height1 - 53 : 10;
@@ -2049,23 +2444,24 @@ editorResized(Widget w, LocalInfo * l, XConfigureEvent * event, Boolean * flg)
     width1 = (width>10)? width-8 : 2;
     height1 = (height1>10)? height1-8 : 2;
     height2 = (height>height1+60)? height - height1 - 50 : 10;
-#endif   
-    XtResizeWidget(XtParent(l->program), width1+8, height1+8, BORDERWIDTH);
+#endif
+#endif
+    XtResizeWidget(XtParent(l->program), width1+11, height1+8, BORDERWIDTH);
     XtResizeWidget(l->program, width1, height1, BORDERWIDTH);
     XtVaSetValues(XtParent(l->program), 
-       XtNwidth, width1+8, XtNheight, height1+8, NULL);
+       XtNwidth, width1+11, XtNheight, height1+8, NULL);
     XtVaSetValues(l->program, 
        XtNwidth, width1, XtNheight, height1, NULL);
-    XtMoveWidget(XtParent(l->program), 0, 39);
+    XtMoveWidget(XtParent(l->program), 0, Global.btn_height + 16);
     XtMoveWidget(l->program, 4, 0);
 
-    XtResizeWidget(XtParent(l->cclog), width1+8, height2+8, BORDERWIDTH);
+    XtResizeWidget(XtParent(l->cclog), width1+11, height2+8, BORDERWIDTH);
     XtResizeWidget(l->cclog, width1, height2, BORDERWIDTH);
     XtVaSetValues(XtParent(l->cclog), 
-       XtNwidth, width1+8, XtNheight, height2+8, NULL);
+       XtNwidth, width1+11, XtNheight, height2+8, NULL);
     XtVaSetValues(l->cclog, 
        XtNwidth, width1, XtNheight, height2, NULL);
-    XtMoveWidget(XtParent(l->cclog), 0, 46+height1);
+    XtMoveWidget(XtParent(l->cclog), 0, Global.btn_height + 25 + height1);
     XtMoveWidget(l->cclog, 4, 0);
 }
 
@@ -2104,7 +2500,7 @@ ScriptEditor(Widget w, Widget paint)
                                   topLevelShellWidgetClass,
 				  Global.toplevel, args, nargs);
     info->pane = XtVaCreateManagedWidget("pane", panedWidgetClass, info->shell,
-                                   NULL);
+                                         NULL);
     topform = XtVaCreateManagedWidget("topform", formWidgetClass, info->pane, 
                                       NULL);
 
@@ -2116,18 +2512,20 @@ ScriptEditor(Widget w, Widget paint)
 				    XtNfromHoriz, bar,
 				    XtNhorizDistance, 32,
 				    XtNvertDistance, 5,
+				    XtNheight, Global.btn_height,	      
 				    NULL);
     cancel = XtVaCreateManagedWidget("close",
 		 		    commandWidgetClass, topform,
 				    XtNfromHoriz, compile,
 				    XtNhorizDistance, 4,
 				    XtNvertDistance, 5,
+				    XtNheight, Global.btn_height,	     
 				    NULL);
 
     info->name = XtVaCreateManagedWidget("name",
 				    labelWidgetClass, topform,
 				    XtNlabel, "",
-				    XtNwidth, 350,
+				    XtNwidth, 400,
 				    XtNfromHoriz, cancel,
                                     XtNhorizDistance, 24,
                                     XtNvertDistance, 6,
@@ -2143,8 +2541,8 @@ ScriptEditor(Widget w, Widget paint)
           XtVaCreateManagedWidget("program", asciiTextWidgetClass, midform,
                                   XtNeditType, XawtextEdit,
                                   XtNresizable, True,
-                                  XtNwidth, 620,
-                                  XtNheight, 300,
+                                  XtNwidth, 720,
+                                  XtNheight, 500,
 				  XtNstring, NULL,
 				  XtNscrollHorizontal, XawtextScrollWhenNeeded,
 				  XtNscrollVertical, XawtextScrollWhenNeeded,
@@ -2159,8 +2557,8 @@ ScriptEditor(Widget w, Widget paint)
           XtVaCreateManagedWidget("cclog", asciiTextWidgetClass, botform,
                                   XtNeditType, XawtextRead,
                                   XtNresizable, True,
-                                  XtNwidth, 620,
-                                  XtNheight, 100,
+                                  XtNwidth, 720,
+                                  XtNheight, 150,
 				  XtNstring, NULL,
 				  XtNscrollHorizontal, XawtextScrollWhenNeeded,
 				  XtNscrollVertical, XawtextScrollWhenNeeded,
@@ -2195,6 +2593,7 @@ ScriptEditor(Widget w, Widget paint)
 
     /* XtRealizeWidget(info->shell); */
     XtPopup(info->shell, XtGrabNone);
+    SetWMInputHint(dpy, XtWindow(info->shell));
     XtVaSetValues(topform, XtNshowGrip, False, NULL);
     XtVaSetValues(midform, XtNshowGrip, True, NULL);
     XtVaSetValues(botform, XtNshowGrip, True, NULL);
