@@ -15,7 +15,7 @@
 
 /* $Id: main.c,v 1.17 2005/03/20 20:15:32 demailly Exp $ */
 
-#define DEFAULT_GEOMETRY "640x480"
+#define DEFAULT_GEOMETRY "1024x768"
 
 #ifdef __VMS
 #define XtDisplay XTDISPLAY
@@ -39,6 +39,10 @@
 #include <X11/xpm.h>
 #include <X11/Xft/Xft.h>
 
+#ifdef XAW3DXFT
+#include "xaw_incdir/Xaw3dXft.h"
+#endif
+
 #define DEFINE_GLOBAL
 #include "xpaint.h"
 #include "Paint.h"
@@ -48,7 +52,7 @@
 #include "messages.h"
 #include "protocol.h"
 #include "image.h"
-#include "rw/rwTable.h"
+#include "xpaintrw/rwTable.h"
 
 #include "XPaintIcon.xpm"
 
@@ -57,23 +61,22 @@ int helpNum;
 String *msgText;
 int msgNum;
 
-char lang[10];
+char lang[32];
 char *routine;
 
+#ifdef XAW3DXFT
+Xaw3dXftData *xaw3dxft_data = NULL;
+Xaw3dXftProc *xaw3dxft_proc = NULL;
+#endif
+
 extern int magnifier_closing_down;
 extern int file_specified_zoom;
+extern int file_JPEG_quality;
 extern char *fontNames[];
 
 extern void BrushInit(Widget toplevel);
 extern int TestScriptC(char *file);
 extern Image * ReadScriptC(char *file);
-#ifdef XAW3DXFT
-extern void SetMenuSpacing(int value);
-extern void SetXftEncoding(int value);
-extern void SetXftDefaultFontName(char *name);
-extern void SetXftInsensitiveTwist(char *value);
-extern void SetXawHilitColor(char *value);
-#endif
 
 static char *appDefaults[] =
 {
@@ -90,7 +93,7 @@ typedef struct {
     String spacing;
     String twistcolor;
     String hilitcolor;
-    String size;
+    String canvassize;
     String winsize;
     String help;
     String shareDir;   
@@ -102,13 +105,16 @@ typedef struct {
     String proc;
     String zoom;
     int undosize;
+    int btnheight;
     int operation;
+    int quality;
+    int toolmode;
     Boolean fullmenu;
     Boolean menubar;
+    Boolean tooltips;
     Boolean canvas;
     Boolean screenshot;
     Boolean magnifier;
-    Boolean horizontal;
     Boolean nowarn;
     Boolean astext;
 } AppInfo;
@@ -133,8 +139,8 @@ static XtResource resources[] =
      XtOffset(AppInfo *, twistcolor), XtRImmediate, (XtPointer) NULL},
     {"hilitcolor", "HilitColor", XtRString, sizeof(String),
      XtOffset(AppInfo *, hilitcolor), XtRImmediate, (XtPointer) NULL},
-    {"size", "Size", XtRString, sizeof(String),
-     XtOffset(AppInfo *, size), XtRImmediate, (XtPointer) DEFAULT_GEOMETRY},
+    {"canvassize", "CanvasSize", XtRString, sizeof(String),
+     XtOffset(AppInfo *, canvassize), XtRImmediate, (XtPointer) DEFAULT_GEOMETRY},
     {"winsize", "WinSize", XtRString, sizeof(String),
      XtOffset(AppInfo *, winsize), XtRImmediate, (XtPointer) "0x0"},
     {"shareDir", "ShareDir", XtRString, sizeof(String),
@@ -157,20 +163,26 @@ static XtResource resources[] =
      XtOffset(AppInfo *, zoom), XtRImmediate, (XtPointer) NULL},
     {"undosize", "UndoSize", XtRInt, sizeof(int),
      XtOffset(AppInfo *, undosize), XtRImmediate, (XtPointer) 4},
+    {"btnheight", "BtnHeight", XtRInt, sizeof(int),
+     XtOffset(AppInfo *, btnheight), XtRImmediate, (XtPointer) 4},
     {"operation", "Operation", XtRInt, sizeof(int),
      XtOffset(AppInfo *, operation), XtRImmediate, (XtPointer) 1},
+    {"quality", "Quality", XtRInt, sizeof(int),
+     XtOffset(AppInfo *, quality), XtRImmediate, (XtPointer) 90},
+    {"toolmode", "Toolmode", XtRInt, sizeof(int),
+     XtOffset(AppInfo *, toolmode), XtRImmediate, (XtPointer) 0},
     {"fullmenu", "FullMenu", XtRBoolean, sizeof(Boolean),
      XtOffset(AppInfo *, fullmenu), XtRImmediate, (XtPointer) False},
     {"menubar", "Menubar", XtRBoolean, sizeof(Boolean),
      XtOffset(AppInfo *, menubar), XtRImmediate, (XtPointer) True},
+    {"tooltips", "Tooltips", XtRBoolean, sizeof(Boolean),
+     XtOffset(AppInfo *, tooltips), XtRImmediate, (XtPointer) True},
     {"canvas", "Canvas", XtRBoolean, sizeof(Boolean),
      XtOffset(AppInfo *, canvas), XtRImmediate, (XtPointer) False},
     {"screenshot", "Screenshot", XtRBoolean, sizeof(Boolean),
      XtOffset(AppInfo *, screenshot), XtRImmediate, (XtPointer) False},
     {"magnifier", "Magnifier", XtRBoolean, sizeof(Boolean),
      XtOffset(AppInfo *, magnifier), XtRImmediate, (XtPointer) False},
-    {"horizontal", "Horizontal", XtRBoolean, sizeof(Boolean),
-     XtOffset(AppInfo *, horizontal), XtRImmediate, (XtPointer) False},
     {"nowarn", "NoWarn", XtRBoolean, sizeof(Boolean),
      XtOffset(AppInfo *, nowarn), XtRImmediate, (XtPointer) False},
     {"astext", "AsText", XtRBoolean, sizeof(Boolean),
@@ -188,7 +200,7 @@ static XrmOptionDescRec options[] =
     {"-spacing", ".spacing", XrmoptionSepArg, (XtPointer) NULL},
     {"-twistcolor", ".twistcolor", XrmoptionSepArg, (XtPointer) NULL},
     {"-hilitcolor", ".hilitcolor", XrmoptionSepArg, (XtPointer) NULL},
-    {"-size", ".size", XrmoptionSepArg, (XtPointer) NULL},
+    {"-canvassize", ".canvassize", XrmoptionSepArg, (XtPointer) NULL},
     {"-winsize", ".winsize", XrmoptionSepArg, (XtPointer) "0x0"},
     {"-sharedir", ".shareDir", XrmoptionSepArg, (XtPointer) SHAREDIR},   
     {"-rcfile", ".rcFile", XrmoptionSepArg, (XtPointer) NULL},
@@ -197,8 +209,11 @@ static XrmOptionDescRec options[] =
     {"-filter", ".filter", XrmoptionSepArg, (XtPointer) NULL},
     {"-proc", ".proc", XrmoptionSepArg, (XtPointer) NULL},
     {"-zoom", ".zoom", XrmoptionSepArg, (XtPointer) "1"},
-    {"-undosize", ".undosize", XrmoptionSepArg, (XtPointer) "2"},
+    {"-undo", ".undosize", XrmoptionSepArg, (XtPointer) "4"},
+    {"-btnheight", ".btnheight", XrmoptionSepArg, (XtPointer) "4"},
     {"-operation", ".operation", XrmoptionSepArg, (XtPointer) "1"},
+    {"-quality", ".quality", XrmoptionSepArg, (XtPointer) "90"},
+    {"-toolmode", ".toolmode", XrmoptionSepArg, (XtPointer) "0"},    
     {"-fullmenu", ".fullmenu", XrmoptionNoArg, (XtPointer) "True"},
     {"-simplemenu", ".fullmenu", XrmoptionNoArg, (XtPointer) "False"},
     {"-menubar", ".menubar", XrmoptionNoArg, (XtPointer) "True"},
@@ -206,11 +221,11 @@ static XrmOptionDescRec options[] =
     {"-canvas", ".canvas", XrmoptionNoArg, (XtPointer) "True"},
     {"-screenshot", ".screenshot", XrmoptionNoArg, (XtPointer) "True"},
     {"-magnifier", ".magnifier", XrmoptionNoArg, (XtPointer) "True"},
-    {"-horizontal", ".horizontal", XrmoptionNoArg, (XtPointer) "True"},
     {"-nowarn", ".nowarn", XrmoptionNoArg, (XtPointer) "True"},
     {"-astext", ".astext", XrmoptionNoArg, (XtPointer) "True"},
     {"-help", ".help", XrmoptionNoArg, (XtPointer) "command"},
-    {"+help", ".help", XrmoptionSepArg, (XtPointer) NULL},
+    {"-tooltips", ".tooltips", XrmoptionNoArg, (XtPointer) "True"},
+    {"-notooltips", ".tooltips", XrmoptionNoArg, (XtPointer) "False"},
     {"-8", ".visualType", XrmoptionNoArg, (XtPointer) "cmap8"},
     {"-visual", ".visualType", XrmoptionSepArg, (XtPointer) NULL},
     {"-dpi", ".dpi", XrmoptionSepArg, (XtPointer) NULL}
@@ -228,6 +243,12 @@ GetDefaultUndosize()
     return appInfo.undosize;
 }
 
+Boolean
+TooltipsCmdOption()
+{
+    return (Boolean)appInfo.tooltips;
+}
+
 void 
 SetDefaultWHZ(int w, int h, int zoom)
 {
@@ -314,12 +335,6 @@ IsFullMenuGlobal()
     return appInfo.fullmenu;
 }
 
-Boolean
-ToolsAreHorizontal()
-{
-    return appInfo.horizontal;
-}
-
 char *
 GetDefaultRC(void)
 {
@@ -553,11 +568,12 @@ LoadMessages()
 	         *  which we were compiled.  Note that it need not necessarily
 	         *  match XPAINT_VERSION, however. */
 	        if (!strncmp(buf, msgs_ver_str, msgs_ver_str_len)) {
-	            char *ptr = buf + msgs_ver_str_len;
+		  char *ptr = buf + msgs_ver_str_len, *ptrp;
 	            int last = strlen(ptr) - 1;
 
 	            if    (last >= 0 && ptr[last] == '\n')  ptr[last--] = '\0';
 	            while (last >= 0 && isspace(ptr[last])) ptr[last--] = '\0';
+                    if ((ptrp = strrchr(ptr, ','))) *ptrp = '\0';
 	            if (strcmp(ptr, MESSAGES_VERSION)) {
 	                fprintf(stderr, "Error: %s is version %s; expected "
 	                  "version %s\n", namebuf, ptr, MESSAGES_VERSION);
@@ -615,16 +631,22 @@ String GetAppDefaultFile()
     int i;
     FILE *fd = NULL;
 
-    if (!*lang && (lg = getenv("LANG"))) strncpy(lang, lg, 8);
+    if (!*lang && (lg = getenv("LANG"))) {
+        strncpy(lang, lg, 31);
+        lang[31] = '\0';
+    }
     if (*lang) {
-        for (i=0; i<=8 ; i++) lang[i] = tolower(lang[i]);
+        for (i=0; lang[i]; i++) lang[i] = tolower(lang[i]);
     }
     else
-        strcpy(lang, "en");
+        strcpy(lang, "en"); /* "en_UK.utf8" */
     sprintf(name, "%s/XPaint_%s", XAPPLOADDIR, lang);
     fd = fopen(name, "r");
-    if (*lang && !fd) {
-        lang[2] = '\0';
+    if (!fd && *lang) {
+        if (lang[2] == '_')
+            lang[2] = '\0';
+        else
+            lang[3] = '\0';
         sprintf(name, "%s/XPaint_%s", XAPPLOADDIR, lang);
 	fd = fopen(name, "r");
     }   
@@ -645,7 +667,7 @@ checkRCsize()
     char *ptr;
     int w, h, i;
 
-    if (!strcmp(appInfo.size, DEFAULT_GEOMETRY)) {
+    if (!strcmp(appInfo.canvassize, DEFAULT_GEOMETRY)) {
         if (appInfo.rcFile)
 	    strcpy(buf, appInfo.rcFile);
 	else
@@ -678,6 +700,7 @@ main(int argc, char *argv[])
     char xpTitle[20];
     int i, j;
     XEvent event;
+    XColor xc;
     Arg args[5];
     int nargs = 0;
     XrmDatabase rdb;
@@ -696,7 +719,6 @@ main(int argc, char *argv[])
     String app_def_file;
     int desiredVisualID = -1;	 
     
-    XftInitFtLibrary();
     InitTypeConverters();
 
     msgText = NULL;
@@ -733,14 +755,31 @@ main(int argc, char *argv[])
 			      resources, XtNumber(resources), NULL, 0);
 
 #ifdef XAW3DXFT
-    /* set UTF8 (code 0) as default encoding */
-    SetXftEncoding((appInfo.encoding)? atoi(appInfo.encoding) : 0);
-    /* set default Xft police = Liberation-9 point if none defined */
-    SetXftDefaultFontName((appInfo.menufont)? appInfo.menufont:"Liberation-9");
-    SetMenuSpacing((appInfo.spacing)? atoi(appInfo.spacing):1);
-    SetXftInsensitiveTwist((appInfo.twistcolor)? appInfo.twistcolor:"#a00000");
-    SetXawHilitColor((appInfo.hilitcolor)? appInfo.hilitcolor:"#332211");
+    /* retrieve addres of specific Xaw3dXftData (if any) */
+    GET_XAW3DXFT_DATA(xaw3dxft_data);
+    if (xaw3dxft_data) {
+        /* set UTF8 (code -1) as default encoding, unless otherwise specified */
+        xaw3dxft_data->encoding = (appInfo.encoding)? atoi(appInfo.encoding):-1;
+        xaw3dxft_data->string_hilight = 0;
+        xaw3dxft_data->string_use_pixmap = 0;
+        xaw3dxft_data->no_hilit_reverse = 1;
+        xaw3dxft_data->button_inverse = 0;
+        xaw3dxft_data->button_dashed = 1;
+        xaw3dxft_data->show_tips = 1;
+        xaw3dxft_data->text_bg_hilight = 1;
+        xaw3dxft_data->text_bg_hilight_color = 0x333300;
+        xaw3dxft_data->menu_spacing = (appInfo.spacing)? atoi(appInfo.spacing):1;
+        /* set default Xft police = Liberation-9 point if none defined */
+        xaw3dxft_proc = xaw3dxft_data->proc;
+        xaw3dxft_proc->set_default_fontname((appInfo.menufont)? 
+                                            appInfo.menufont:"Liberation-9");
+        xaw3dxft_proc->set_insensitive_twist((appInfo.twistcolor)? 
+                                             appInfo.twistcolor:"#a00000");
+        xaw3dxft_proc->set_hilit_color((appInfo.hilitcolor)?
+                                       appInfo.hilitcolor:"#332211");
+    }
 #endif
+
    /*
     **  Load help and messages files
      */
@@ -875,7 +914,7 @@ main(int argc, char *argv[])
 	usage(argv[0], "Invalid option");
 
     checkRCsize();
-
+ 
     /*
     **  A little initialization
      */
@@ -896,7 +935,6 @@ main(int argc, char *argv[])
     Global.brushes = NULL;
     Global.cap = 0;
     Global.join = 0;
-    Global.escape = False;
     Global.transparent = True;
     Global.astext = False;
     Global.dashlist[0] = '\0';
@@ -905,6 +943,7 @@ main(int argc, char *argv[])
     Global.dpi = 300.0;
     Global.numregions = 0;
     Global.regiondata = NULL;
+    Global.delimitation = NULL;
     Global.curpaint = None;
     Global.popped_up = None;
     Global.popped_parent = None;
@@ -912,9 +951,10 @@ main(int argc, char *argv[])
     Global.loadedfiles = NULL;
     Global.numpage = 1;
     Global.operation = 0;
+    Global.btn_height = appInfo.btnheight;
     Global.alpha_bg = 0;
     Global.alpha_fg = 144;
-    Global.alpha_threshold = 128;
+    Global.alpha_threshold = 127;
     memset(Global.bg, 0xff, 3);
     if (Global.depth <= 8) {
         Global.bg[4] = BlackPixelOfScreen(XtScreen(toplevel));
@@ -924,6 +964,8 @@ main(int argc, char *argv[])
     Global.xft_text = NULL;
     Global.xft_name = NULL;
     Global.xft_font = NULL;
+    /* image parameter defaults */
+    memset(&Global.image_param, 0, sizeof(ImageParameters));
 
     /*
     **
@@ -932,6 +974,11 @@ main(int argc, char *argv[])
     if (isIcon)
 	XrmPutStringResource(&rdb, "Canvas.iconic", "on");
 
+    sprintf(phrase, "%d", Global.btn_height);
+    XrmPutStringResource(&rdb, "xpaint*MenuButton.height", phrase);
+    XrmPutStringResource(&rdb, "Canvas*MenuButton.height", phrase);
+    XrmPutStringResource(&rdb, "xpaint*Command.height", phrase);
+    XrmPutStringResource(&rdb, "Canvas*Command.height", phrase);        
 	 
     /*
     **  GRR 960525:  check depth and warn user (use AlertBox() instead?)
@@ -986,8 +1033,22 @@ main(int argc, char *argv[])
     **  Call the initializers
      */
 
+#ifdef XAW3DXFT
+    if (xaw3dxft_data)
+        xaw3dxft_data->show_tips = appInfo.tooltips;
+    if (Global.depth > 8 &&
+        XAllocNamedColor(dpy, DefaultColormapOfScreen(XtScreen(toplevel)), 
+			 "#FFFFA0", &xc, &xc) == 0)
+        xc.pixel = WhitePixelOfScreen(XtScreen(toplevel));
+    if (xaw3dxft_data)
+        xaw3dxft_data->tip_background_color = xc.pixel;
+#endif
+
     OperationInit(toplevel);
     if (appInfo.operation>0) OperationSet(NULL, -appInfo.operation);
+    file_JPEG_quality = appInfo.quality;
+    if (file_JPEG_quality < 50) file_JPEG_quality = 50;
+    if (file_JPEG_quality > 100) file_JPEG_quality = 100;
 
     /*
     **  A few rogue initializers
@@ -1012,7 +1073,7 @@ main(int argc, char *argv[])
     }
     Global.astext = appInfo.astext;
 
-    XParseGeometry(appInfo.size, &i, &j, 
+    XParseGeometry(appInfo.canvassize, &i, &j, 
                    &Global.default_width, &Global.default_height);
     Global.default_zoom = GetInitZoom();
 
@@ -1029,8 +1090,10 @@ main(int argc, char *argv[])
      */
     if (appInfo.magnifier && argc==1)
         XtVaSetValues(toplevel, XtNmappedWhenManaged, False, NULL);
+
     XtRealizeWidget(toplevel);
-    XtSetMinSizeHints(toplevel, 120, 82);
+    
+    SetWMInputHint(dpy, XtWindow(toplevel));
 
     /*
     **  Now open any file on the command line
@@ -1068,14 +1131,17 @@ main(int argc, char *argv[])
 	**  If nothing is coming up, bring up a blank canvas
 	 */
         ++opened;
-	GraphicCreate(toplevel, 0);
+	GraphicCreate(toplevel, 2);
     }
 
-    XtUnmanageChild(Global.bar);
-    XMapWindow(dpy, XtWindow(Global.bar));
+    XtUnmanageChild(Global.topbar);
+    XMapWindow(dpy, XtWindow(Global.topbar));
     XtUnmanageChild(Global.back);
     XMapWindow(dpy, XtWindow(Global.back));
 
+    Global.toolmode = appInfo.toolmode;
+    SetToolIcons(3+Global.toolmode);
+    
     /* Check if there's a filter being loaded */
     if (appInfo.filter) {
         if (TestScriptC(appInfo.filter) == 2)
@@ -1100,10 +1166,11 @@ main(int argc, char *argv[])
     if (Global.numfiles>opened)
         GetFileName(Global.toplevel, BROWSER_LOADED, NULL, 
                     GraphicOpenFile, NULL);
-
+    
     /*
     **  MainLoop
      */
+
     do {
 	XtAppNextEvent(Global.appContext, &event);
 	if (event.type == ButtonPress || event.type == ButtonRelease)
