@@ -249,9 +249,10 @@ void SetFocusOn(Widget w)
     XWindowAttributes win_attributes;
     if (!w) return;
     if (!XtIsRealized(w)) return;
+    memset(&win_attributes, 0, sizeof(XWindowAttributes));
     XGetWindowAttributes(XtDisplay(w), XtWindow(w), &win_attributes);
     if (win_attributes.map_state==IsViewable)
-        XSetInputFocus(XtDisplay(w), XtWindow(w), RevertToPointerRoot, CurrentTime);
+        XSetInputFocus(XtDisplay(w), XtWindow(w), RevertToNone, CurrentTime);
   */
 }
 
@@ -268,7 +269,7 @@ HighlightChild(Widget w, XEvent * event, String * params, Cardinal * nparams)
 
     if (event->type == ButtonPress || event->type == MotionNotify) {
         if (x>=0 && x<=wi && y>=0 && y<=he)
-            XtCallActionProc(w, "highlight", event, params, 0);
+	    XtCallActionProc(w, "highlight", event, params, 0);
         else
             XtCallActionProc(w, "unhighlight", event, params, 0);
         SetFocusOn(w);
@@ -276,6 +277,21 @@ HighlightChild(Widget w, XEvent * event, String * params, Cardinal * nparams)
     SetFocusOn(Global.popped_up);
 }
 
+static void 
+CheckNotify(Widget w, XEvent * event, String * params, Cardinal * nparams)
+{
+    static Widget w0 = None;
+    if (event->type == ButtonRelease && w == w0) {
+        w0 = None;
+        return;
+    }
+    XtCallActionProc(w, "notify", event, params, 0);
+    if (event->type == ButtonPress)
+        w0 = w;
+    else
+        w0 = None;
+} 
+
 static void 
 popupMainAction(Widget w, XEvent * event, String * params, Cardinal * nparams)
 {
@@ -397,14 +413,14 @@ MenuHandler(Widget w, XtPointer l, XEvent * event, Boolean * flg)
     if (!w || !XtIsRealized(w)) return;
     XtVaGetValues(w, XtNwidth, &u, NULL);
 
-    if (event->type==KeyPress || event->type == KeyRelease) {
+    if (event->type == KeyPress || event->type == KeyRelease) {
         len = XLookupString((XKeyEvent *)event, 
                             buf, sizeof(buf) - 1, &keysym, NULL);
         if (keysym == XK_Escape)
             PopdownMenusGlobal();
     }
 
-    if (event->type==ButtonPress) {
+    if (event->type == ButtonPress) {
         if (Global.popped_up && l!=Global.popped_up)
             XtPopdown(Global.popped_up);
         XtPopup(l, XtGrabNone);
@@ -414,7 +430,7 @@ MenuHandler(Widget w, XtPointer l, XEvent * event, Boolean * flg)
         return;
     }
 
-    if (event->type==LeaveNotify && 
+    if (event->type == LeaveNotify && 
            (event->xcrossing.x<0 ||event->xcrossing.x>=u || 
             event->xcrossing.y<0)) {
         if (XtIsRealized(l)) {
@@ -428,13 +444,13 @@ MenuHandler(Widget w, XtPointer l, XEvent * event, Boolean * flg)
             Global.popped_parent = XtParent(w);
 	}
     }
-    if (event->type==EnterNotify) {
+    if (event->type == EnterNotify) {
         if (Global.popped_up && XtIsRealized(Global.popped_up) &&
             l!=Global.popped_up) {
 	    XtPopdown(Global.popped_up);
 	    Global.popped_up = None;
 	}
-        if (Global.popped_parent==XtParent(w)) {
+        if (Global.popped_parent == XtParent(w)) {
             XtVaGetValues(w, XtNx, &x, XtNy, &y, 
                           XtNwidth, &u, XtNheight, &v, NULL);
             XtMoveWidget(l, event->xcrossing.x_root-event->xcrossing.x, 
@@ -500,6 +516,7 @@ createItem(Widget parent, PaintMenuItem * item,
 					smeBSBObjectClass, parent,
 					NULL);
     }
+    XtVaSetValues(entry, XtNborderWidth, 0, NULL);
 
     if (item->flags & MF_GROUP1)
 	grp = 0;
@@ -535,7 +552,6 @@ createItem(Widget parent, PaintMenuItem * item,
 
     XtAddCallback(entry, XtNcallback,
 		      (XtCallbackProc) closeparent, item->data);
-
     item->widget = entry;
 }
 
@@ -550,11 +566,12 @@ MenuBarCreate(Widget parent, int nbar, PaintMenuBar bar[])
     Widget groups[MAX_GROUPS];
     Arg args[4];
     int nargs;
-    static XtActionsRec act[4] = {
+    static XtActionsRec act[5] = {
         {"popdown-all",  (XtActionProc) PopdownAll},
         {"escape",  (XtActionProc) PopdownEscape},
         {"popdown-child",  (XtActionProc) PopdownChild},
-        {"highlight-child",  (XtActionProc) HighlightChild}
+        {"highlight-child",  (XtActionProc) HighlightChild},
+        {"check-notify",  (XtActionProc) CheckNotify}
     };
 
     initMenu(parent);
@@ -582,6 +599,7 @@ MenuBarCreate(Widget parent, int nbar, PaintMenuBar bar[])
 	button = XtVaCreateManagedWidget(bar[list].name,
 					 menuButtonWidgetClass, parent,
 					 XtNmenuName, nm,
+					 XtNheight, Global.btn_height,
 					 XtNfromHoriz, prevButton,
 #ifdef XAW3D					 
 					 XtNhorizDistance, 2,
@@ -606,7 +624,7 @@ MenuBarCreate(Widget parent, int nbar, PaintMenuBar bar[])
                       ButtonPressMask | KeyPressMask | KeyReleaseMask |
                       EnterWindowMask | LeaveWindowMask, False,
                       (XtEventHandler) MenuHandler, (XtPointer) menu);
-        XtAppAddActions(XtWidgetToApplicationContext(button), act, 4);
+        XtAppAddActions(XtWidgetToApplicationContext(button), act, 5);
 
 	bar[list].widget = menu;
 
