@@ -8,6 +8,7 @@
 #include <sys/param.h>
 #endif
 #include <stdio.h>
+#include <stdlib.h>
 
 /*
 **  By default everything uses drand48(),
@@ -23,14 +24,6 @@
 #if !defined(__GLIBC__) & !defined(random) & !defined(__CYGWIN__)
 long random(void);
 #endif
-
-#if !defined(__VMS) & !defined(linux) & !defined(__EMX__) & !defined(__FreeBSD__) & !defined(__CYGWIN__) & !defined(__NetBSD__) & !defined(__DragonFly__) & !defined(__GLIBC__)
-#if defined(BSD4_4) || defined(HPArchitecture) || defined(SGIArchitecture) || defined(_AIX) || defined(_SCO_DS)
-void srandom(unsigned int);
-#else
-int srandom(unsigned int);
-#endif	/* BSD4_4 */
-#endif	/* linux */
 #endif
 
 #ifdef USE_DRAND
@@ -57,9 +50,6 @@ extern long lrand48();
 #define strcasecmp stricmp
 #endif
 
-#define ICONWIDTH 48
-#define ICONHEIGHT 40
-
 /* brushOp.c */
 extern void setStandardCursor(Widget w);
 
@@ -137,6 +127,7 @@ extern void SetFullMenu(Widget w, Boolean flag);
 extern Boolean IsFullMenuSet(Widget w);
 extern Boolean IsFullMenuGlobal();
 extern Boolean ToolsAreHorizontal();
+extern Boolean TooltipsCmdOption();
 
 /* menu.c */
 extern void PopdownMenusGlobal();
@@ -149,6 +140,7 @@ extern int privateXErrorHandler(Display *dpy, XErrorEvent *myerr);
 extern Widget GetToplevel(Widget w);
 extern Widget GetShell(Widget w);
 extern void RaiseWindow(Display *dpy, Window win);
+extern void SetWMInputHint(Display *dpy, Window win);
 extern void SetIBeamCursor(Widget w);
 extern void SetCrossHairCursor(Widget w);
 extern void SetPencilCursor(Widget w);
@@ -194,6 +186,8 @@ extern void RemoveFileFromGlobalList(char * file);
 extern void setWriteTextSensitive(Widget w, Boolean bool);
 extern void StdWriteText(Widget w, XtPointer infoArg, XtPointer junk);
 extern void StdOpenFile(Widget w, XtPointer paintArg, XtPointer junk);
+extern void 
+createRegionFromMaskData(XtPointer paintArg, Boolean *maskdata, Boolean target);
 
 /* pattern.c */
 extern void PatternEdit(Widget w, Pixel *pixels,
@@ -219,6 +213,7 @@ extern void takeScreenshot(Widget w, XtPointer junk, XtPointer junk2);
 extern void exitPaint(Widget w, XtPointer junk, XtPointer junk2);
 extern void setToolIconOnWidget(Widget w);
 extern void OperationSet(String names[], int num);
+extern void SetToolIcons(int mode);
 extern int getIndexOp();
 
 /* snapshot.c */
