@@ -23,15 +23,23 @@
 
 #include <X11/Intrinsic.h>
 #include <X11/StringDefs.h>
+#include <X11/Shell.h>
 #include <X11/cursorfont.h>
 #include <X11/xpm.h>
 
 #include "xaw_incdir/Form.h"
 #include "xaw_incdir/SmeBSB.h"
 #include "xaw_incdir/Box.h"
+#if !defined(NEXTAW) && !defined(XAW95)
+#include "xaw_incdir/Tip.h"
+#endif
 #include "xaw_incdir/Toggle.h"
 #include "xaw_incdir/Viewport.h"
 
+#ifdef XAW3DXFT
+#include "xaw_incdir/Xaw3dXft.h"
+#endif
+
 #include "xpaint.h"
 #include "misc.h"
 #include "menu.h"
@@ -62,16 +70,16 @@
 #include "bitmaps/tools/boxOp.xpm"
 /* #include "bitmaps/tools/rayOp.xpm" */
 #include "bitmaps/tools/filledBoxOp.xpm"
-#include "bitmaps/tools/filledOvalOp.xpm"
+#include "bitmaps/tools/filledEllipseOp.xpm"
 #include "bitmaps/tools/filledPolygonOp.xpm"
 #include "bitmaps/tools/filledFreehandOp.xpm"
 #include "bitmaps/tools/filledSplineOp.xpm"
-#include "bitmaps/tools/ovalOp.xpm"
+#include "bitmaps/tools/ellipseOp.xpm"
 #include "bitmaps/tools/splineOp.xpm"
 #include "bitmaps/tools/polygonOp.xpm"
 #include "bitmaps/tools/freehandOp.xpm"
 #include "bitmaps/tools/selectBoxOp.xpm"
-#include "bitmaps/tools/selectOvalOp.xpm"
+#include "bitmaps/tools/selectEllipseOp.xpm"
 #include "bitmaps/tools/selectPolygonOp.xpm"
 #include "bitmaps/tools/selectSplineOp.xpm"
 #include "bitmaps/tools/selectFreehandOp.xpm"
@@ -81,6 +89,12 @@
 #include "bitmaps/tools/fractalFillOp.xpm"
 #endif
 
+#ifdef XAW3DXFT
+extern Xaw3dXftData *xaw3dxft_data;
+#endif
+
+#define NUM XtNumber(iconList)
+
 /* static void changeLineAction(Widget, XEvent *); */
 static void changeDynPencilAction(Widget, XEvent *);
 static void changeBrushAction(Widget, XEvent *);
@@ -196,9 +210,9 @@ static char dynPencilDragStr[10] = "15";
 static void dynPencilOkCallback(Widget w, XtPointer junk, XtPointer infoArg)
 {
     TextPromptInfo *info = (TextPromptInfo*)infoArg;
-    float width = atof(info->prompts[0].rstr);
-    float mass = atof(info->prompts[1].rstr);
-    float drag = atof(info->prompts[2].rstr);
+    double width = atof(info->prompts[0].rstr);
+    double mass = atof(info->prompts[1].rstr);
+    double drag = atof(info->prompts[2].rstr);
     
     if (width < 1 || width > 100) {
 	Notice(w, msgText[WIDTH_SHOULD_BE_BETWEEN_ONE_AND_ONE_HUNDRED]);
@@ -310,7 +324,7 @@ static void arrowHeadOkCallback(Widget w, XtPointer junk, XtPointer infoArg)
     TextPromptInfo *info = (TextPromptInfo*)infoArg;
     int headtype = atoi(info->prompts[0].rstr);
     int size = atoi(info->prompts[1].rstr);
-    float angle = atof(info->prompts[2].rstr);
+    double angle = atof(info->prompts[2].rstr);
     
     if (headtype < 1 || headtype > 3) {
 	Notice(w, msgText[TYPE_SHOULD_BE_BETWEEN_ONE_AND_MAXTYPE]);
@@ -334,7 +348,7 @@ static void arrowHeadOkCallback(Widget w, XtPointer junk, XtPointer infoArg)
     sprintf(arrowHeadAngleStr, "%2.4g", angle);
 }
 
-static void arrowHeadMenuCallback(Widget w)
+static void arrowParamMenuCallback(Widget w)
 {
     static TextPromptInfo info;
     static struct textPromptInfo value[3];
@@ -368,7 +382,7 @@ static void
 gradientFillOkCallback(Widget w, XtPointer junk, XtPointer infoArg)
 {
     TextPromptInfo *info = (TextPromptInfo *) infoArg;
-    float HOffset, VOffset, Pad, Angle;
+    double HOffset, VOffset, Pad, Angle;
     int Steps;
 
     Angle = atof(info->prompts[0].rstr);
@@ -401,8 +415,8 @@ gradientFillOkCallback(Widget w, XtPointer junk, XtPointer infoArg)
             NUMBER_OF_STEPS_SHOULD_BE_BETWEEN_ONE_AND_THREE_HUNDRED]);
 	return;
     }
-    GradientFillSetParameters((float)(VOffset/100.0), (float)(HOffset/100.0), 
-                              (float)(Pad/100.0), (float)Angle, Steps);
+    GradientFillSetParameters((double)(VOffset/100.0), (double)(HOffset/100.0), 
+                              (double)(Pad/100.0), (double)Angle, Steps);
 
     sprintf(gradientFillVOffsetStr, "%3.4g", VOffset);
     sprintf(gradientFillHOffsetStr, "%3.4g", HOffset);
@@ -456,7 +470,7 @@ static void boxOkCallback(Widget w, XtPointer junk, XtPointer infoArg)
 {
     TextPromptInfo *info = (TextPromptInfo*)infoArg;
     int   boxSize = atoi(info->prompts[0].rstr);
-    float boxRatio = atof(info->prompts[1].rstr);
+    double boxRatio = atof(info->prompts[1].rstr);
     
     if (boxSize < 0 || boxSize > 1000) {
 	Notice(w, msgText[BOX_ROUND_CORNER_ABSOLUTE_SIZE_SHOULD_BE_BETWEEN_ZERO_AND_ONE_THOUSAND]);
@@ -504,7 +518,7 @@ static void polygonOkCallback(Widget w, XtPointer junk, XtPointer infoArg)
 {
     TextPromptInfo *info = (TextPromptInfo*)infoArg;
     int sides = atoi(info->prompts[0].rstr);
-    float ratio = atof(info->prompts[1].rstr);
+    double ratio = atof(info->prompts[1].rstr);
     
     if (sides < 3 || sides > 100) {
 	Notice(w, msgText[NUMBER_OF_SIDES_SHOULD_BE_BETWEEN_THREE_AND_ONE_HUNDRED]);
@@ -580,7 +594,7 @@ exitPaint(Widget w, XtPointer junk, XtPointer junk2)
 	exitOkCallback(w, NULL, NULL);
 	return;
     }
-    AlertBox(w, msgText[UNSAVED_CHANGES_WISH_TO_QUIT],
+    AlertBox(GetShell(w), msgText[UNSAVED_CHANGES_WISH_TO_QUIT],
 	     exitOkCallback, exitCancelCallback, NULL);
 }
 
@@ -617,7 +631,9 @@ static void selectPopupCB(Widget w, XtPointer junk, XtPointer junk2);
 static void dynPencilPopupCB(Widget w, XtPointer junk, XtPointer junk2);
 static void sprayPopupCB(Widget w, XtPointer junk, XtPointer junk2);
 static void linePopupCB(Widget w, XtPointer junk, XtPointer junk2);
-static void ovalPopupCB(Widget w, XtPointer junk, XtPointer junk2);
+static void arrowPopupCB(Widget w, XtPointer junk, XtPointer junk2);
+static void ellipsePopupCB(Widget w, XtPointer junk, XtPointer junk2);
+static void freehandPopupCB(Widget w, XtPointer junk, XtPointer junk2);
 static void splinePopupCB(Widget w, XtPointer junk, XtPointer junk2);
 static void boxPopupCB(Widget w, XtPointer junk, XtPointer junk2);
 
@@ -665,6 +681,7 @@ static PaintMenuItem smearPopup[] =
 {
     MI_SEPARATOR(),
     MI_SIMPLECB("select", BrushSelect, NULL),
+    MI_SEPARATOR(),
     MI_SIMPLECB("help", HelpDialog, "toolbox.tools.smear"),
 };
 
@@ -673,7 +690,7 @@ static PaintMenuItem linePopup[] =
     MI_SEPARATOR(),
     MI_FLAGCB("multi", MF_CHECK, linePopupCB, 0),
     MI_FLAGCB("vector", MF_CHECK, linePopupCB, 1),
-    MI_SIMPLECB("param", arrowHeadMenuCallback, NULL),
+    MI_SIMPLECB("param", arrowParamMenuCallback, NULL),
     MI_SEPARATOR(),
     MI_SIMPLECB("help", HelpDialog, "toolbox.tools.line"),
 };
@@ -694,7 +711,9 @@ static PaintMenuItem arcPopup[] =
 static PaintMenuItem arrowPopup[] =
 {
     MI_SEPARATOR(),
-    MI_SIMPLECB("param", arrowHeadMenuCallback, NULL),
+    MI_FLAGCB("head_only", MF_CHECK, arrowPopupCB, 0),
+    MI_SIMPLECB("param", arrowParamMenuCallback, NULL),
+    MI_SEPARATOR(),
     MI_SIMPLECB("help", HelpDialog, "toolbox.tools.arrow"),
 };
 
@@ -702,6 +721,7 @@ static PaintMenuItem textPopup[] =
 {
     MI_SEPARATOR(),
     MI_SIMPLECB("select", changeFontAction, NULL),
+    MI_SEPARATOR(),
     MI_SIMPLECB("help", HelpDialog, "toolbox.tools.text"),
 };
 
@@ -748,24 +768,26 @@ static PaintMenuItem selectBoxPopup[] =
     MI_SIMPLECB("help", HelpDialog, "toolbox.tools.select"),
 };
 
-static PaintMenuItem ovalPopup[] =
+static PaintMenuItem ellipsePopup[] =
 {
     MI_SEPARATOR(),
-    MI_FLAGCB("center", MF_CHECK, ovalPopupCB, 0),
-    MI_SIMPLECB("help", HelpDialog, "toolbox.tools.oval"),
+    MI_FLAGCB("center", MF_CHECK, ellipsePopupCB, 0),
+    MI_SEPARATOR(),
+    MI_SIMPLECB("help", HelpDialog, "toolbox.tools.ellipse"),
 };
 
-static PaintMenuItem filledOvalPopup[] =
+static PaintMenuItem filledEllipsePopup[] =
 {
     MI_SEPARATOR(),
-    MI_FLAGCB("center", MF_CHECK, ovalPopupCB, 0),
-    MI_SIMPLECB("help", HelpDialog, "toolbox.tools.oval"),
+    MI_FLAGCB("center", MF_CHECK, ellipsePopupCB, 0),
+    MI_SEPARATOR(),
+    MI_SIMPLECB("help", HelpDialog, "toolbox.tools.ellipse"),
 };
 
-static PaintMenuItem selectOvalPopup[] =
+static PaintMenuItem selectEllipsePopup[] =
 {
     MI_SEPARATOR(),
-    MI_FLAGCB("center", MF_CHECK, ovalPopupCB, 0),
+    MI_FLAGCB("center", MF_CHECK, ellipsePopupCB, 0),
     MI_SEPARATOR(),
     MI_FLAGCB("shape", MF_CHECKON | MF_GROUP1, selectPopupCB, 0),
     MI_FLAGCB("not_color", MF_CHECK | MF_GROUP1, selectPopupCB, 1),
@@ -774,7 +796,16 @@ static PaintMenuItem selectOvalPopup[] =
     MI_SIMPLECB("help", HelpDialog, "toolbox.tools.select"),
 };
 
-GENERATE_HELP(freehandPopup, "toolbox.tools.freehand")
+static PaintMenuItem freehandPopup[] = 
+{
+    MI_SEPARATOR(),
+    MI_FLAGCB("open", MF_CHECKON | MF_GROUP1, freehandPopupCB, 0),
+    MI_FLAGCB("closed", MF_CHECK | MF_GROUP1, freehandPopupCB, 1),
+    MI_SEPARATOR(),
+    MI_SIMPLECB("help", HelpDialog, "toolbox.tools.freehand")
+};
+
+GENERATE_HELP(filledFreehandPopup, "toolbox.tools.freehand")
 
 static PaintMenuItem selectFreehandPopup[] =
 {
@@ -861,7 +892,7 @@ static PaintMenuItem fillPopup[] =
 {
     MI_SEPARATOR(),
     MI_FLAGCB("fill", MF_CHECKON | MF_GROUP1, fillPopupCB, 0),
-    MI_FLAGCB("change", MF_CHECK | MF_GROUP1, fillPopupCB, 1),
+    MI_FLAGCB("fill_color", MF_CHECK | MF_GROUP1, fillPopupCB, 1),
     MI_FLAGCB("fill_range", MF_CHECK | MF_GROUP1, fillPopupCB, 2),
     MI_SEPARATOR(),
     MI_SIMPLECB("help", HelpDialog, "toolbox.tools.fill"),
@@ -877,7 +908,7 @@ static PaintMenuItem gradientFillPopup[] =
     MI_SIMPLECB("param", changeGradientFillParamAction, NULL),
     MI_SEPARATOR(),
     MI_FLAGCB("fill", MF_CHECKON | MF_GROUP2, fillPopupCB, 0),
-    MI_FLAGCB("change", MF_CHECK | MF_GROUP2, fillPopupCB, 1),
+    MI_FLAGCB("fill_color", MF_CHECK | MF_GROUP2, fillPopupCB, 1),
     MI_FLAGCB("fill_range", MF_CHECK | MF_GROUP2, fillPopupCB, 2),
     MI_SEPARATOR(),
     MI_SIMPLECB("help", HelpDialog, "toolbox.tools.gradientFill"),
@@ -893,7 +924,7 @@ static PaintMenuItem fractalFillPopup[] =
     MI_SIMPLECB("param", changeFractalFillParamAction, NULL),
     MI_SEPARATOR(),
     MI_FLAGCB("fill", MF_CHECKON | MF_GROUP2, fillPopupCB, 0),
-    MI_FLAGCB("change", MF_CHECK | MF_GROUP2, fillPopupCB, 1),
+    MI_FLAGCB("fill_color", MF_CHECK | MF_GROUP2, fillPopupCB, 1),
     MI_FLAGCB("fill_range", MF_CHECK | MF_GROUP2, fillPopupCB, 2),
     MI_SEPARATOR(),
     MI_SIMPLECB("help", HelpDialog, "toolbox.tools.fractalFill"),
@@ -910,14 +941,14 @@ brushPopupCB(Widget w, XtPointer junk, XtPointer junk2)
 }
 
 static void 
-ovalPopupCB(Widget w, XtPointer junk, XtPointer junk2)
+ellipsePopupCB(Widget w, XtPointer junk, XtPointer junk2)
 {
     Boolean m = !CircleGetStyle();
 
     CircleSetStyle(m);
-    MenuCheckItem(ovalPopup[1].widget, m);
-    MenuCheckItem(filledOvalPopup[1].widget, m);
-    MenuCheckItem(selectOvalPopup[1].widget, m);
+    MenuCheckItem(ellipsePopup[1].widget, m);
+    MenuCheckItem(filledEllipsePopup[1].widget, m);
+    MenuCheckItem(selectEllipsePopup[1].widget, m);
 }
 
 static void 
@@ -957,6 +988,17 @@ arcPopupCB(Widget w, XtPointer junk, XtPointer junk2)
     MenuCheckItem(w, True);
 }
 
+static void 
+freehandPopupCB(Widget w, XtPointer junk, XtPointer junk2)
+{
+    int nm = (int)(long)junk;
+
+    MenuCheckItem(freehandPopup[nm + 1].widget, True);
+    MenuCheckItem(freehandPopup[nm + 1].widget, True);
+    MenuCheckItem(freehandPopup[nm + 1].widget, True);
+    FreehandSetMode(nm);
+}
+
 static void 
 splinePopupCB(Widget w, XtPointer junk, XtPointer junk2)
 {
@@ -972,7 +1014,7 @@ void
 OperationSelectCall(int n)
 {
     MenuCheckItem(selectBoxPopup[n + 5].widget, True);
-    MenuCheckItem(selectOvalPopup[n + 3].widget, True);
+    MenuCheckItem(selectEllipsePopup[n + 3].widget, True);
     MenuCheckItem(selectFreehandPopup[n + 1].widget, True);
     MenuCheckItem(selectPolygonPopup[n + 6].widget, True);
     MenuCheckItem(selectSplinePopup[n + 5].widget, True);
@@ -1054,6 +1096,16 @@ linePopupCB(Widget w, XtPointer junk, XtPointer junk2)
     }
 }
 
+static void 
+arrowPopupCB(Widget w, XtPointer junk, XtPointer junk2)
+{
+    Boolean m;
+    m = TailGetStyle();
+    TailSetStyle(!m);
+    MenuCheckItem(arrowPopup[1].widget, m);
+}
+
+
 static void 
 erasePopupCB(Widget w, XtPointer junk, XtPointer junk2)
 {
@@ -1063,6 +1115,10 @@ erasePopupCB(Widget w, XtPointer junk, XtPointer junk2)
     MenuCheckItem(w, m);
 }
 
+#ifdef XAW3DXFT
+static void enableTooltipsCB(Widget w, XtPointer junk, XtPointer junk2);
+#endif
+
 /*
 **  Done with operation popup menus
  */
@@ -1081,7 +1137,7 @@ GENERATE_OP(Arc)
 GENERATE_OP(Arrow)
 GENERATE_OP(Erase)
 GENERATE_OP(Box)
-GENERATE_OP(Oval)
+GENERATE_OP(Ellipse)
 GENERATE_OP(Brush)
 GENERATE_OP(Font)
 GENERATE_OP(Smear)
@@ -1095,12 +1151,12 @@ GENERATE_OP(GradientFill)
 GENERATE_OP(FractalFill)
 #endif
 GENERATE_OP(FilledBox)
-GENERATE_OP(FilledOval)
+GENERATE_OP(FilledEllipse)
 GENERATE_OP(FilledPolygon)
 GENERATE_OP(FilledFreehand)
 GENERATE_OP(FilledSpline)
 GENERATE_OP(SelectBox)
-GENERATE_OP(SelectOval)
+GENERATE_OP(SelectEllipse)
 GENERATE_OP(SelectFreehand)
 GENERATE_OP(SelectPolygon)
 GENERATE_OP(SelectSpline)
@@ -1117,6 +1173,7 @@ typedef struct {
     int nitems;
     PaintMenuItem *popupMenu;
     Pixmap icon;
+    Widget widget;
 } IconListItem;
 
 static Widget iconListWidget;
@@ -1125,75 +1182,75 @@ static IconListItem iconList[] =
     /* Vertical arrangement */
 
     {"pencil", XPMMAP(pencil), &PencilOp,
-     NULL, MENU(pencil), None},
+     NULL, MENU(pencil), None, None},
     {"dynpencil", XPMMAP(dynPen), &DynPencilOp, 	
-     "<BtnDown>(2): changeDynPencil()", MENU(dynPencil), None},
+     "<BtnDown>(2): changeDynPencil()", MENU(dynPencil), None, None},
     {"dotPencil", XPMMAP(dotPen), &DotPencilOp,
-     NULL, MENU(dotPencil), None},
+     NULL, MENU(dotPencil), None, None},
 
     {"brush", XPMMAP(brush), &BrushOp,
-     "<BtnDown>(2): changeBrush()", MENU(brush), None},
+     "<BtnDown>(2): changeBrush()", MENU(brush), None, None},
     {"spray", XPMMAP(spray), &SprayOp,
-     "<BtnDown>(2): changeSpray()", MENU(spray), None},
+     "<BtnDown>(2): changeSpray()", MENU(spray), None, None},
     {"smear", XPMMAP(smear), &SmearOp,
-     "<BtnDown>(2): changeBrush()", MENU(smear), None},
+     "<BtnDown>(2): changeBrush()", MENU(smear), None, None},
 
     {"line", XPMMAP(line), &LineOp,
-     "<BtnDown>(2): changeArrow()", MENU(line), None},
+     "<BtnDown>(2): changeArrow()", MENU(line), None, None},
     {"brokenline", XPMMAP(brokenline), &BrokenlineOp,
-     NULL, MENU(brokenline), None},
+     NULL, MENU(brokenline), None, None},
     {"arc", XPMMAP(arc), &ArcOp,
-     NULL, MENU(arc), None},
+     NULL, MENU(arc), None, None},
 
     {"arrow", XPMMAP(arrow), &ArrowOp,
-     "<BtnDown>(2): changeArrow()", MENU(arrow), None},
+     "<BtnDown>(2): changeArrow()", MENU(arrow), None, None},
     {"text", XPMMAP(text), &FontOp,
-     "<BtnDown>(2): changeFont()", MENU(text), None},
+     "<BtnDown>(2): changeFont()", MENU(text), None, None},
     {"erase", XPMMAP(erase), &EraseOp,
-     "<BtnDown>(2): changeBrush()", MENU(erase), None},
+     "<BtnDown>(2): changeBrush()", MENU(erase), None, None},
 
     {"box", XPMMAP(box), &BoxOp,
-     "<BtnDown>(2): changeBoxParam()", MENU(box), None},
+     "<BtnDown>(2): changeBoxParam()", MENU(box), None, None},
     {"filledBox", XPMMAP(filledBox), &FilledBoxOp,
-     "<BtnDown>(2): changeBoxParam()", MENU(filledBox), None},
+     "<BtnDown>(2): changeBoxParam()", MENU(filledBox), None, None},
     {"selectBox", XPMMAP(selectBox), &SelectBoxOp,
-     "<BtnDown>(2): changeBoxParam()", MENU(selectBox), None},
+     "<BtnDown>(2): changeBoxParam()", MENU(selectBox), None, None},
 
-    {"oval", XPMMAP(oval), &OvalOp,
-     NULL, MENU(oval), None},
-    {"filledOval", XPMMAP(filledOval), &FilledOvalOp,
-     NULL, MENU(filledOval), None},
-    {"selectOval", XPMMAP(selectOval), &SelectOvalOp,
-     NULL, MENU(selectOval), None},
+    {"ellipse", XPMMAP(ellipse), &EllipseOp,
+     NULL, MENU(ellipse), None, None},
+    {"filledEllipse", XPMMAP(filledEllipse), &FilledEllipseOp,
+     NULL, MENU(filledEllipse), None, None},
+    {"selectEllipse", XPMMAP(selectEllipse), &SelectEllipseOp,
+     NULL, MENU(selectEllipse), None, None},
 
     {"freehand", XPMMAP(freehand), &FreehandOp,
-     NULL, MENU(freehand), None},
+     NULL, MENU(freehand), None, None},
     {"filledFreehand", XPMMAP(filledFreehand), &FilledFreehandOp,
-     NULL, MENU(freehand), None},
+     NULL, MENU(filledFreehand), None, None},
     {"selectFreehand", XPMMAP(selectFreehand), &SelectFreehandOp,
-     NULL, MENU(selectFreehand), None},
+     NULL, MENU(selectFreehand), None, None},
 
     {"polygon", XPMMAP(polygon), &PolygonOp,
-     "<BtnDown>(2): changePolygonParam()", MENU(polygon), None},
+     "<BtnDown>(2): changePolygonParam()", MENU(polygon), None, None},
     {"filledPolygon", XPMMAP(filledPolygon), &FilledPolygonOp,
-     "<BtnDown>(2): changePolygonParam()", MENU(filledPolygon), None},
+     "<BtnDown>(2): changePolygonParam()", MENU(filledPolygon), None, None},
     {"selectPolygon", XPMMAP(selectPolygon), &SelectPolygonOp,
-     "<BtnDown>(2): changePolygonParam()", MENU(selectPolygon), None},
+     "<BtnDown>(2): changePolygonParam()", MENU(selectPolygon), None, None},
 
     {"spline", XPMMAP(spline), &SplineOp,
-     NULL, MENU(spline), None},
+     NULL, MENU(spline), None, None},
     {"filledSpline", XPMMAP(filledSpline), &FilledSplineOp,
-     NULL, MENU(filledSpline), None},
+     NULL, MENU(filledSpline), None, None},
     {"selectSpline", XPMMAP(selectSpline), &SelectSplineOp,
-     NULL, MENU(selectSpline), None},
+     NULL, MENU(selectSpline), None, None},
 
     {"fill", XPMMAP(fill), &FillOp,
-     NULL, MENU(fill), None},
+     NULL, MENU(fill), None, None},
     {"gradientFill", XPMMAP(gradientFill), &GradientFillOp,
-     "<BtnDown>(2): changeGradientFillParam()", MENU(gradientFill), None},
+     "<BtnDown>(2): changeGradientFillParam()", MENU(gradientFill), None, None},
 #ifdef FEATURE_FRACTAL
     {"fractalFill", XPMMAP(fractalFill), &FractalFillOp,
-     "<BtnDown>(2): changeFractalFillParam()", MENU(fractalFill), None},
+     "<BtnDown>(2): changeFractalFillParam()", MENU(fractalFill), None, None},
 #endif
 
     /* Vertical arrangement */
@@ -1208,18 +1265,53 @@ getIndexOp()
     return 0;
 }
 
-static void
-permuteIcons()
+void
+SetToolIcons(int mode)
 {
-#define NUM XtNumber(iconList)
+    Dimension w, h;
+    Position xp, yp, x, y;
     int i;
-    IconListItem tempList[NUM];
-
-    if (ToolsAreHorizontal()) {
-    for (i=0; i<NUM; i++)
-        tempList[i] = iconList[3*(i%10)+(i/10)];
-    for (i=0; i<NUM; i++)
-        iconList[i] = tempList[i];
+#ifdef XAW3D
+    int val[8] = { 38, 36, 130, 388, 396, 134, 244, 206 };
+#else
+    int val[8] = { 38, 36, 127, 382, 390, 130, 240, 202 };
+#endif
+    
+    XtVaGetValues(iconList[0].widget, XtNx, &x, XtNy, &y, NULL);
+    XtVaGetValues(Global.toplevel, XtNwidth, &w, XtNheight, &h, NULL);
+    if (mode >= 3) {
+      mode = mode % 3;
+      XtResizeWidget(Global.toplevel,
+		     val[2*mode+2], val[2*mode+3] + Global.btn_height, 0);
+      XtUnmanageChild(Global.topvport);
+      XtResizeWidget(Global.topvport,
+		     val[2*mode+2] - 8 , val[2*mode+3] - 8, 0);
+    }
+    XMapWindow(XtDisplay(Global.topvport), XtWindow(Global.topvport));
+#ifdef XAW3D		   
+    XMoveWindow(XtDisplay(Global.topvport), XtWindow(Global.topvport),
+	        4, Global.btn_height+14);
+#else
+    XMoveWindow(XtDisplay(Global.topvport), XtWindow(Global.topvport),
+	        4, Global.btn_height+15);
+#endif		    
+    for (i=1; i<NUM; i++) {
+      XtUnmanageChild(iconList[i].widget);
+      if (mode == 0)
+          XMoveWindow(XtDisplay(iconList[i].widget),
+		      XtWindow(iconList[i].widget),
+		      x + (i%3)*val[0], y + (i/3)*val[1]);	  
+      else
+      if (mode == 1)
+          XMoveWindow(XtDisplay(iconList[i].widget),
+		      XtWindow(iconList[i].widget),
+		      x + (i/3)*val[0], y + (i%3)*val[1]);
+      else
+          XMoveWindow(XtDisplay(iconList[i].widget),
+		      XtWindow(iconList[i].widget),
+		      x + (i%6)*val[0], y + (i/6)*val[1]);	
+      XMapWindow(XtDisplay(iconList[i].widget),
+		   XtWindow(iconList[i].widget));
     }
 }
 
@@ -1257,10 +1349,10 @@ OperationSet(String names[], int num)
 
 static PaintMenuItem canvasMenu[] =
 {
-    MI_SIMPLECB("new", GraphicCreate, 0),
-    MI_SIMPLECB("new-size", GraphicCreate, 1),
-    MI_SIMPLECB("open", GraphicCreate, 2),
-    MI_SIMPLECB("loaded", GraphicCreate, 3),
+    MI_SIMPLECB("open", GraphicCreate, 0),
+    MI_SIMPLECB("loaded", GraphicCreate, 1),
+    MI_SIMPLECB("new", GraphicCreate, 2),
+    MI_SIMPLECB("new-size", GraphicCreate, 3),
     MI_SIMPLECB("magnifier", GraphicCreate, 4),
     MI_SIMPLECB("screenshot", takeScreenshot, NULL),
     MI_SEPARATOR(),
@@ -1269,6 +1361,11 @@ static PaintMenuItem canvasMenu[] =
 
 static PaintMenuItem helpMenu[] =
 {
+#ifdef XAW3DXFT
+    #define TOOLTIPS_INDEX 0
+    MI_FLAGCB("tooltips", MF_CHECK, enableTooltipsCB, 0),
+    MI_SEPARATOR(),        
+#endif
     MI_SIMPLECB("intro", HelpDialog, "introduction"),
     MI_SIMPLECB("tools", HelpDialog, "toolbox.tools"),
     MI_SIMPLECB("canvas", HelpDialog, "canvas"),
@@ -1280,11 +1377,6 @@ static PaintMenuItem helpMenu[] =
 static PaintMenuBar menuBar[] =
 {
     {None, "canvas", XtNumber(canvasMenu), canvasMenu},
-#if 0
-    {None, "other", XtNumber(otherMenu), otherMenu},
-    {None, "line", XtNumber(lineMenu), lineMenu},
-    {None, "font", XtNumber(fontMenu), fontMenu},
-#endif
     {None, "help", XtNumber(helpMenu), helpMenu},
 };
 
@@ -1294,6 +1386,16 @@ static PaintMenuBar menuBar[] =
 static int argListLen = 0;
 static Arg *argList;
 
+#ifdef XAW3DXFT
+static void
+enableTooltipsCB(Widget w, XtPointer junk, XtPointer junk2)
+{
+    if (xaw3dxft_data)
+        xaw3dxft_data->show_tips = !xaw3dxft_data->show_tips;
+    MenuCheckItem(helpMenu[TOOLTIPS_INDEX].widget, xaw3dxft_data->show_tips);
+}
+#endif
+
 void 
 OperationSetPaint(Widget paint)
 {
@@ -1345,7 +1447,7 @@ changeSprayAction(Widget w, XEvent * event)
 static void 
 changeArrowAction(Widget w, XEvent * event)
 {
-    arrowHeadMenuCallback(w);
+    arrowParamMenuCallback(w);
 }
 
 static void 
@@ -1424,36 +1526,34 @@ void
 toolsResized(Widget w, XtPointer l, XConfigureEvent * event, Boolean * flg)
 {
     Dimension width, height;
-
+    Position x, y, xp, yp;
+    int i, u, v;
+    
     if (event->type != ConfigureNotify) return;
 
     XtVaGetValues(w, XtNwidth, &width,
 		     XtNheight, &height, NULL);
-
     if (width<40 || height<72)
         XtVaSetValues(w, XtNsensitive, False, NULL);
 
-    XtResizeWidget((Widget)l, (width>40)? width - 8:32 ,
-                                  (height>70)? height - 38:32,
-#ifdef XAW3D		   
-		                  1);
-    XtMoveWidget((Widget)l, 4, 35);
+    XtUnmanageChild(Global.topvport);    
+    XtResizeWidget(Global.topvport, width - 8, height - (Global.btn_height+20), 0);
+
+#ifdef XAW3DXFT
+    XtMoveWidget(Global.back, width - 2*Global.btn_height/3 - 6, 8);
 #else
-		                  0);
-    XtMoveWidget((Widget)l, 4, 36);
-#endif		
-#if defined(XAWPLAIN) || defined(XAW95) || defined(XAW3D)
-#ifdef XAW3DG
-    if (width>20)
-        XtMoveWidget(Global.back, width-20, 8);
-#else
-    if (width>17)
-        XtMoveWidget(Global.back, width-17, 8);
-#endif
-#else
-    if (width>15)
-        XtMoveWidget(Global.back, width-15, 8);
+    if (width>18)
+        XtMoveWidget(Global.back, width-18, 8);
 #endif
+
+    if (height > 2*width) Global.toolmode = 0;
+    else
+    if (width > 2*height) Global.toolmode = 1;
+    else
+        Global.toolmode = 2;
+
+    SetToolIcons(Global.toolmode);
+
 }
 
 /*
@@ -1464,7 +1564,6 @@ OperationInit(Widget toplevel)
 {
     static XtActionsRec acts[] =
     {
-      /*	{"changeLine", (XtActionProc) changeLineAction}, */
 	{"changeDynPencil", (XtActionProc) changeDynPencilAction},
 	{"changeBrush", (XtActionProc) changeBrushAction},
 	{"changeSpray", (XtActionProc) changeSprayAction},
@@ -1477,6 +1576,8 @@ OperationInit(Widget toplevel)
     };
     int i;
     Pixmap pix;
+    char *str;
+    char buf[512];
     Widget form, vport, box, icon, firstIcon = None;
     char *defTrans = "\
 <Key>Escape: escape()\n\
@@ -1500,8 +1601,8 @@ OperationInit(Widget toplevel)
     /*
     **  Create the menu bar
      */
-    Global.bar = MenuBarCreate(form, XtNumber(menuBar), menuBar);
-    XtVaSetValues(Global.bar, 
+    Global.topbar = MenuBarCreate(form, XtNumber(menuBar), menuBar);
+    XtVaSetValues(Global.topbar, 
 #ifdef XAW3D
 #ifdef XAW3DG
                   XtNhorizDistance, 0,
@@ -1520,7 +1621,7 @@ OperationInit(Widget toplevel)
 				    viewportWidgetClass, form,
 				    XtNallowVert, True,
 				    XtNuseRight, True,
-				    XtNfromVert, Global.bar,
+				    XtNfromVert, Global.topbar,
 				    XtNborderWidth, 0,
 #ifdef XAW95
 				    XtNvertDistance, 3,
@@ -1530,7 +1631,8 @@ OperationInit(Widget toplevel)
                                     XtNborderWidth, 0,
 #endif				    
 				    NULL);
-
+    Global.topvport = vport;
+    
     box = XtVaCreateManagedWidget("box",
 				    boxWidgetClass, vport,
 #ifdef XAW3D
@@ -1540,7 +1642,7 @@ OperationInit(Widget toplevel)
 				    NULL);
 
     Global.back = XtVaCreateManagedWidget("!", labelWidgetClass, form,
-				      XtNwidth, 12,
+				      XtNwidth, 2*Global.btn_height/3,
 				      XtNvertDistance, 8,
                                       XtNborderWidth, 1,
                                       XtNsensitive, False,
@@ -1549,8 +1651,6 @@ OperationInit(Widget toplevel)
     XtAddEventHandler(Global.back, ButtonPressMask | ButtonReleaseMask, False,
         (XtEventHandler) switchtoCanvasCallback, NULL);
 
-    permuteIcons();
-
     attributes.valuemask = XpmCloseness;
     attributes.closeness =  40000;
 
@@ -1579,6 +1679,7 @@ OperationInit(Widget toplevel)
 	    exit(1);
 	}
 	
+	cur->widget = icon;
 	cur->icon = pix;
 
 	if (pix != None)
@@ -1627,69 +1728,57 @@ OperationInit(Widget toplevel)
 #endif
 	if (cur->nitems != 0 && cur->popupMenu != NULL)
 	    MenuPopupCreate(icon, "popup-menu", cur->nitems, cur->popupMenu);
+
+#if defined(XAW3DXFT) || defined(XAW3D)
+#if !defined(NEXTAW) && !defined(XAW95)
+	if (TooltipsCmdOption()) {
+	    XtVaGetValues(XtParent(cur->popupMenu->widget), XtNlabel, &str, NULL);
+            sprintf(buf, "%s : %s\n%s", msgText[TOOLTIP_TOOL], str,
+                    msgText[TOOLTIP_AUXILIARY_MENU]);
+            XawTipEnable(icon, buf);
+	}
+#endif
+#else
+#ifdef XAWPLAIN
+        XtVaGetValues(XtParent(cur->popupMenu->widget), XtNlabel, &str, NULL);
+        sprintf(buf, "%s : %s\n%s", msgText[TOOLTIP_TOOL], str,
+                msgText[TOOLTIP_AUXILIARY_MENU]);
+	XtVaSetValues(icon, XtNtip, buf, NULL);
+        XawTipEnable(icon);
+#endif
+#endif
     }
 
-    if (ToolsAreHorizontal())
-        XtResizeWidget(vport,
-#if defined(XAW3D) && !defined(XAW3DG)		      
-#ifdef BIGTOOLICONS
-	          565, 157, 0);
-#else
-	          424, 108, 0);
-#endif
-#endif
-#ifdef XAW3DG
-#ifdef BIGTOOLICONS
-	          529, 149, 0);
-#else
-	          388, 112, 0);
-#endif
-#endif
-#ifdef XAW95
-#ifdef BIGTOOLICONS
-	          550, 155, 0);
-#else
-	          388, 106, 0);
-#endif
-#endif
-#ifdef XAWPLAIN		      
-#ifdef BIGTOOLICONS
-		  544, 152, 0);
-#else
-		  386, 102, 0);
-#endif
-#endif	      
-    else
-        XtResizeWidget(vport,
+    XtResizeWidget(vport,
 #if defined(XAW3D) && !defined(XAW3DG)
 #ifdef BIGTOOLICONS
-		      173, 521, 0);
+	           173, 521, 0);
 #else
-		      124, 361, 0);
+	           124, 361, 0);
 #endif
 #endif
 #ifdef XAW3DG
 #ifdef BIGTOOLICONS
-	              165, 485, 0);
+	           165, 485, 0);
 #else
-                      122, 364, 0);
+                   122, 364, 0);
 #endif
 #endif
 #ifdef XAW95
 #ifdef BIGTOOLICONS
-                      172, 503, 0);
+                   172, 503, 0);
 #else
-                      123, 346, 0);
+                   123, 346, 0);
 #endif
 #endif
 #ifdef XAWPLAIN		      
 #ifdef BIGTOOLICONS
-                      167, 502, 0);
+                   167, 502, 0);
 #else
-                      118, 340, 0);
+                   118, 340, 0);
 #endif
 #endif	      
-  
+
     AddDestroyCallback(toplevel, (DestroyCallbackFunc) exitPaint, NULL);
 
     XtAddEventHandler(toplevel, StructureNotifyMask, False,
@@ -1704,6 +1793,11 @@ OperationInit(Widget toplevel)
                 ButtonPressMask | ButtonReleaseMask |
                 KeyPressMask | KeyReleaseMask, False, 
                 (XtEventHandler) processEvent, (XtPointer)NULL);
+
+#ifdef XAW3DXFT
+    if (xaw3dxft_data)
+        MenuCheckItem(helpMenu[TOOLTIPS_INDEX].widget,xaw3dxft_data->show_tips);
+#endif
 }
 
 Image *
