@@ -149,7 +149,7 @@ static PaintMenuItem imageMenu[] =
 };
 static PaintMenuItem helpMenu[] =
 {
-    MI_SIMPLECB("help", HelpDialog, "patternSelector"),
+    MI_SIMPLECB("help", HelpDialog, "canvas.selectorMenu.patternSelector"),
 };
 static PaintMenuBar menuBar[] =
 {
@@ -702,8 +702,12 @@ patternResized(Widget w, LocalInfo * l, XConfigureEvent * event, Boolean * flg)
     XtVaGetValues(w, XtNwidth, &width, XtNheight, &height, NULL);
     if (width<400) width = 400;
     if (height<300) height = 300;
+#ifdef XAW3DXFT
+    height1 = (height-ICONHEIGHT - 10) * 2/3;
+    height2 = height - height1 - ICONHEIGHT - 15;
+#else    
 #ifdef XAW3D
-    height1 = ((height-61)*265)/480;
+    height1 = ((height-ICONHEIGHT)*265)/480;
 #ifdef XAW3DG
     if (height1<271) height1 = 271;
 #else
@@ -711,15 +715,16 @@ patternResized(Widget w, LocalInfo * l, XConfigureEvent * event, Boolean * flg)
 #endif   
     height2 = ((height-61)*208)/480;
 #else   
-    height1 = ((height-59)*276)/480;
+    height1 = ((height-69)*276)/480;
     if (height1<276) height1 = 276;    
     height2 = ((height-59)*196)/480;
-#endif   
+#endif
+#endif    
     XtResizeWidget(l->form, width-8, height1, 1);
-    XtMoveWidget(l->form, 4, 54);
+    XtMoveWidget(l->form, 4, ICONHEIGHT+10);
     XtResizeWidget(l->form1, width-8, height2, 1);
-    XtMoveWidget(l->form1, 4, 60 + height1);      
-    XtResizeWidget(l->form2, width-14, height2-8, 0);
+    XtMoveWidget(l->form1, 4, ICONHEIGHT + 11 + height1);      
+    XtResizeWidget(l->form2, width-14, height2, 0);
     XtMoveWidget(l->form2, 4, 4);   
 #ifdef XAWPLAIN
     XtResizeWidget(l->vport, width-345, height1-32, 0);   
@@ -1447,7 +1452,8 @@ sizeChoiceCallback(Widget w, XtPointer larg, XtPointer junk)
     sprintf(bufA, "%d", width);
     sprintf(bufB, "%d", height);
 
-    TextPrompt(w, "sizeselect", &info, (XtCallbackProc) sizeOkChoiceCallback,
+    TextPrompt(w, msgText[SIZE_SELECTION], 
+               &info, (XtCallbackProc) sizeOkChoiceCallback,
 	       NULL, larg);
 }
 static void 
@@ -1490,6 +1496,45 @@ gridCallback(Widget w, XtPointer larg, XtPointer junk)
     MenuCheckItem(w, v);
 }
 
+static void
+fatzoomOkCallback(Widget w, XtPointer arg, XtPointer data)
+{
+    Widget fat = (Widget) arg;
+    TextPromptInfo *info = (TextPromptInfo *) data;
+    int zoom;
+
+    zoom = StrToZoom(info->prompts[0].rstr);
+
+    if (zoom == 0 || zoom < -9 || zoom > 32) {
+	Notice(fat, msgText[INVALID_ZOOM]);
+    } else {
+        setZoom(fat, zoom);
+    }
+}
+
+static void
+fatzoomCallback(Widget w, XtPointer arg, XtPointer junk2)
+{
+    static TextPromptInfo info;
+    static struct textPromptInfo values[2];
+    static char buf[80];
+    int zoom;
+    Widget fat = (Widget) arg;
+
+    info.nprompt = 1;
+    info.prompts = values;
+    info.title = msgText[CHANGE_ZOOM_FACTOR_FOR_IMAGE];
+    values[0].prompt = msgText[ZOOM];
+    values[0].len = 4;
+    values[0].str = buf;
+
+    XtVaGetValues(fat, XtNzoom, &zoom, NULL);
+    strcpy(buf, ZoomToStr(zoom));
+
+    TextPrompt(GetShell(fat), msgText[ZOOM_FACTOR],
+               &info, fatzoomOkCallback, NULL, fat);
+}
+
 /*
 static void
 changeShellSize(Widget w, LocalInfo * l, XEvent * event, Boolean * flg)
@@ -1574,14 +1619,13 @@ void LoadRCInfo(RCInfo *rcInfo, Palette *map)
 					      rcInfo->ncolors);
     rcInfo->colorPixels = (Pixel *) XtCalloc(sizeof(Pixel),
 					     rcInfo->ncolors);
+    
     for (i = 0; i < rcInfo->ncolors; i++)
 	rcInfo->colorFlags[i] = False;
 
     for (i = 0; i < rcInfo->ncolors; i++) {
-	if (XLookupColor(dpy, map->cmap,
-			 rcInfo->colors[i], &col, &rgb) ||
-	    XParseColor(dpy, map->cmap,
-			rcInfo->colors[i], &col)) {
+        if (XLookupColor(dpy, map->cmap, rcInfo->colors[i], &col, &rgb) ||
+	    XParseColor(dpy, map->cmap, rcInfo->colors[i], &col)) {
 	    rcInfo->colorPixels[i] = PaletteAlloc(map, &col);
 	    rcInfo->colorFlags[i] = True;
 	    for (j = 0; j < i; j++)
@@ -1767,6 +1811,7 @@ PatternEdit(Widget w, Pixel *pixels, Pixmap *patterns, void *brushes,
                                       NULL);
 
     cancel = XtVaCreateManagedWidget("close", commandWidgetClass, topf,
+				      XtNheight, Global.btn_height,
                                       XtNfromHoriz, info->icon,
 				      XtNvertDistance, 13,
 				      XtNhorizDistance, 15,
@@ -1788,6 +1833,7 @@ PatternEdit(Widget w, Pixel *pixels, Pixmap *patterns, void *brushes,
 				         XtNleft, XtChainLeft,
 				         XtNtop, XtChainTop,
 				         XtNbottom, XtChainBottom,
+				         XtNheight, Global.btn_height,
 					 NULL);
 
     selectPattern = XtVaCreateManagedWidget("select",
@@ -1797,6 +1843,7 @@ PatternEdit(Widget w, Pixel *pixels, Pixmap *patterns, void *brushes,
 				       XtNright, XtChainRight,
 				       XtNtop, XtChainTop,
 				       XtNbottom, XtChainBottom,
+				       XtNheight, Global.btn_height,
 				       NULL);
 
     asbrushPattern = XtVaCreateManagedWidget("asbrush",
@@ -1806,6 +1853,7 @@ PatternEdit(Widget w, Pixel *pixels, Pixmap *patterns, void *brushes,
 				       XtNright, XtChainRight,
 				       XtNtop, XtChainTop,
 				       XtNbottom, XtChainBottom,
+				       XtNheight, Global.btn_height,
 				       NULL);
 
     info->vport = XtVaCreateManagedWidget("viewport", viewportWidgetClass, 
@@ -1821,7 +1869,7 @@ PatternEdit(Widget w, Pixel *pixels, Pixmap *patterns, void *brushes,
 			         XtNtop, XtChainTop,
 				 XtNbottom, XtChainBottom,
 				 XtNwidth, 286,
-				 XtNheight, 290,
+				 XtNheight, 250,
 				 NULL);
 
     if (cpick != None) {
@@ -1942,7 +1990,7 @@ PatternEdit(Widget w, Pixel *pixels, Pixmap *patterns, void *brushes,
     XtAddCallback(imageMenu[IM_GRID].widget,
 		  XtNcallback, gridCallback, (XtPointer) info);
     XtAddCallback(imageMenu[IM_ZOOM].widget,
-		  XtNcallback, zoomCallback, (XtPointer) info->fat);
+		  XtNcallback, fatzoomCallback, (XtPointer) info->fat);
     XtAddCallback(imageMenu[IM_BGRD].widget,
 		  XtNcallback, changeBackground, (XtPointer) info->paint);
 
@@ -2000,6 +2048,8 @@ PatternEdit(Widget w, Pixel *pixels, Pixmap *patterns, void *brushes,
 
     copyPaletteArea(info, pixels, patterns, npixels, npatterns);
     XtPopup(Global.patternshell, XtGrabNone);
+    SetWMInputHint(dpy, XtWindow(Global.patternshell));
+
     XtVaGetValues(Global.patternshell, XtNtitle, &ptr, NULL);
     StoreName(Global.patternshell, ptr);
    
