@@ -75,8 +75,8 @@ extern int WriteResizedPS(char *file, Image * image, void *data);
 
 extern Image *outputImage;
 
-static int PAGEWIDTH = 200;
-static int PAGEHEIGHT = 283;
+static int PAGEWIDTH;
+static int PAGEHEIGHT;
 #if defined(XAW95) || defined(XAW3DG)
 #define EXTRAWIDTH 15
 #define EXTRAHEIGHT 13
@@ -85,6 +85,10 @@ static int PAGEHEIGHT = 283;
 #define EXTRAHEIGHT 10
 #endif
 
+static int oldw = -1, oldh = -1;
+
+static XtTranslations text_translations = None;
+
 static String textTranslations =
     "#override\n\
 <Key>Return: print-text-ok()\n\
@@ -92,8 +96,6 @@ static String textTranslations =
 Ctrl<Key>M: print-text-ok()\n\
 Ctrl<Key>J: print-text-ok()\n";
 
-static XtTranslations translations = None;
-
 static PaintMenuItem fileMenu[] =
 {
     MI_SIMPLE("print"),
@@ -146,7 +148,7 @@ static PaintMenuBar printMenuBar[] =
 {
     {None, "file", XtNumber(fileMenu), fileMenu},
     {None, "help", XtNumber(helpMenu), helpMenu},
-    {None, "format", XtNumber(formatMenu), formatMenu},
+    {None, "format", XtNumber(formatMenu), formatMenu}
 };
 
 static PaintMenuItem externMenu[] =
@@ -154,7 +156,7 @@ static PaintMenuItem externMenu[] =
     MI_FLAG("GIF", MF_CHECK | MF_GROUP1),
     MI_FLAG("PNG", MF_CHECK | MF_GROUP1),
     MI_FLAG("PPM", MF_CHECK | MF_GROUP1),
-    MI_FLAG("XPM", MF_CHECK | MF_GROUP1),
+    MI_FLAG("XPM", MF_CHECK | MF_GROUP1)
 };
 
 static PaintMenuBar externMenuBar[] =
@@ -173,23 +175,18 @@ typedef struct {
     PaintWidget paintwidget;
     Widget shell, form, bar;
     Widget format, print, preview, cancel;
-    Widget page, box, vp, equal;
-    Widget wplus, wpplus, wminus, wmminus, wcenter;
+    Widget page, box, vp;
+    Widget wplus, wpplus, wminus, wmminus, wcenter, equal;
     Widget hplus, hpplus, hminus, hmminus, hcenter;
     Widget portraitlabel, portraittoggle;
     Widget landscapelabel, landscapetoggle;
     Widget graylabel, graytoggle;
-    Widget compresslabel, compresstoggle;
-    Widget pdflabel, pdftoggle;
-    Widget sizelabel, size;
-    Widget positionlabel, position;
-    Widget samplinglabel, sampling;
-    Widget printlabel, printcmd, printtoggle;
-    Widget filelabel, filename, fileselect, filetoggle;
-    Widget psviewlabel, psviewcmd;
-    Widget printerlist, printentry;
-    Widget printermenu;
-    Widget resultlabel, printresult;
+    Widget compresslabel, compresstoggle, pdflabel, pdftoggle;
+    Widget sizelabel, positionlabel, samplinglabel, 
+           printlabel, filelabel, psviewlabel, resultlabel;
+    Widget size, position, sampling, printcmd, filename, psviewcmd, printresult;
+    Widget printerlist, printtoggle, fileselect, filetoggle;
+    Widget printentry, printermenu;
     Widget formatChecks[XtNumber(formatMenu)];
 } PrintInfo;
 
@@ -379,11 +376,12 @@ drawArrows(Widget w, PrintInfo * l, XEvent * event, Boolean * flg)
 {
     Display *dpy = XtDisplay(l->form);
     Window win = XtWindow(l->form);
-    int i, j, top;
+    int a, i, j, top;
 
 #ifdef XAW3D
-    i = 126;
-    j = PAGEHEIGHT+EXTRAHEIGHT+60;
+    a = 4 * Global.btn_height / 5 + 4;
+    i = 5 * a + 31;
+    j = PAGEHEIGHT+EXTRAHEIGHT+3*Global.btn_height/2+27;
 #else
     i = 120;   
     j = PAGEHEIGHT+EXTRAHEIGHT+51;
@@ -394,9 +392,9 @@ drawArrows(Widget w, PrintInfo * l, XEvent * event, Boolean * flg)
     XDrawLine(dpy, win, l->gc2, i-6, j, i+4, j-5);
 
 #ifdef XAW3D 
-    i = PAGEWIDTH+EXTRAWIDTH+16;
-    j = PAGEHEIGHT+EXTRAHEIGHT+42;
-    top = 183;
+    i = PAGEWIDTH+EXTRAWIDTH+ a/2 + 8;
+    j = PAGEHEIGHT+EXTRAHEIGHT+Global.btn_height+ 18;
+    top = Global.btn_height+ 5*a + 45;
 #else
     i = PAGEWIDTH+EXTRAWIDTH+17;   
     j = PAGEHEIGHT+EXTRAHEIGHT+34;
@@ -408,10 +406,23 @@ drawArrows(Widget w, PrintInfo * l, XEvent * event, Boolean * flg)
     XDrawLine(dpy, win, l->gc2, i, top-6, i-5, top+4);
 }
 
+static void
+buttonPressed(Widget w, XtPointer arg, XConfigureEvent * event, 
+              Boolean * flg)
+{
+    if (event->type == ButtonPress || event->type == ButtonRelease) {
+        XtSetKeyboardFocus((Widget)arg, None);
+        XtSetKeyboardFocus((Widget)arg, w);
+    }
+}
+
 static void 
 closeMenus(Widget w, PrintInfo * l, XEvent * event, Boolean * flg)
 {
     PopdownMenusGlobal();
+    if (event->type = LeaveNotify)
+        XSetInputFocus(XtDisplay(l->form), XtWindow(l->form),
+                       RevertToNone, CurrentTime);
 }
 
 static void
@@ -504,7 +515,7 @@ printCallback(Widget w, XtPointer infoArg, XtPointer junk2)
         if (dopdf) {
             doWritePS(info, info->tmpfile);
             sprintf(cmd, "ps2pdf %s %s", info->tmpfile, name);
-            system(cmd);
+            if (system(cmd));
             unlink(info->tmpfile);
 	} else
             doWritePS(info, name);
@@ -515,7 +526,7 @@ printCallback(Widget w, XtPointer infoArg, XtPointer junk2)
         doWritePS(info, info->tmpfile);
         sprintf(cmd, "( %s %s ; rm -f %s ) &", 
             name, info->tmpfile, info->tmpfile);
-        system(cmd);
+        if (system(cmd));
         sprintf(cmd, msgText[FILE_SENT_TO_PRINTER], name);
     }
     FlashString(cmd);
@@ -548,7 +559,7 @@ previewCallback(Widget w, XtPointer infoArg, XtPointer junk2)
     char cmd[2048];
     char *ps_viewer = NULL;
     char *name = NULL;
-    Boolean dopdf, dofile, bool;
+    Boolean dopdf, dofile;
 
     PopdownMenusGlobal();
     XtVaGetValues(info->psviewcmd, XtNstring, &ps_viewer, NULL);
@@ -561,20 +572,20 @@ previewCallback(Widget w, XtPointer infoArg, XtPointer junk2)
 
     doWritePS(info, info->tmpfile);
 
-    if ((bool=dofile && name && *name)) {
+    if ((dofile && name && *name)) {
        if (dopdf) 
           sprintf(cmd, "( ps2pdf %s %s ; %s %s ; rm -f %s ) &", 
 	          info->tmpfile, name, ps_viewer, name, info->tmpfile);
        else
           sprintf(cmd, "mv -f %s %s ; %s %s &", 
 	          info->tmpfile, name, ps_viewer, name);
-       system(cmd);
+       if (system(cmd));
     } else {
        sprintf(cmd, "( %s %s ; rm -f %s ) &", 
 	       ps_viewer, info->tmpfile, info->tmpfile);
-       system(cmd);
+       if (system(cmd));
     }
-    sprintf(cmd, msgText[FILE_WRITTEN], (bool)? name:info->tmpfile);
+    sprintf(cmd, msgText[FILE_WRITTEN], (dofile)? name:info->tmpfile);
     FlashString(cmd);
 }
 
@@ -601,6 +612,7 @@ closePrintCallback(Widget w, XtPointer infoArg, XtPointer junk2)
     XFreeGC(dpy, info->gc2);
     XtDestroyWidget(info->shell);
     info->shell = 0;
+    oldw = oldh = -1;
 }
 
 void
@@ -972,64 +984,144 @@ void createPrintData()
     strcat(buf, ".ps");
     printinfo->tmpfile = XtNewString(buf);
 }
+
 void
 printpopupResized(Widget w, PrintInfo * l, XConfigureEvent * event, 
                   Boolean * flg)
 {
     Dimension width, height;
     Widget *wg;
-    static int oldw;
-    int x, y, i, n;
+    Position x, y;
+    int dx, dy, i, n;
     XtVaGetValues(l->shell, XtNwidth, &width, XtNheight, &height, NULL);
-    XtVaGetValues(l->sampling, XtNhorizDistance, &x, NULL);
-    XResizeWindow(XtDisplay(l->form), XtWindow(l->form), width, height);
-
-    XClearArea(XtDisplay(l->form), XtWindow(l->form),
-               0, 30, PAGEWIDTH+100, PAGEHEIGHT+100, 0);
-    oldw = PAGEWIDTH;
-    PAGEWIDTH = width - x - 218;
-    if (PAGEWIDTH<156) PAGEWIDTH = 156;
-    PAGEHEIGHT = height - 87;
-    if (PAGEHEIGHT<220) PAGEHEIGHT = 220;
-    XtResizeWidget(l->vp, PAGEWIDTH+EXTRAWIDTH, PAGEHEIGHT+EXTRAHEIGHT, 0);
-    XtResizeWidget(l->box, PAGEWIDTH+EXTRAWIDTH, PAGEHEIGHT+EXTRAHEIGHT, 0);
-    XtResizeWidget(l->page, PAGEWIDTH, PAGEHEIGHT, 1);
-
-#ifdef XAW3D   
-    XtMoveWidget(l->equal, PAGEWIDTH+EXTRAWIDTH+8, PAGEHEIGHT+EXTRAHEIGHT+49);
-#else   
-    XtMoveWidget(l->equal, PAGEWIDTH+EXTRAWIDTH+8, PAGEHEIGHT+EXTRAHEIGHT+42);
-#endif
-   
-    wg = &l->wplus;
-
-    for (i=0; i<5; i++) {
-       XtVaGetValues(wg[i], XtNx, &x, NULL);
+    if (width == oldw && height == oldh) return;
+    if (oldw == -1) {
+      x = 0;
+      wg = &l->size;
+      for (i=0; i<7; i++) {
+        XtVaGetValues(wg[i], XtNx, &y, NULL);
+        if (y > x) x = y;
+      }
+      XResizeWindow(XtDisplay(l->form), XtWindow(l->form), width, height);
+      XMapWindow(XtDisplay(l->form), XtWindow(l->form));	      
+      for (i=0; i<7; i++) {
+        XtVaGetValues(wg[i], XtNy, &y, NULL);
+        XtMoveWidget(wg[i], x, y);
+	XMapWindow(XtDisplay(wg[i]), XtWindow(wg[i]));	
+      }
+      wg = &l->printerlist;
+      for (i=0; i<4; i++) if (wg[i]) {
+        XtVaGetValues(wg[i], XtNy, &y, NULL);
+        XtMoveWidget(wg[i], x - Global.btn_height - 2, y);
+	XMapWindow(XtDisplay(wg[i]), XtWindow(wg[i]));
+      }
+      oldw = width;
+      oldh = height;
+      wg = &l->wplus;
+      for (i=0; i<6; i++) {
+         XtVaGetValues(wg[i], XtNx, &x, NULL);      
 #ifdef XAW3D       
-       XtMoveWidget(wg[i], x, PAGEHEIGHT+EXTRAHEIGHT+49);
+         XtMoveWidget(wg[i], x, PAGEHEIGHT+EXTRAHEIGHT+Global.btn_height+28);
 #else       
-       XtMoveWidget(wg[i], x, PAGEHEIGHT+EXTRAHEIGHT+42);
-#endif       
+         XtMoveWidget(wg[i], x, PAGEHEIGHT+EXTRAHEIGHT+Global.btn_height+25);
+#endif
+	 XMapWindow(XtDisplay(wg[i]), XtWindow(wg[i]));	 
+      }
+      wg = &l->compresslabel;
+      XtVaGetValues(wg[3], XtNx, &x, XtNy, &y, NULL);
+      n = Global.btn_height * 4 / 5 + 8;
+      n = width - n - x;
+      for (i=3; i>=0; i--) {      
+        if (i==1) n = n/2;
+        XtVaGetValues(wg[i], XtNx, &x, XtNy, &y, NULL);
+        XtMoveWidget(wg[i], x + n, y);
+	XMapWindow(XtDisplay(wg[i]), XtWindow(wg[i]));	
+      }
+      drawArrows(l->shell, l, NULL, False);
+      findDrawSize(l);
+      showDrawing(l);
+    } else {
+      i = 4 * Global.btn_height / 5 + 4;
+      x = 5 * i + 25;
+      y = PAGEHEIGHT+Global.btn_height+8;
+      XClearArea(XtDisplay(l->form), XtWindow(l->form),
+	         x, y,  PAGEWIDTH+EXTRAWIDTH, y+2*Global.btn_height+10, True);
+      x = PAGEWIDTH+EXTRAWIDTH + 4;
+      y = PAGEHEIGHT+EXTRAHEIGHT+Global.btn_height+ 18;
+      n = Global.btn_height+ 5*i + 39;
+      XClearArea(XtDisplay(l->form), XtWindow(l->form),
+	         x, n, x+18, y, True);
+      dx = width - oldw;
+      dy = height - oldh;
+      PAGEWIDTH += dx;
+      i = 0;
+      n = 200;
+      if (PAGEWIDTH<n) {
+	i = n - PAGEWIDTH;
+	dx += i;
+	width += i;
+	PAGEWIDTH = n;
+      }
+      PAGEHEIGHT += dy;
+      /* n = Global.btn_height * 297 / 21; */
+      n = 283;
+      if (PAGEHEIGHT<n) {
+	i = n - PAGEHEIGHT;
+	dy += i;
+	height += i;
+	PAGEHEIGHT = n;
+      }
+      if (i) {
+        XResizeWindow(XtDisplay(l->shell), XtWindow(l->shell), width, height);
+        XResizeWindow(XtDisplay(l->form), XtWindow(l->form), width, height);	
+      }
+      oldw = width;
+      oldh = height;
+      XResizeWindow(XtDisplay(l->form), XtWindow(l->form), width, height);
+      XtResizeWidget(l->vp, PAGEWIDTH+EXTRAWIDTH, PAGEHEIGHT+EXTRAHEIGHT, 0);
+      XtResizeWidget(l->box, PAGEWIDTH+EXTRAWIDTH, PAGEHEIGHT+EXTRAHEIGHT, 0);
+      XtResizeWidget(l->page, PAGEWIDTH, PAGEHEIGHT, 1);
+      wg = &l->hplus;
+      n = &l->printermenu - wg;
+      if (dx) for (i=0; i<n; i++) {      
+        XtVaGetValues(wg[i], XtNx, &x, XtNy, &y, NULL);
+        XtMoveWidget(wg[i], x + dx, y);
+      }
+      XtVaGetValues(l->equal, XtNx, &x, XtNy, &y, NULL);
+      XtMoveWidget(l->equal, x + dx, y + dy);
+      wg = &l->wplus;
+      if (dy) for (i=0; i<5; i++) {      
+        XtVaGetValues(wg[i], XtNx, &x, XtNy, &y, NULL);
+        XtMoveWidget(wg[i], x, y + dy);
+      }      
+      wg = &l->printerlist;
+      for (i=0; i<4; i++) if (wg[i]) {
+	XMapWindow(XtDisplay(wg[i]), XtWindow(wg[i]));
+      }
+      findDrawSize(l);
+      showDrawing(l);
     }
-
-    wg = &l->hplus;
-    n = &l->printresult - wg;
-    for (i=0; i<=n; i++) if (wg[i]) {
-       XtVaGetValues(wg[i], XtNx, &x, XtNy, &y, NULL);
-       XtMoveWidget(wg[i], x+PAGEWIDTH-oldw, y);
-    }
-
-    drawArrows(l->shell, l, NULL, False);
-    findDrawSize(l);
-    showDrawing(l);
 }
 
+static void 
+CheckNotify(Widget w, XEvent * event, String * params, Cardinal * nparams)
+{
+    Position pos;
+    XtVaGetValues(w, XtNy, &pos, NULL);
+    XtCallActionProc(w, "notify", event, params, 0);
+    if (event->xbutton.y_root > pos) XtPopdown(w);
+} 
+
 void 
 PrintPopup(Widget w, XtPointer paintArg)
 {
     static Cursor pageCursor = None;
     static XGCValues gcv;
-    static XtTranslations paint_trans = None;
+    static XtTranslations paint_translations = None;
+    static XtTranslations button_translations = None;
+    static String buttonTranslations =
+    "#override\n\
+<BtnDown>: check-notify()";
 
     Display *dpy;
     Window root;
@@ -1037,7 +1129,7 @@ PrintPopup(Widget w, XtPointer paintArg)
     char wname[8];
     XColor scol, ecol;
     Pixel white, black, defpix;
-    int i;
+    int a, b, aw, bw, i;
     Position x, y;
     Arg args[6];
     int nargs = 0, new = 0;
@@ -1083,6 +1175,8 @@ PrintPopup(Widget w, XtPointer paintArg)
     XtVaGetValues((Widget)printinfo->paintwidget, XtNdrawWidth, 
                   &printinfo->width, XtNdrawHeight, &printinfo->height, NULL);
 
+    PAGEWIDTH = 10 * Global.btn_height;
+    PAGEHEIGHT = Global.btn_height * 297 / 21;
     printinfo->shell = XtVisCreatePopupShell("print", topLevelShellWidgetClass,
 				  GetShell(w), args, nargs);
     white = WhitePixelOfScreen(XtScreen(printinfo->shell));
@@ -1110,7 +1204,7 @@ PrintPopup(Widget w, XtPointer paintArg)
 
     printinfo->format = XtVaCreateManagedWidget("formattype",
 				       asciiTextWidgetClass, printinfo->form,
-				       XtNtranslations, translations,
+				       XtNtranslations, text_translations,
  				       XtNeditType, XawtextEdit,
 				       XtNwrap, XawtextWrapNever,
 				       XtNresize, XawtextResizeWidth,
@@ -1119,14 +1213,20 @@ PrintPopup(Widget w, XtPointer paintArg)
 				       XtNwidth, 100,
 				       NULL);
 
+    XtAddEventHandler(printinfo->format, ButtonPressMask | ButtonReleaseMask, 
+                                  False, (XtEventHandler) buttonPressed,
+                                  (XtPointer) printinfo->form);
+
     printinfo->print = XtVaCreateManagedWidget("print",
 				       commandWidgetClass, printinfo->form,
+				       XtNheight, Global.btn_height,
 				       XtNfromHoriz, printinfo->format,
 				       XtNvertDistance, 8,
 				       NULL);
 
     printinfo->preview = XtVaCreateManagedWidget("preview",
 					 commandWidgetClass, printinfo->form,
+				         XtNheight, Global.btn_height,
 				         XtNfromHoriz, printinfo->print,
 				         XtNhorizDistance, 9,
 				         XtNvertDistance, 8,
@@ -1134,6 +1234,7 @@ PrintPopup(Widget w, XtPointer paintArg)
 
     printinfo->cancel = XtVaCreateManagedWidget("cancel",
 					   commandWidgetClass, printinfo->form,
+					   XtNheight, Global.btn_height,	
 				           XtNfromHoriz, printinfo->preview,
 				           XtNhorizDistance, 9,
 				           XtNvertDistance, 8,
@@ -1155,6 +1256,8 @@ PrintPopup(Widget w, XtPointer paintArg)
     printinfo->box = XtVaCreateManagedWidget("printBox", boxWidgetClass, 
                             printinfo->vp,
 			    XtNbackgroundPixmap, GetBackgroundPixmap(printinfo->vp),
+			    XtNwidth, PAGEWIDTH+EXTRAWIDTH,
+			    XtNheight, PAGEHEIGHT+EXTRAHEIGHT,		     
 			    NULL);
 
     printinfo->page = XtVaCreateManagedWidget("page", paintWidgetClass, 
@@ -1165,49 +1268,53 @@ PrintPopup(Widget w, XtPointer paintArg)
 				   XtNbackground, white,
 			           XtNwidth, PAGEWIDTH, XtNheight, PAGEHEIGHT,
 				   NULL);
-    if (paint_trans == None) {
-	paint_trans = XtParseTranslationTable(
-	   "#override\n\t<BtnDown>,<BtnUp>,<BtnMotion>: \n");
+    if (paint_translations == None) {
+	paint_translations = XtParseTranslationTable(
+		   "#override\n\t<BtnDown>,<BtnUp>,<BtnMotion>: \n");
     }
-    XtVaSetValues(printinfo->page, XtNtranslations, paint_trans, NULL);
+    XtVaSetValues(printinfo->page, XtNtranslations, paint_translations, NULL);
     XtVaSetValues(printinfo->page, XtNmenuwidgets, NULL, NULL);
 
     if (pageCursor == None)
         pageCursor = XCreateFontCursor(dpy, XC_crosshair);
 
 
+    a = Global.btn_height * 4 / 5 + 4;
+    aw = 9 * Global.btn_height;
+    bw = 2 * Global.btn_height;
+
     printinfo->wpplus = XtVaCreateManagedWidget("wpplus",
 				       commandWidgetClass, printinfo->form,
 				       XtNfromVert, printinfo->vp,
-				       XtNwidth, 18,
+				       XtNwidth, a, XtNheight, a,
 				       NULL);
 
     printinfo->wplus = XtVaCreateManagedWidget("wplus",
 				       commandWidgetClass, printinfo->form,
 				       XtNfromVert, printinfo->vp,
 				       XtNfromHoriz, printinfo->wpplus,
-				       XtNwidth, 18,
+				       XtNwidth, a, XtNheight, a,
 				       NULL);
 
     printinfo->wminus = XtVaCreateManagedWidget("wminus",
 				       commandWidgetClass, printinfo->form,
 				       XtNfromVert, printinfo->vp,
 				       XtNfromHoriz, printinfo->wplus,
-				       XtNwidth, 18,
+				       XtNwidth, a, XtNheight, a,
 				       NULL);
 
     printinfo->wmminus = XtVaCreateManagedWidget("wmminus",
 				       commandWidgetClass, printinfo->form,
 				       XtNfromVert, printinfo->vp,
 				       XtNfromHoriz, printinfo->wminus,
-				       XtNwidth, 18,
+				       XtNwidth, a, XtNheight, a,
 				       NULL);
 
     printinfo->wcenter = XtVaCreateManagedWidget("wcenter",
 				       commandWidgetClass, printinfo->form,
 				       XtNfromVert, printinfo->vp,
 				       XtNfromHoriz, printinfo->wmminus,
-				       XtNwidth, 18,
+				       XtNwidth, a, XtNheight, a,
 				       NULL);
 
     printinfo->hpplus = XtVaCreateManagedWidget("hpplus",
@@ -1215,35 +1322,35 @@ PrintPopup(Widget w, XtPointer paintArg)
 				       XtNfromHoriz, printinfo->vp,
 				       XtNfromVert, printinfo->bar,
 				       XtNvertDistance, 6,
-				       XtNwidth, 18,
+				       XtNwidth, a, XtNheight, a,
 				       NULL);
 
     printinfo->hplus = XtVaCreateManagedWidget("hplus",
 				       commandWidgetClass, printinfo->form,
 				       XtNfromHoriz, printinfo->vp,
 				       XtNfromVert, printinfo->hpplus,
-				       XtNwidth, 18,
+				       XtNwidth, a, XtNheight, a,
 				       NULL);
 
     printinfo->hminus = XtVaCreateManagedWidget("hminus",
 				       commandWidgetClass, printinfo->form,
 				       XtNfromHoriz, printinfo->vp,
 				       XtNfromVert, printinfo->hplus,
-				       XtNwidth, 18,
+				       XtNwidth, a, XtNheight, a,
 				       NULL);
 
     printinfo->hmminus = XtVaCreateManagedWidget("hmminus",
 				       commandWidgetClass, printinfo->form,
 				       XtNfromHoriz, printinfo->vp,
 				       XtNfromVert, printinfo->hminus,
-				       XtNwidth, 18,
+				       XtNwidth, a, XtNheight, a,
 				       NULL);
 
     printinfo->hcenter = XtVaCreateManagedWidget("hcenter",
 				       commandWidgetClass, printinfo->form,
 				       XtNfromHoriz, printinfo->vp,
 				       XtNfromVert, printinfo->hmminus,
-				       XtNwidth, 18,
+				       XtNwidth, a, XtNheight, a,
 				       NULL);
 
     printinfo->equal = XtVaCreateManagedWidget("equal",
@@ -1251,25 +1358,28 @@ PrintPopup(Widget w, XtPointer paintArg)
 				       XtNfromVert, printinfo->vp,
 				       XtNfromHoriz, printinfo->vp,
 				       XtNstate, printinfo->tied,
-				       XtNwidth, 18,
+				       XtNvertDistance, 12,
+				       XtNwidth, a, XtNheight, a,
 				       NULL);
 
-    if (translations == None) {
+    if (text_translations == None) {
 	static XtActionsRec act =
 	{"print-text-ok", (XtActionProc) textAction};
 
 	XtAppAddActions(XtWidgetToApplicationContext(w), &act, 1);
 
-	translations = XtParseTranslationTable(textTranslations);
+	text_translations = XtParseTranslationTable(textTranslations);
     }
 
+    b = 2 * (Global.btn_height - 19);
+
     printinfo->portraitlabel = XtVaCreateManagedWidget("portraitlabel",
 				       labelWidgetClass, printinfo->form,
 				       XtNborderWidth, 0,
 				       XtNfromHoriz, printinfo->hpplus,
 				       XtNfromVert, printinfo->bar,
 				       XtNhorizDistance, 30,
-				       XtNvertDistance, 6,
+				       XtNvertDistance, b,
 				       NULL);
 
     printinfo->portraittoggle = XtVaCreateManagedWidget("portraittoggle",
@@ -1278,8 +1388,8 @@ PrintPopup(Widget w, XtPointer paintArg)
 				       XtNfromHoriz, printinfo->portraitlabel,
 				       XtNfromVert, printinfo->bar,
 				       XtNhorizDistance, 12,
-				       XtNvertDistance, 6,
-				       XtNwidth, 18,
+				       XtNvertDistance, b,
+				       XtNwidth, a, XtNheight, a,
 				       NULL);
 
     printinfo->landscapelabel = XtVaCreateManagedWidget("landscapelabel",
@@ -1287,18 +1397,19 @@ PrintPopup(Widget w, XtPointer paintArg)
 				       XtNborderWidth, 0,
 				       XtNfromHoriz, printinfo->portraittoggle,
 				       XtNfromVert, printinfo->bar,
-				       XtNvertDistance, 6,
+				       XtNhorizDistance, 4*Global.btn_height,
+				       XtNvertDistance, b,
 				       NULL);
 
     printinfo->landscapetoggle = XtVaCreateManagedWidget("landscapetoggle",
 				       toggleWidgetClass, printinfo->form,
                                        XtNstate, printinfo->orient,
-				       XtNfromHoriz, printinfo->hpplus,
+				       XtNfromHoriz, printinfo->landscapelabel,
                                        XtNradioGroup,printinfo->portraittoggle,
 				       XtNfromVert, printinfo->bar,
-				       XtNhorizDistance, 324,
-				       XtNvertDistance, 6,
-				       XtNwidth, 18,
+				       XtNhorizDistance, 6,
+				       XtNvertDistance, b,
+				       XtNwidth, a, XtNheight, a,
 				       NULL);
 
     printinfo->graylabel = XtVaCreateManagedWidget("graylabel",
@@ -1315,9 +1426,9 @@ PrintPopup(Widget w, XtPointer paintArg)
                                        XtNstate, 0,
 				       XtNfromHoriz, printinfo->graylabel,
 				       XtNfromVert, printinfo->portraittoggle,
-				       XtNhorizDistance, 12,
+				       XtNhorizDistance, 6,
 				       XtNvertDistance, 6,
-				       XtNwidth, 18,
+				       XtNwidth, a, XtNheight, a,
 				       NULL);
 
     printinfo->compresslabel = XtVaCreateManagedWidget("compresslabel",
@@ -1325,7 +1436,7 @@ PrintPopup(Widget w, XtPointer paintArg)
 				       XtNborderWidth, 0,
 				       XtNfromHoriz, printinfo->graytoggle,
 				       XtNfromVert, printinfo->portraittoggle,
-				       XtNhorizDistance, 16,
+				       XtNhorizDistance, Global.btn_height,
 				       XtNvertDistance, 6,
 				       NULL);
 
@@ -1337,26 +1448,26 @@ PrintPopup(Widget w, XtPointer paintArg)
 				       XtNhorizDistance, 12,
 				       XtNvertDistance, 6,
 				       XtNstate, True,
-				       XtNwidth, 18,
+				       XtNwidth, a, XtNheight, a,
 				       NULL);
 
     printinfo->pdflabel = XtVaCreateManagedWidget("pdflabel",
 				       labelWidgetClass, printinfo->form,
 				       XtNborderWidth, 0,
-				       XtNfromHoriz, printinfo->hpplus,
+				       XtNfromHoriz, printinfo->compresstoggle,
 				       XtNfromVert, printinfo->portraittoggle,
+				       XtNhorizDistance, 4*Global.btn_height,
 				       XtNvertDistance, 6,
-				       XtNhorizDistance, 280,
 				       NULL);
 
     printinfo->pdftoggle = XtVaCreateManagedWidget("pdftoggle",
 				       toggleWidgetClass, printinfo->form,
                                        XtNstate, 0,
-				       XtNfromHoriz, printinfo->hpplus,
+				       XtNfromHoriz, printinfo->pdflabel,
 				       XtNfromVert, printinfo->portraittoggle,
-				       XtNhorizDistance, 324,
+				       XtNhorizDistance, 6,
 				       XtNvertDistance, 6,
-				       XtNwidth, 18,
+				       XtNwidth, a, XtNheight, a,
 				       NULL);
 
     printinfo->sizelabel = XtVaCreateManagedWidget("sizelabel",
@@ -1370,15 +1481,19 @@ PrintPopup(Widget w, XtPointer paintArg)
 
     printinfo->size = XtVaCreateManagedWidget("sizevalue",
 				       asciiTextWidgetClass, printinfo->form,
-				       XtNtranslations, translations,
+				       XtNtranslations, text_translations,
  				       XtNeditType, XawtextEdit,
 				       XtNwrap, XawtextWrapNever,
 				       XtNresize, XawtextResizeWidth,
-				       XtNfromHoriz, printinfo->hpplus,
+				       XtNfromHoriz, printinfo->sizelabel,
 				       XtNfromVert, printinfo->graytoggle,
-				       XtNwidth, 150,
+				       XtNwidth, aw,
+				       XtNhorizDistance, Global.btn_height+8,
 				       XtNvertDistance, 12,
 				       NULL);
+    XtAddEventHandler(printinfo->size, ButtonPressMask | ButtonReleaseMask, 
+                                  False, (XtEventHandler) buttonPressed,
+                                  (XtPointer) printinfo->form);
 
     printinfo->positionlabel = XtVaCreateManagedWidget("positionlabel",
 				       labelWidgetClass, printinfo->form,
@@ -1390,14 +1505,18 @@ PrintPopup(Widget w, XtPointer paintArg)
 
     printinfo->position = XtVaCreateManagedWidget("positionvalue",
 				       asciiTextWidgetClass, printinfo->form,
-				       XtNtranslations, translations,
+				       XtNtranslations, text_translations,
  				       XtNeditType, XawtextEdit,
 				       XtNwrap, XawtextWrapNever,
 				       XtNresize, XawtextResizeWidth,
-				       XtNfromHoriz, printinfo->hpplus,
+				       XtNfromHoriz, printinfo->positionlabel,
+				       XtNhorizDistance, Global.btn_height+8,
 				       XtNfromVert, printinfo->size,
-				       XtNwidth, 150,
+				       XtNwidth, aw,
 				       NULL);
+    XtAddEventHandler(printinfo->position, ButtonPressMask | ButtonReleaseMask, 
+                                  False, (XtEventHandler) buttonPressed,
+                                  (XtPointer) printinfo->form);
 
     printinfo->samplinglabel = XtVaCreateManagedWidget("samplinglabel",
 				       labelWidgetClass, printinfo->form,
@@ -1409,14 +1528,18 @@ PrintPopup(Widget w, XtPointer paintArg)
 
     printinfo->sampling = XtVaCreateManagedWidget("samplingvalue",
 				       asciiTextWidgetClass, printinfo->form,
-				       XtNtranslations, translations,
+				       XtNtranslations, text_translations,
  				       XtNeditType, XawtextEdit,
 				       XtNwrap, XawtextWrapNever,
 				       XtNresize, XawtextResizeWidth,
 				       XtNfromVert, printinfo->position,
-				       XtNfromHoriz, printinfo->hpplus,
-				       XtNwidth, 150,
+				       XtNfromHoriz, printinfo->samplinglabel,
+				       XtNhorizDistance, Global.btn_height+8,
+				       XtNwidth, aw,
 				       NULL);
+    XtAddEventHandler(printinfo->sampling, ButtonPressMask | ButtonReleaseMask, 
+                                  False, (XtEventHandler) buttonPressed,
+                                  (XtPointer) printinfo->form);
 
     printinfo->printlabel = XtVaCreateManagedWidget("printlabel",
 				       labelWidgetClass, printinfo->form,
@@ -1429,15 +1552,20 @@ PrintPopup(Widget w, XtPointer paintArg)
 
     printinfo->printcmd = XtVaCreateManagedWidget("printcmd",
 				       asciiTextWidgetClass, printinfo->form,
-				       XtNtranslations, translations,
+				       XtNtranslations, text_translations,
  				       XtNeditType, XawtextEdit,
 				       XtNwrap, XawtextWrapNever,
 				       XtNscrollHorizontal, True,
 				       XtNfromVert, printinfo->sampling,
-				       XtNfromHoriz, printinfo->hpplus,
+				       XtNfromHoriz, printinfo->printlabel,
+				       XtNhorizDistance, Global.btn_height+8,
 				       XtNvertDistance, 14,
-				       XtNwidth, 150, XtNheight, 40,
+				       XtNwidth, aw, XtNheight, bw,
 				       NULL);
+    XtAddEventHandler(printinfo->printcmd, ButtonPressMask | ButtonReleaseMask, 
+                                  False, (XtEventHandler) buttonPressed,
+                                  (XtPointer) printinfo->form);
+
     if (printcmd_str) 
        XtVaSetValues(printinfo->printcmd, XtNstring, printcmd_str, NULL);
 
@@ -1457,19 +1585,32 @@ PrintPopup(Widget w, XtPointer paintArg)
 				       XtNmenuName, "printermenu",
 				       XtNborderWidth, 1,
 				       XtNlabel, "?",
-				       XtNwidth, 18,
-				       XtNheight, 16,
+				       XtNwidth, a, XtNheight, a,
 				       XtNfromVert, printinfo->sampling,
-				       XtNfromHoriz, printinfo->printcmd,
+				       XtNfromHoriz, printinfo->printlabel,
 				       XtNvertDistance, 15,
 #ifdef XAW3D
                                        XtNborderWidth, 0,							    
 #endif							    
 				       NULL);
+
           printinfo->printermenu = XtVaCreatePopupShell("printermenu",
 				       simpleMenuWidgetClass, 
                                        printinfo->printerlist,
 				       NULL);
+
+          if (button_translations == None) {
+              static XtActionsRec act[1] =
+	      {"check-notify",  (XtActionProc) CheckNotify};
+
+              XtAppAddActions(
+                  XtWidgetToApplicationContext(printinfo->printermenu),
+                  act, 1);
+	      button_translations = XtParseTranslationTable(buttonTranslations);
+              XtOverrideTranslations(printinfo->printermenu, 
+                                     button_translations);
+          }
+
           for (i = 0; i<=num_printers; i++) {
 	        sprintf(wname, "%d", i);
 	        printinfo->printentry = 
@@ -1486,11 +1627,14 @@ PrintPopup(Widget w, XtPointer paintArg)
     printinfo->printtoggle = XtVaCreateManagedWidget("printtoggle",
 				       toggleWidgetClass, printinfo->form,
                                        XtNstate, !printinfo->output,
-				       XtNfromVert, printinfo->sampling,
-				       XtNfromHoriz, printinfo->printcmd,
-				       XtNvertDistance, 
-					  (printinfo->printerlist==0)? 28: 37,
-				       XtNwidth, 18,
+				       XtNfromVert,
+					(printinfo->printerlist)?
+					 printinfo->printerlist:
+					 printinfo->sampling,
+				       XtNfromHoriz, printinfo->printlabel,
+				       XtNhorizDistance, 4,
+				       XtNvertDistance, 4,
+				       XtNwidth, a, XtNheight, a,
 				       NULL);
    
     printinfo->filelabel = XtVaCreateManagedWidget("filelabel",
@@ -1503,14 +1647,20 @@ PrintPopup(Widget w, XtPointer paintArg)
 
     printinfo->filename = XtVaCreateManagedWidget("filename",
 				       asciiTextWidgetClass, printinfo->form,
-				       XtNtranslations, translations,
+				       XtNtranslations, text_translations,
  				       XtNeditType, XawtextEdit,
 				       XtNwrap, XawtextWrapNever,
 				       XtNscrollHorizontal, True,
 				       XtNfromVert, printinfo->printcmd,
-				       XtNfromHoriz, printinfo->hpplus,
-				       XtNwidth, 150, XtNheight, 40,
+				       XtNfromHoriz, printinfo->filelabel,
+				       XtNhorizDistance, Global.btn_height+8,
+				       XtNvertDistance, 6,
+				       XtNwidth, aw, XtNheight, bw,
 				       NULL);
+    XtAddEventHandler(printinfo->filename, ButtonPressMask | ButtonReleaseMask, 
+                                  False, (XtEventHandler) buttonPressed,
+                                  (XtPointer) printinfo->form);
+
     if (canvasfile) {
         XtVaSetValues(printinfo->filename, XtNstring, canvasfile, NULL);
 	free(canvasfile);
@@ -1519,21 +1669,20 @@ PrintPopup(Widget w, XtPointer paintArg)
     printinfo->fileselect = XtVaCreateManagedWidget("fileselect",
 				       commandWidgetClass, printinfo->form,
 				       XtNlabel, "?",
-				       XtNwidth, 18,
-				       XtNheight, 16,
+				       XtNwidth, a, XtNheight, a,
 				       XtNfromVert, printinfo->printcmd,
-				       XtNfromHoriz, printinfo->filename,
-                                       XtNvertDistance, 5,
+				       XtNfromHoriz, printinfo->filelabel,
+                                       XtNvertDistance, 6,
 				       NULL);
 
     printinfo->filetoggle = XtVaCreateManagedWidget("filetoggle",
 				       toggleWidgetClass, printinfo->form,
                                        XtNstate, printinfo->output,
 				       XtNfromVert, printinfo->fileselect,
-				       XtNfromHoriz, printinfo->filename,
+				       XtNfromHoriz, printinfo->filelabel,
 				       XtNradioGroup, printinfo->printtoggle,
-                                       XtNvertDistance, 5,
-				       XtNwidth, 18,
+                                       XtNvertDistance, 4,
+				       XtNwidth, a, XtNheight, a,
 				       NULL);
 
     printinfo->psviewlabel = XtVaCreateManagedWidget("psviewlabel",
@@ -1546,35 +1695,44 @@ PrintPopup(Widget w, XtPointer paintArg)
 
     printinfo->psviewcmd = XtVaCreateManagedWidget("psviewcmd",
 				       asciiTextWidgetClass, printinfo->form,
-				       XtNtranslations, translations,
+				       XtNtranslations, text_translations,
  				       XtNeditType, XawtextEdit,
 				       XtNwrap, XawtextWrapNever,
 				       XtNscrollHorizontal, True,
 				       XtNfromVert, printinfo->filename,
-				       XtNfromHoriz, printinfo->hpplus,
-				       XtNwidth, 150, XtNheight, 40,
+				       XtNfromHoriz, printinfo->psviewlabel,
+				       XtNhorizDistance, Global.btn_height+8,
+				       XtNvertDistance, 6,
+				       XtNwidth, aw, XtNheight, bw,
 				       NULL);
-   
+    XtAddEventHandler(printinfo->psviewcmd, ButtonPressMask| ButtonReleaseMask, 
+                                  False, (XtEventHandler) buttonPressed,
+                                  (XtPointer) printinfo->form);
+
     printinfo->resultlabel = XtVaCreateManagedWidget("resultlabel",
 				       labelWidgetClass, printinfo->form,
 				       XtNborderWidth, 0,
 				       XtNfromVert, printinfo->psviewcmd,
 				       XtNfromHoriz, printinfo->hpplus,
-				       XtNvertDistance, 14,
+				       XtNvertDistance, 6,
 				       XtNhorizDistance, 30,
 				       NULL);
 
     printinfo->printresult = XtVaCreateManagedWidget("printresult",
 				       asciiTextWidgetClass, printinfo->form,
-				       XtNtranslations, translations,
+				       XtNtranslations, text_translations,
  				       XtNeditType, XawtextEdit,
 				       XtNwrap, XawtextWrapNever,
 				       XtNscrollHorizontal, True,
 				       XtNfromVert, printinfo->psviewcmd,
 				       XtNfromHoriz, printinfo->resultlabel,
-				       XtNvertDistance, 14,
-				       XtNheight, 40,
+				       XtNhorizDistance, Global.btn_height+8,
+				       XtNvertDistance, 6,
+				       XtNwidth, aw, XtNheight, bw,
 				       NULL);
+    XtAddEventHandler(printinfo->printresult, ButtonPressMask|ButtonReleaseMask,
+                                  False, (XtEventHandler) buttonPressed,
+                                  (XtPointer) printinfo->form);
 
     if (psviewcmd_str) 
        XtVaSetValues(printinfo->psviewcmd, XtNstring, psviewcmd_str, NULL);
@@ -1650,6 +1808,7 @@ PrintPopup(Widget w, XtPointer paintArg)
     AddDestroyCallback(printinfo->shell, (DestroyCallbackFunc) closePrintCallback, printinfo);
 
     XtPopup(printinfo->shell, XtGrabNone);
+    SetWMInputHint(dpy, XtWindow(printinfo->shell));
     XtUnmanageChild(printinfo->form);
     XMapWindow(XtDisplay(printinfo->form), XtWindow(printinfo->form));
 
@@ -1682,8 +1841,9 @@ PrintPopup(Widget w, XtPointer paintArg)
 	printinfo->y = y;
 	setPercent(printinfo);
     }
-
+    
     XDefineCursor(dpy, XtWindow(printinfo->page), pageCursor);
+    XFlush(XtDisplay(printinfo->form));
 }
 
 static void 
@@ -1793,7 +1953,7 @@ viewExternCallback(Widget w, XtPointer infoArg, XtPointer junk2)
     }
     StateSetBusy(False);
 
-    system(cmd);
+    if (system(cmd));
     free(file);
 }
 
@@ -1809,14 +1969,14 @@ externpopupResized(Widget w, ExternInfo * l, XConfigureEvent * event,
     XtVaGetValues(l->extview, XtNx, &x, NULL);
     width1 = width - x - 8;
     if (width1 < 100) width1 = 100;
-    height1 = height - 58;
+    height1 = height - 2 * Global.btn_height - 16;
     if (height1 < 40) height1 = 40;
-    XtMoveWidget(l->extview, x, 30);
+    XtMoveWidget(l->extview, x, Global.btn_height + 8);
     XtResizeWidget(l->extview, width1, height1, 0);
     XtVaGetValues(l->viewbut, XtNx, &x, NULL);
-    XtMoveWidget(l->viewbut, x, height-25);
+    XtMoveWidget(l->viewbut, x, height - Global.btn_height - 3);
     XtVaGetValues(l->cancel, XtNx, &x, NULL);
-    XtMoveWidget(l->cancel, x, height-25);
+    XtMoveWidget(l->cancel, x, height - Global.btn_height - 3);
 }
 
 void 
@@ -1869,48 +2029,53 @@ ExternPopup(Widget w, XtPointer paintArg)
     externinfo->formatlabel = XtVaCreateManagedWidget("formatlabel",
 				       labelWidgetClass, externinfo->form,
 				       XtNfromHoriz, externinfo->bar,
+				       XtNheight, Global.btn_height,
 				       XtNborderWidth, 0,
 				       NULL);
 
     externinfo->extviewlabel = XtVaCreateManagedWidget("extviewlabel",
 				       labelWidgetClass, externinfo->form,
 				       XtNfromVert, externinfo->bar,
+				       XtNheight, Global.btn_height,
 				       XtNvertDistance, 10,
 				       XtNborderWidth, 0,
 				       NULL);
 
-    if (translations == None) {
+    if (text_translations == None) {
 	static XtActionsRec act =
 	{"print-text-ok", (XtActionProc) textAction};
 
 	XtAppAddActions(XtWidgetToApplicationContext(w), &act, 1);
 
-	translations = XtParseTranslationTable(textTranslations);
+	text_translations = XtParseTranslationTable(textTranslations);
     }
 
     externinfo->extview = XtVaCreateManagedWidget("extviewcmd",
 				       asciiTextWidgetClass, externinfo->form,
-				       XtNtranslations, translations,
+				       XtNtranslations, text_translations,
  				       XtNeditType, XawtextEdit,
 				       XtNwrap, XawtextWrapNever,
 				       XtNscrollHorizontal, True,
-				       XtNwidth, 150, XtNheight, 40,
+				       XtNwidth, 450, XtNheight, 100,
 				       XtNfromVert, externinfo->bar,
 				       XtNvertDistance, 10,
 				       XtNfromHoriz, externinfo->extviewlabel,
 				       NULL);
+
     if (externinfo->cmd)
         XtVaSetValues(externinfo->extview, XtNstring, externinfo->cmd, NULL);
 
     externinfo->viewbut = XtVaCreateManagedWidget("view",
 				       commandWidgetClass, externinfo->form,
 				       XtNfromVert, externinfo->extview,
+				       XtNheight, Global.btn_height,
 				       NULL);
 
     externinfo->cancel = XtVaCreateManagedWidget("cancel",
 				       commandWidgetClass, externinfo->form,
 				       XtNfromVert, externinfo->extview,
 				       XtNfromHoriz, externinfo->viewbut,
+				       XtNheight, Global.btn_height,
 				       NULL);
 
     for (i=0; i<XtNumber(externMenu); i++) {
@@ -1948,6 +2113,8 @@ ExternPopup(Widget w, XtPointer paintArg)
                   externinfo);
 
     XtPopup(externinfo->shell, XtGrabNone);
+    XtSetKeyboardFocus(externinfo->form, externinfo->extview);
+    SetWMInputHint(dpy, XtWindow(externinfo->shell));
     XtUnmanageChild(externinfo->form);
     XMapWindow(XtDisplay(externinfo->form), XtWindow(externinfo->form));
 
